/*
 * AgentMsg.java
 *
 * Package Version: 4.0
 *   Class Version: 1.0
 * 
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 13 February 2008
 * 
 * This class defines a DEVSJAVA message between the interaction model (IM) and
 * an agent. A single message is sent between the agents and IM for each data
 * type. The message header is the agent's type (e.g., village or household), the
 * agent's integer ID, the location ID (the village number that the household is
 * in or -1 if it is a village), and the message type (e.g., examine, cultivate,
 * etc.). Any type of data object may be sent and it is up to the IM and agent(s)
 * involved to ensure proper object conversion/management.
 * 
 * Class Fixes / Additions:
 * 1.0:
 *  - None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import java.util.*;
import GenCol.*;

public class AgentMsg extends entity
{
	public final TypeAgents agentType;
	public final int agentID;
	public final int locationID;
	public final Messages msgType;
	private ArrayList Data;
	private Iterator currElement;

	/**
	 * AgentMsg - constructor
	 * @param agent_type - one of the TypeAgents objects
	 * @param agent_id - the agent's unique ID
	 * @param loc_id - the agent's location ID
	 * @param msg_type - one of the Messages objects (all data should be for the same message type)
	 */
	public AgentMsg ( TypeAgents agent_type, int agent_id, Messages msg_type, int loc_id )
	{
		super(msg_type.toString());
		
		this.agentType = agent_type;
		this.agentID = agent_id;
		this.locationID = loc_id;
		this.msgType = msg_type;
		
		this.Data = new ArrayList();
		this.currElement = null;
		
	}
	
	/**
	 * AgentMsg - contructor
	 * @param agent_type - one of the TypeAgents objects
	 * @param agent_id - the agent's unique ID
	 * @param msg_type - one of the Messages objects (all data should be for the same message type)
	 */
	public AgentMsg ( TypeAgents agent_type, int agent_id, Messages msg_type )
	{
		this(agent_type, agent_id, msg_type, -1);
		
	}	
	
	/**
	 * Add - adds data object to the message array
	 * @param data - an Object representing data to be sent with this message
	 */
	public void Add ( Object data )
	{
		Data.add((Object)data);
		
	}
	
	/**
	 * Add - adds data from an array list to the message array
	 * @param data - an ArrayList object containing data to be sent with this message
	 */
	public void Add ( ArrayList data )
	{
		Data.addAll(data);
		
	}
	
	/**
	 * Next - gets the next data Object from the data array list
	 * @return - Object (data element)
	 */
	public Object Next ()
	{
		Object data = null;
		
		if ( this.currElement == null )
			this.currElement = Data.iterator();
		
		if ( this.currElement.hasNext() )
			data = this.currElement.next();
		
		else
			this.currElement = null;
		
		return data;
		
	}
	
	/**
	 * ResetPtr - resets the data pointer to the start of the array list
	 * so that the next call to Next() returns the first element in the list
	 */
	public void ResetPtr ()
	{
		this.currElement = null;
		
	}
	
	/**
	 * IsEmpty - checks if the message data array has any elements
	 * @return - boolean (true if empty; false otherwise)
	 */
	public boolean IsEmpty ()
	{
		return Data.isEmpty();
		
	}
	
	/**
	 * Size - returns the number of data elements in the data array
	 * @return - int (number of elements)
	 */
	public int Size ()
	{
		return Data.size();
		
	}

}
