/*
 * MAgentCalendar.java
 *
 * Package Version: 4.0
 *   Class Version: 1.0
 * 
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 13 February 2008
 * 
 * This class defines a DEVSJAVA atomic model which maintains the time of year.
 * The agent model uses this information to make decisions about which actions
 * to take. For example, planting versus harvesting crop.
 * 
 * Class Fixes / Additions:
 * 1.0:
 *  - None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import GenCol.*;
import model.modeling.*;
import view.*;
import view.modeling.*;

//public class MAgentCalendar extends ViewableAtomic

public class MAgentCalendar extends ViewableAtomic
{
	private String season;
	
	public MAgentCalendar ()
	{
		this("Calendar", "spring");
		
	}
	
	public MAgentCalendar ( String name, String season )
	{
		super(" " + name + " ");
		
		this.season = season.toLowerCase();
		
		addInport("control");
		addOutport("calendar");
		
		addTestInput("control", new entity("advance"));
		
	}
	
	public void initialize ()
	{
		holdIn(this.season, 0.0);
		
	}
	
	public void deltext ( double e, message x )
	{
		entity msg = null;
		
		Continue(e);
		
		try
		{
			for (int i = 0; i < x.getLength(); i++)
			{
				if ( messageOnPort(x, "control", i) )
				{
					msg = x.getValOnPort("control", i);
					
					if ( msg.toString() == "advance" )
					{
						if ( this.season == "spring" )
							this.season = "fall";
						
						else if ( this.season == "fall" )
							this.season = "spring";
						
						else
							throw new IllegalArgumentException("Invalid season, " + this.season);
					}
				}
			}
		}
		
		catch (IllegalArgumentException iae)
		{
			System.err.print(iae.getMessage());
			
			this.season = "spring";
		}
		
		holdIn(this.season, 0.0);
		
	}
	
	public void deltint ()
	{
		if ( this.season == "spring" )
			this.season = "fall";
		
		else if ( this.season == "fall" )
			this.season = "spring";
		
		holdIn(this.season, 0.5);
		
	}
	
	public void deltcon ( double e, message x )
	{
		deltext(e,x);
		deltint();
		
	}
	
	public message out ()
	{
		message m = new message();
		AgentMsg msg = new AgentMsg(TypeAgents.HOUSEHOLD, -1, Messages.CALENDAR);
		msg.Add(this.season);
		
		m.add(makeContent("calendar", msg));
		
		return m;
		
	}
	
}


