/*
 * MSettlements.java
 * 
 * Package Version: 4.0
 *   Class Version: 1.0
 *  
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 05 May 2008
 * 
 * This file contains the MSettlements coupled model. It represents the
 * entire scenrio in which the human agent models act. This includes both
 * the agents themselves and, currently, the interface model that provides agent
 * access to the landscape model data upon which the agents make decisions.
 * 
 * Upon object construction, village coupled model is created.
 * 
 * Class Fixes / Additions:
 * 1.0:
 * 	- The class now accepts the data from the ModelInitialization.
 * 
 * Bugs / Issues:
 * 1.0:
 * 	- None.
 */

package PenaguilaALv4;

import java.awt.*;
import java.util.*;
import GenCol.Pair;
import view.modeling.*;


public class MSettlements extends ViewableDigraph
{
	private MInteractionModel im;
	private MVillageDemultiplexer vd;
	private MAgentCalendar ac;
	private MVillage vg;
	
	public MSettlements ()
	{
		// default constructor parameters are provided for testing purposes in simView.
		this("Settlements", 3, 3, 6, 250);
	}
	
	public MSettlements ( String name, int numVillages, int householdsPerVillage, int peoplePerHousehold, int hhMaxDistanceCost)
	{
		super(name);

		// Create landscape interface.
		im = new MInteractionModel ("Interactions", ModelInitialization.ip);
		im.setBackgroundColor(Color.cyan);
		add(im);
		
		// Create village demultiplexer to manage inter-village messages.
		vd = new MVillageDemultiplexer("VgMsgPasser");
		vd.setBackgroundColor(Color.YELLOW);
		add(vd);
		
		// Create agent calendar
		ac = new MAgentCalendar("Season", "spring");
		ac.setBackgroundColor(Color.green);
		add(ac);
		
		addCoupling(im, "agent_out", vd, "msgs");
		addCoupling(ac, "calendar", vd, "msgs");
		
		// Create villages and couplings with demultiplexer.
		// Note village location is using GRASS east_north format
		Pair[] villageLocation = null;
		
		if ( ModelInitialization.ip != null )
		{
			Iterator vlg = ModelInitialization.ip.VData.iterator();
			
			while ( vlg.hasNext() )
			{
				InitParameters.VillageData vdata = (InitParameters.VillageData) vlg.next();
				Integer ns = new Integer((int)vdata.vns);
				Integer ew = new Integer((int)vdata.vew);
								
				vg = new MVillage(vdata.vid, vdata.vname, new Pair(ew, ns),
						vdata.num_hh, vdata.pop_per_hh, ModelInitialization.ip.maxfarmcost,
						ModelInitialization.ip);
								
				SetVillage(vg);
//				vg.setBlackBox(true);
			}
		}
		
		else
		{
			villageLocation = new Pair[3];
			villageLocation[0] = new Pair(new Integer(592435), new Integer(4924535));
	//		villageLocation[1] = new Pair(new Integer(590255), new Integer(4924405));
			villageLocation[1] = new Pair(new Integer(591515), new Integer(4926685));
			villageLocation[2] = new Pair(new Integer(596685), new Integer(4921715));
		
			for ( int v = 0; v < numVillages; v++ )
			{
				vg = new MVillage(v, new String("Village_"+v), villageLocation[v],
						householdsPerVillage, peoplePerHousehold, hhMaxDistanceCost, null);

				SetVillage(vg);
			}
		}
		
		initialize();
		
	}
	
	private void SetVillage ( MVillage vg )
	{
		Integer vID = new Integer(vg.GetID());
		String new_port = "v_" + vID.toString();
		
		vd.AddVillage(vID, new_port);
		vd.setBackgroundColor(Color.magenta);
		vd.addOutport(new_port);
		vd.AssignHouseholds(vg.GetHouseholdMembers(), vID);
		addCoupling(vd, new_port, vg, "in");
		addCoupling(vg, "ext_msgs", vd, "msgs");
		addCoupling(vg, "land", im, "agent_in");
		add(vg);
	}

    
/*
 * ComputeLayout = false;
        preferredSize = new Dimension(775, 215);
        ((ViewableComponent)withName("Village_0")).setPreferredLocation(new Point(236, 17));
        ((ViewableComponent)withName("    Interactions    ")).setPreferredLocation(new Point(0, 25));
        ((ViewableComponent)withName(" Season ")).setPreferredLocation(new Point(29, 82));
        ((ViewableComponent)withName("VgMsgPasser")).setPreferredLocation(new Point(24, 145));
 */
    /**
     * Automatically generated by the SimView program.
     * Do not edit this manually, as such changes will get overwritten.
     */
    public void layoutForSimView()
    {
        preferredSize = new Dimension(1140, 408);
        ((ViewableComponent)withName(" Season ")).setPreferredLocation(new Point(5, 252));
        ((ViewableComponent)withName("Village_0")).setPreferredLocation(new Point(241, 23));
        ((ViewableComponent)withName("    Interactions    ")).setPreferredLocation(new Point(-8, 59));
        ((ViewableComponent)withName("VgMsgPasser")).setPreferredLocation(new Point(19, 155));
    }
}
