/*
 * Scripts.java
 *
 * Package Version: 3.0
 *   Class Version: 1.0
 *   
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 06 July 2007
 * 
 * This class provides an enumeration for all GRASS scripts that the interaction
 * model (IM) uses in the simulation. 
 * 
 * NOTE: This implementation is NOT remote method invocation (RMI) safe
 * 	nor will it work if different objects are created using multiple jar files.
 * 
 * Class Fixes / Additions:
 * 1.0:
 *  - None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

public class TypeScripts
{
	//////////////////////////////////
	//         SCRIPT TYPES         //
	//////////////////////////////////
	public static final TypeScripts SOIL_DYNAMICS = new TypeScripts("soild_dynamics_model.sh");		// executes erosion model against the current land cover
	public static final TypeScripts IMPACTS = new TypeScripts("agent_impacts.sh");					// current agent impacts to landscape
	public static final TypeScripts LANDCOVER_UPDATE = new TypeScripts("landcover_update.sh");		// land cover update script
	public static final TypeScripts LAND_MODEL_INIT = new TypeScripts("land_model_init.sh");		// land model initialization script
	public static final TypeScripts MAPSET = new TypeScripts("create_mapset.sh");					// GRASS mapset creation script
	public static final TypeScripts SITE_UPDATE = new TypeScripts("site_update.sh");				// updates village site data in the land model
	
	private final String script_name;		// the name of the script
	
	// must know # variables, variable names,
	
	private TypeScripts ( String script_name )
	{
		this.script_name = script_name;
		
	}
	
	public String toString ()
	{
		return script_name.toLowerCase();
		
	}
	
}

/*
 *	IScript Interface has the following methods:
 *		Name()
 *		Execute()
 *		
 *	Abstract Script Class inherits from IScript
 *		has name attribute
 *		constructor takes name
 *		Name() provides script name (lowercase)
 */
