/*
 * VillageData.java
 *
 * Package Version: 3.0
 *   Class Version: 1.0
 *   
 * 		   Project: MEDLAND Project
 * 					Arizona State University
 * 
 * 			Author: Gary R. Mayer
 *   		  Date: 28 June 2007
 * 
 * This file stores the village data used by the interaction model (IM).
 * 
 * Note that locations are stored in GRASS east_north format.
 * 
 * Class Fixes / Additions:
 * 1.0:
 *  - None.
 * 
 * Bugs / Issues:
 * 1.0:
 *  - None.
 */

package PenaguilaALv4;

import java.util.*;

public final class VillageData
{
	public final int villageID;
	public final String site;
	public final int population;
	public final int area;
	public boolean updated;
	private ArrayList Locations;
	
	/**
	 * VillageData
	 * @param id - a unique id number for the village
	 * @param loc - site location (GRASS e-n format)
	 * @param pop - total village population
	 * @param area - area village requires for housing to support population
	 */
	public VillageData ( int id, String loc, int pop, int area )
	{
		this.villageID = id;
		this.site = loc;
		this.population = pop;
		this.area = area;
		this.Locations = null;
		this.updated = false;
		
	}
	
	public void AddLocation ( String loc )
	{
		if ( this.Locations == null )
			this.Locations = new ArrayList();
		
		this.Locations.add((String)loc);
		
	}
	
	public boolean InVillage ( String loc )
	{	
		if ( this.Locations == null )
			return false;
		
		else
			return this.Locations.contains(loc);
		
	}
	
	public ArrayList GetLocations ()
	{
		return this.Locations;
		
	}
	
	public void ClearLocations ()
	{
		this.Locations = null;
		
	}
	
	public int NumLocations ()
	{
		if ( this.Locations != null )
			return this.Locations.size();
		
		else
			return -1;
		
	}
	
}
