/*
 * ModelInitialization.java
 *
 * Created on October 25, 2007, 7:58 PM
 * 
 * Updated June 2010. More validation checks have been added to the Validate Entries method,
 * -more variables have been included in the GUI
 * -and a number of other small changes have been made concurrent with changes in the model.
 */

package PenaguilaALv4;

import grass.GrassFacade;
import java.io.*;

import javax.swing.*;
import java.util.*;

public class ModelInitialization extends javax.swing.JFrame
{
    DefaultListModel listDataModel = new DefaultListModel();
    JFileChooser jFileChooserGISDBASE = null;
    JFileChooser jFileChooserAPSIM = null;
    JFileChooser jFileChooserGRASS = null;
    JFileChooser jFileChooserPython = null;
    JFileChooser jFileChooserRC6 = null;
    String temp_name = "";
    int num_date_placeholders;
    final String dlm = "||";
    final String dlm_regex = "\\|\\|";  // regular expression (regex) requires backslashes to find '|' character
    private static ModelInitialization mi = null;
    private static final String ver = "5.25";
    private static final String ver_date = "May 2012";
    
    // IMPORTANT NOTE: SimView waits on the creation of the models until ip is no longer null.
    //	ip MUST remain null until the parameters are validated and all ip values are assigned.
    //  Using this approach, ip must remain a static variable.
    public static InitParameters ip = null;
    
    public static InitParameters GetInitialization ()
    {
        java.awt.EventQueue.invokeLater(new Runnable()
            {
                public void run()
                {
                    mi = new ModelInitialization();
                    mi.setVisible(true);
                }
            });
        
        return ip;
        
    }
    
    public static void Close ()
    {
    	if ( mi != null )
    	{
    		mi.setVisible(false);
    		mi = null;
    	}
    	
    }
        
    /**
     * Creates new form ModelInitialization
     */
    private ModelInitialization() {

    	
    	//parameterLoader(); // used to run the model without the GUI, model paramaters are set within the method
    	//section below must be uncommented to initiate the GUI
    	 initComponents();
         
         // initialization
         jTextVillagePctInitClearing.setText("50");
         jSliderVillagePctInitClearing.setValue(50);
         jTextYieldExpScalar.setText("75");
         jSliderYieldExpScalar.setValue(75);
         jLabelVersion.setText(ver + ", " + ver_date);
         jFileChooserGISDBASE = new JFileChooser();
         jFileChooserAPSIM = new JFileChooser();
         jFileChooserGRASS = new JFileChooser();
         jFileChooserPython = new JFileChooser();
         jFileChooserRC6 = new JFileChooser();
         jFileChooserGISDBASE.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         jFileChooserAPSIM.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         jFileChooserGRASS.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         jFileChooserPython.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         jFileChooserRC6.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
         
         num_date_placeholders = 3;
         
    }
    
    private void parameterLoader()   {
        File file = new File("/Users/sbergin/aparameter");
        boolean load_error = false;
        String data = "";
        String[] sdata = (String[])null;
        ip = new InitParameters();
        System.out.println("param loader");
        try
        {
          BufferedReader in = new BufferedReader(new FileReader(file));

          data = in.readLine();
          while ((data != null) && (data.compareTo("[VERSION]") != 0)) {
            data = in.readLine();
          }
          if (data == null)
            throw new IllegalArgumentException("Version data expected but not found.\n" + data.toString());
          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata.length != 2) {
            throw new IllegalArgumentException("Invalid version data.");
          }
          if ((sdata[0].compareTo("4.00") != 0) || (sdata[1].compareTo("January 2011") != 0)) {
            throw new IllegalArgumentException("Invalid version data.");
          }
          data = in.readLine();
          if (data.compareTo("[VILLAGES]") != 0) {
            throw new IllegalArgumentException("Village data expected but not found.\n" + data.toString());
          }

          data = in.readLine();
          int num_villages = Integer.parseInt(data);

          for (int v = 0; v < num_villages; v++)
          {
            data = in.readLine();
            sdata = data.split("\\|\\|");

            if (sdata.length != 6) {
              throw new IllegalArgumentException("Invalid village data row.");
            }
            ip.AddVillage(v, sdata[0], 
              Float.valueOf(sdata[1]).floatValue(), 
              Float.valueOf(sdata[2]).floatValue(), 
              Integer.valueOf(sdata[3]).intValue(), 
              Integer.valueOf(sdata[4]).intValue(), 
              Integer.valueOf(sdata[5]).intValue());
          }

          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata.length != 3) {
            throw new IllegalArgumentException("Invalid agent model NS coordinate/resolution data. " + sdata.length);
          }
          for (int d = 0; d < 3; d++) {
            if (sdata[d].compareTo(" ") == 0)
              sdata[d] = "";
          }
          ip.agent_n = Float.valueOf(sdata[0]).floatValue();
          ip.agent_s = Float.valueOf(sdata[1]).floatValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata.length != 3) {
            throw new IllegalArgumentException("Invalid agent model EW coordinate/resolution data.");
          }
          for (int d = 0; d < 3; d++) {
            if (sdata[d].compareTo(" ") == 0)
              sdata[d] = "";
          }
          ip.agent_e = Float.valueOf(sdata[0]).floatValue();
          ip.agent_w = Float.valueOf(sdata[1]).floatValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata.length != 2) {
            throw new IllegalArgumentException("Invalid agent model checkbox data.");
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          data = in.readLine();
          System.out.println(data);

          if (data.compareTo("[HOUSEHOLDS]") != 0) {
            throw new IllegalArgumentException("Household data expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.birth_init_prob = Integer.valueOf(sdata[0]).intValue();
          ip.birth_prob_group_size = Integer.valueOf(sdata[1]).intValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.birth_prob_delta = Integer.valueOf(sdata[0]).intValue();
          ip.birth_prob_min = Integer.valueOf(sdata[1]).intValue();
          ip.birth_prob_max = Integer.valueOf(sdata[2]).intValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.death_init_prob = Integer.valueOf(sdata[0]).intValue();
          ip.death_prob_group_size = Integer.valueOf(sdata[1]).intValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.death_prob_delta = Integer.valueOf(sdata[0]).intValue();
          ip.death_prob_min = Integer.valueOf(sdata[1]).intValue();
          ip.death_prob_max = Integer.valueOf(sdata[2]).intValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.wheat_reqd_per_capita = Integer.valueOf(sdata[0]).intValue();
          ip.man_days_per_capita = Integer.valueOf(sdata[1]).intValue();
          ip.meat_milk_calories_pc = Double.valueOf(sdata[2]).doubleValue();
          ip.maxfarmcost = Integer.valueOf(sdata[3]).intValue();
          ip.yield_expecation_scalar = Integer.valueOf(sdata[4]).intValue();
          ip.labor_pop_pct = Integer.valueOf(sdata[5]).intValue();
          ip.pct_clearing = Integer.valueOf(sdata[6]).intValue();

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[RESOURCES]") != 0) {
            throw new IllegalArgumentException("GRASS resource data expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.wheat_labor_reqd = Float.valueOf(sdata[0]).floatValue();
          ip.wheat_init_expected_yield = Float.valueOf(sdata[1]).floatValue();
          ip.wheat_calories_provided = Double.valueOf(sdata[2]).doubleValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.barley_labor_reqd = Double.valueOf(sdata[0]).doubleValue();
          ip.barley_init_expected_yield = Double.valueOf(sdata[1]).doubleValue();

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.maximum_wheat_yield = Double.valueOf(sdata[0]).doubleValue();
          ip.maximum_barley_yield = Double.valueOf(sdata[1]).doubleValue();

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[GRASS ENVIRONMENT]") != 0) {
            throw new IllegalArgumentException("GRASS environment data expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.GISDBASE = sdata[0];
          ip.location = sdata[1];
          ip.mapset = sdata[2];

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[ENVIRONMENT INITIALIZATION MAPS]") != 0) {
            throw new IllegalArgumentException("GRASS initialization maps expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          ip.DEM = sdata[0];
          ip.land_cover = sdata[1];
          ip.fertility_map = sdata[2];

          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata[0].compareTo("value") == 0)
          {
            ip.friction_value = Float.parseFloat(sdata[1]);
          }
          else if (sdata[0].compareTo("map") == 0)
          {
            ip.friction_map = sdata[1];
          }
          else
          {
            throw new IllegalArgumentException("Invalid parameter type.");
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata[0].compareTo("value") == 0)
          {
            ip.soilK_value = Float.parseFloat(sdata[1]);
          }
          else if (sdata[0].compareTo("map") == 0)
          {
            ip.soilK_map = sdata[1];
          }
          else
          {
            throw new IllegalArgumentException("Invalid parameter type.");
          }

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[INTERACTION TIMING]") != 0) {
            throw new IllegalArgumentException("GRASS initialization maps expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          ip.date_start = Integer.parseInt(sdata[0]);
          ip.date_change_dir = Integer.parseInt(sdata[1]);
          ip.date_change_dir = Integer.parseInt(sdata[2]);
          ip.date_end = Integer.parseInt(sdata[3]);
          ip.date_prefix = sdata[4];
          ip.date_suffix = sdata[5];
          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[SYSTEM SETTINGS]") != 0) {
            throw new IllegalArgumentException("SYSTEM SETTINGS expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          ip.APSIMPath = sdata[0];
          ip.GrassPath = sdata[1];
          ip.PythonPath = sdata[2];
          ip.grassrc6loc = sdata[3];

          ip.WorldWindDisplayOn = false;

          ip.HouseholdLogOn = true;

          ip.VillageLogOn = true;

          ip.IMLogOn = true;

          ip.KeepRasterMaps = false;

          String RulesPath = ip.APSIMPath + "Rules/";
          ip.landcover_reclass_rules = (RulesPath + "luse_reclass_rules.txt");
          ip.landcover_color_rules = (RulesPath + "luse_colors.txt");
          ip.cfactor_reclass_rules = (RulesPath + "cfactor_recode_rules.txt");
          ip.cfactor_color_rules = (RulesPath + "cfactor_colors.txt");
          ip.impacts_color_rules = (RulesPath + "impacts_colors.txt");
          ip.fertility_color_rules = (RulesPath + "sfertil_colors.txt");

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[GRASS EVOL SETTINGS]") != 0) {
            throw new IllegalArgumentException("GRASS Evol Settings expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");

          ip.soil_density = Double.valueOf(sdata[0]).doubleValue();
          ip.r_factor = Double.valueOf(sdata[1]).doubleValue();
          //ip.rain_days = Integer.valueOf(sdata[2]).intValue();
          //ip.rain_value = Double.valueOf(sdata[3]).doubleValue();
          //ip.infiltration = Double.valueOf(sdata[4]).doubleValue();
          ip.stream_transport = sdata[5];
          ip.load_exponent = Double.valueOf(sdata[6]).doubleValue();

          if (Integer.valueOf(sdata[7]).intValue() != 1) {
            ip.evolSmoothing = "low";
          }
          else {
            ip.evolSmoothing = "high";
          }

          ip.kappa = Integer.valueOf(sdata[8]).intValue();
          ip.cutoff1 = Double.valueOf(sdata[9]).doubleValue();
          ip.cutoff2 = Double.valueOf(sdata[10]).doubleValue();
          ip.cutoff3 = Double.valueOf(sdata[11]).doubleValue();

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[GRASS LANDSCAPE SETTINGS]") != 0) {
            throw new IllegalArgumentException("GRASS landscape settings expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.soil_depth_min = Double.valueOf(sdata[0]).doubleValue();
          ip.soil_depth_max = Double.valueOf(sdata[1]).doubleValue();
          //ip.fertility_impact_value = Integer.valueOf(sdata[2]).intValue();
          ip.soil_recovery = Integer.valueOf(sdata[3]).intValue();

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[OVICAPRID VALUES]") != 0) {
            throw new IllegalArgumentException("GRASS ovicparid value expected but not found.\n" + data.toString());
          }

          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.ovicaprid_per_person = Integer.valueOf(sdata[0]).intValue();
          ip.sheep_ratio = Integer.valueOf(sdata[1]).intValue();
          ip.goat_ratio = Integer.valueOf(sdata[2]).intValue();
          ip.ovicaprid_density = Double.valueOf(sdata[3]).doubleValue();
          ip.goat_fodder = Integer.valueOf(sdata[4]).intValue();
          ip.goat_return = Double.valueOf(sdata[5]).doubleValue();
          ip.sheep_fodder = Integer.valueOf(sdata[6]).intValue();
          ip.sheep_return = Double.valueOf(sdata[7]).doubleValue();

          if (Integer.valueOf(sdata[8]).intValue() != 1)
            ip.fallow_field_grazing = true;
          else {
            ip.fallow_field_grazing = false;
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.init_grazing_expectation = Double.valueOf(sdata[0]).doubleValue();
          ip.barley_fodder_percentage = Double.valueOf(sdata[1]).doubleValue();
          ip.pct_lactating_animals = Double.valueOf(sdata[2]).doubleValue();
          ip.pct_meat_animals = Double.valueOf(sdata[3]).doubleValue();
          ip.sheep_return = Double.valueOf(sdata[4]).doubleValue();
          ip.goat_return = Double.valueOf(sdata[5]).doubleValue();

          data = in.readLine();
          System.out.println(data);
          if (data.compareTo("[CLIMATE AND WOOD GATHERING]") != 0) {
            throw new IllegalArgumentException("GRASS climate and wood gathering values expected but not found.\n" + data.toString());
          }
          data = in.readLine();
          sdata = data.split("\\|\\|");
          ip.wood_pc = Double.valueOf(sdata[0]).doubleValue();
          ip.wood_intensity = Double.valueOf(sdata[1]).doubleValue();
          ip.climateDataPath = sdata[2];
          if (Integer.valueOf(sdata[3]).intValue() != 1)
          {
            ip.climateDataSeries = false;
          }
          else {
            ip.climateDataSeries = true;
          }

        }
        catch (Exception exc)
        {
          load_error = true;
          System.err.println(exc.toString());
        }
        finally
        {
          if (load_error)
            System.out.println("There was an error loading the data. Please Check the values and try again. ");
        }
        	
    }
    
    private void manualInit() {
    	 	
    	ip = new InitParameters();
        
        // Set Village data
    	float ns = 4284845.0f;
    	float ew = 725951.0f;
    	//int id, String name, float ns, float ew, int clearing, int hh, int hh_pop
        ip.AddVillage(0,"Village_0", ns,ew, 50, 3, 6);
        
        ns = 4282716.5f;
    	ew = 725262.06f;
        ip.AddVillage(1,"Village_1", ns,ew, 50, 3, 6);
                
        ip.pct_clearing = 50;
        
        // Get agent coordinates and resolution
        // Assume validation will insert landscape coords, and res if checked.
        ip.agent_n = 4287301.5f;
        ip.agent_s = 4281381.5f;
        ip.agent_e = 722508.7f;
        ip.agent_w = 729111.0f;
        //ip.agent_ns_res = 5.0002913f;
        //ip.agent_ew_res = 5.001776f;
         
        // Set Household data
        ip.birth_init_prob = 3;
        ip.birth_prob_group_size = 6;
        ip.birth_prob_delta = 1;
        ip.birth_prob_min = 1;
        ip.birth_prob_max = 5;
        ip.death_init_prob = 2;
        ip.death_prob_group_size = 6;
        ip.death_prob_delta = 5;
        ip.death_prob_min = 2;
        ip.death_prob_max = 10;
        ip.labor_pop_pct = 70;        
        ip.man_days_per_capita = 300;
        ip.wheat_reqd_per_capita = 0;
        //ip.max_distance_cost = 800;
        ip.yield_expecation_scalar = 100;
        
        // Set Resources data
        ip.wheat_labor_reqd = 50;
        ip.wheat_init_expected_yield = 450;
        ip.wheat_calories_provided = 3500;
        
        // Set Environment data
        ip.GISDBASE = "/Users/sbergin/Documents/Grassdata/";
        ip.location = "Penaguila";
        ip.mapset = "APSIMMapset";
        ip.DEM = "elev";
        ip.land_cover = "landcover";
        ip.friction_value = 1.0f;
        ip.friction_map = "friction_constant_map";
        ip.soilK_value = 0.4f;
        ip.soilK_map = "kfactor_constant_map";
        
        // Set Interaction data
        ip.date_start = 0;
        ip.date_change_dir = 1;
        ip.date_end = 250;
        ip.date_prefix = "y";
        ip.date_suffix = "_";
        
        	
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnGroupFrictionMap = new javax.swing.ButtonGroup();
        btnGroupSoilKMap = new javax.swing.ButtonGroup();
        jButtonConfigInitialize = new javax.swing.JButton();
        jTabbedPaneMain = new javax.swing.JTabbedPane();
        jTabbedPaneAgents = new javax.swing.JTabbedPane();
        jPanelVillages = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel27 = new javax.swing.JLabel();
        jTextNumVillages = new javax.swing.JTextField();
        jLabel31 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jButtonVillageAdd = new javax.swing.JButton();
        jSliderVillagePctInitClearing = new javax.swing.JSlider();
        jTextVillageName = new javax.swing.JTextField();
        jTextVillagePctInitClearing = new javax.swing.JTextField();
        jButtonVillageClearData = new javax.swing.JButton();
        jButtonVillageRemove = new javax.swing.JButton();
        jLabel30 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jTextVillageNSCoord = new javax.swing.JTextField();
        jScrollPane4 = new javax.swing.JScrollPane();
        jTableVillageData = new javax.swing.JTable();
        jTextVillageEWCoord = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel1 = new javax.swing.JLabel();
        jCheckBoxEnvironmentCoordinates = new javax.swing.JCheckBox();
        jCheckBoxEnvironmentResolutions = new javax.swing.JCheckBox();
        jLabelHumanNCoord = new javax.swing.JLabel();
        jLabelHumanSCoord = new javax.swing.JLabel();
        jLabelHumanECoord = new javax.swing.JLabel();
        jLabelHumanWCoord = new javax.swing.JLabel();
        jTextHumanNorthCoords = new javax.swing.JTextField();
        jTextHumanSouthCoords = new javax.swing.JTextField();
        jTextHumanEastCoords = new javax.swing.JTextField();
        jTextHumanWestCoords = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        jTextVillageNumHouseholds = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jTextVillageHhStartPopulation = new javax.swing.JTextField();
        jPanelHouseholds = new javax.swing.JPanel();
        jLabel8 = new javax.swing.JLabel();
        jTextManDaysPerPerson = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jSpinnerBirthProbPerPeople = new javax.swing.JSpinner();
        jSpinnerDeathProbPerPeople = new javax.swing.JSpinner();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel23 = new javax.swing.JLabel();
        jLabel24 = new javax.swing.JLabel();
        jLabel25 = new javax.swing.JLabel();
        jLabel32 = new javax.swing.JLabel();
        jLabel33 = new javax.swing.JLabel();
        jSpinnerBirthProbMin = new javax.swing.JSpinner();
        jLabel34 = new javax.swing.JLabel();
        jSpinnerBirthProbMax = new javax.swing.JSpinner();
        jLabel35 = new javax.swing.JLabel();
        jLabel36 = new javax.swing.JLabel();
        jSpinnerDeathProbMin = new javax.swing.JSpinner();
        jSpinnerDeathProbMax = new javax.swing.JSpinner();
        jLabel37 = new javax.swing.JLabel();
        jLabel39 = new javax.swing.JLabel();
        jLabel40 = new javax.swing.JLabel();
        jLabel28 = new javax.swing.JLabel();
        jLabel29 = new javax.swing.JLabel();
        jTextYieldExpScalar = new javax.swing.JTextField();
        jLabel41 = new javax.swing.JLabel();
        jLabel65 = new javax.swing.JLabel();
        jTextMaxFarmDistance = new javax.swing.JTextField();
        jLabel69 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        jLabel70 = new javax.swing.JLabel();
        jSpinnerPopulationPctLabor = new javax.swing.JSpinner();
        jSliderYieldExpScalar = new javax.swing.JSlider();
        jSpinnerBirthInitProb = new javax.swing.JTextField();
        jSpinnerBirthProbDelta = new javax.swing.JTextField();
        jSpinnerDeathInitProb = new javax.swing.JTextField();
        jSpinnerDeathProbDelta = new javax.swing.JTextField();
        jComboBoxHHSizeLimit = new javax.swing.JComboBox();
        jLabel44 = new javax.swing.JLabel();
        jLabel192 = new javax.swing.JLabel();
        jTextHHSizeLimit = new javax.swing.JTextField();
        jLabel42 = new javax.swing.JLabel();
        jTextAgpastKcal = new javax.swing.JTextField();
        jTextAgriProportion = new javax.swing.JTextField();
        jTextPastoralProportion = new javax.swing.JTextField();
        jLabel46 = new javax.swing.JLabel();
        jLabel170 = new javax.swing.JLabel();
        jLabel171 = new javax.swing.JLabel();
        jPanelWood = new javax.swing.JPanel();
        jLabel186 = new javax.swing.JLabel();
        jTextYieldExpScalar2 = new javax.swing.JTextField();
        jLabel189 = new javax.swing.JLabel();
        jLabel159 = new javax.swing.JLabel();
        jLabel160 = new javax.swing.JLabel();
        jTextWoodPc = new javax.swing.JTextField();
        jLabel161 = new javax.swing.JLabel();
        jLabel162 = new javax.swing.JLabel();
        jTextFieldWoodIntensity = new javax.swing.JTextField();
        jLabel163 = new javax.swing.JLabel();
        jLabel176 = new javax.swing.JLabel();
        jTextFieldWoodDistanceWeight = new javax.swing.JTextField();
        jPanelResources1 = new javax.swing.JPanel();
        jLabel127 = new javax.swing.JLabel();
        jLabel122 = new javax.swing.JLabel();
        jLabel140 = new javax.swing.JLabel();
        jLabel125 = new javax.swing.JLabel();
        jLabel120 = new javax.swing.JLabel();
        jLabel142 = new javax.swing.JLabel();
        jLabel137 = new javax.swing.JLabel();
        jLabel139 = new javax.swing.JLabel();
        jTextFieldgrazingDensity = new javax.swing.JTextField();
        jTextFieldGoatYield = new javax.swing.JTextField();
        jLabel138 = new javax.swing.JLabel();
        jLabel136 = new javax.swing.JLabel();
        jTextFieldnumOfOvicaprids = new javax.swing.JTextField();
        jLabel126 = new javax.swing.JLabel();
        jTextFieldGoatFodder = new javax.swing.JTextField();
        jTextFieldgoatRatio = new javax.swing.JTextField();
        jLabel144 = new javax.swing.JLabel();
        jLabel143 = new javax.swing.JLabel();
        jLabel123 = new javax.swing.JLabel();
        jTextFieldsheepRatio = new javax.swing.JTextField();
        jLabel141 = new javax.swing.JLabel();
        jComboBoxFallFieldGrz = new javax.swing.JComboBox();
        jTextFieldSheepFodder = new javax.swing.JTextField();
        jTextFieldSheepYield = new javax.swing.JTextField();
        jLabel172 = new javax.swing.JLabel();
        jTextFieldInitGrazingReturn = new javax.swing.JTextField();
        jLabel173 = new javax.swing.JLabel();
        jLabel174 = new javax.swing.JLabel();
        jTextFieldPctBarleyFodder = new javax.swing.JTextField();
        jLabel175 = new javax.swing.JLabel();
        jLabel177 = new javax.swing.JLabel();
        jTextFieldGrazingDistWight = new javax.swing.JTextField();
        jTextFieldLcovDistWight = new javax.swing.JTextField();
        jLabel178 = new javax.swing.JLabel();
        jLabel182 = new javax.swing.JLabel();
        jTextMaxGrazeDistance = new javax.swing.JTextField();
        jPanelResources = new javax.swing.JPanel();
        jLabel26 = new javax.swing.JLabel();
        jLabel38 = new javax.swing.JLabel();
        jLabel71 = new javax.swing.JLabel();
        jLabel73 = new javax.swing.JLabel();
        jLabel74 = new javax.swing.JLabel();
        jTextWheatLaborRqd = new javax.swing.JTextField();
        jTextWheatInitYield = new javax.swing.JTextField();
        jLabel75 = new javax.swing.JLabel();
        jLabel76 = new javax.swing.JLabel();
        jTextWheatCalories = new javax.swing.JTextField();
        jLabel145 = new javax.swing.JLabel();
        jLabel146 = new javax.swing.JLabel();
        jTextBarleyLaborRqd = new javax.swing.JTextField();
        jTextBarleyInitYield = new javax.swing.JTextField();
        jLabel166 = new javax.swing.JLabel();
        jLabel167 = new javax.swing.JLabel();
        jLabel168 = new javax.swing.JLabel();
        jLabel169 = new javax.swing.JLabel();
        jTextFieldMaxWheatYield = new javax.swing.JTextField();
        jTextFieldMaxBarleyYield = new javax.swing.JTextField();
        jLabel179 = new javax.swing.JLabel();
        jTextFieldSoilFtyWight = new javax.swing.JTextField();
        jLabel180 = new javax.swing.JLabel();
        jTextFieldDepthWght = new javax.swing.JTextField();
        jLabel181 = new javax.swing.JLabel();
        jTextFieldFarmDistWght = new javax.swing.JTextField();
        jLabel184 = new javax.swing.JLabel();
        jTextFieldSeedPct = new javax.swing.JTextField();
        jLabel185 = new javax.swing.JLabel();
        jTextFieldBpointLC1 = new javax.swing.JTextField();
        jTextFieldBpointLC2 = new javax.swing.JTextField();
        jTextFieldBpointDA1 = new javax.swing.JTextField();
        jTextFieldBpointDA2 = new javax.swing.JTextField();
        jLabel183 = new javax.swing.JLabel();
        jTextField5 = new javax.swing.JTextField();
        jTextField6 = new javax.swing.JTextField();
        jLabel193 = new javax.swing.JLabel();
        jLabel194 = new javax.swing.JLabel();
        jLabel195 = new javax.swing.JLabel();
        jLabel196 = new javax.swing.JLabel();
        jLabel197 = new javax.swing.JLabel();
        jComboBoxLandTenure = new javax.swing.JComboBox();
        jLabel198 = new javax.swing.JLabel();
        jPanelEnvironment = new javax.swing.JPanel();
        jTabbedPaneEnv = new javax.swing.JTabbedPane();
        jPanelMaps = new javax.swing.JPanel();
        jLabel21 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jTextGISDBASE = new javax.swing.JTextField();
        jButtonBrowseRootDir = new javax.swing.JButton();
        jComboBoxLocation = new javax.swing.JComboBox();
        jLabel18 = new javax.swing.JLabel();
        jTextMapset = new javax.swing.JTextField();
        jLabel19 = new javax.swing.JLabel();
        jButtonGenerateMapset = new javax.swing.JButton();
        jLabel45 = new javax.swing.JLabel();
        jLabel22 = new javax.swing.JLabel();
        jComboBoxDEM = new javax.swing.JComboBox();
        jLabel48 = new javax.swing.JLabel();
        jComboBoxLandCover = new javax.swing.JComboBox();
        jLabel51 = new javax.swing.JLabel();
        jRadioButtonFrictionValue = new javax.swing.JRadioButton();
        jTextFrictionValue = new javax.swing.JTextField();
        jLabel47 = new javax.swing.JLabel();
        jRadioButtonFrictionMap = new javax.swing.JRadioButton();
        jComboBoxFrictionMap = new javax.swing.JComboBox();
        jLabel49 = new javax.swing.JLabel();
        jRadioButtonSoilKValue = new javax.swing.JRadioButton();
        jTextSoilKValue = new javax.swing.JTextField();
        jLabel50 = new javax.swing.JLabel();
        jRadioButtonSoilKMap = new javax.swing.JRadioButton();
        jComboBoxSoilKMap = new javax.swing.JComboBox();
        jLabel60 = new javax.swing.JLabel();
        jLabel43 = new javax.swing.JLabel();
        jLabel67 = new javax.swing.JLabel();
        jComboBoxFertility = new javax.swing.JComboBox();
        jComboBoxBedrock = new javax.swing.JComboBox();
        jLabel147 = new javax.swing.JLabel();
        jTextFieldMaxLcov = new javax.swing.JTextField();
        jLabel190 = new javax.swing.JLabel();
        jComboBoxMaxLandCover = new javax.swing.JComboBox();
        jLabel156 = new javax.swing.JLabel();
        jPanelMaps1 = new javax.swing.JPanel();
        jLabel150 = new javax.swing.JLabel();
        jTextFieldrFactor = new javax.swing.JTextField();
        jLabel92 = new javax.swing.JLabel();
        jLabel99 = new javax.swing.JLabel();
        jTextFieldrainValue = new javax.swing.JTextField();
        jTextFieldStormEvents = new javax.swing.JTextField();
        jLabel100 = new javax.swing.JLabel();
        jLabel151 = new javax.swing.JLabel();
        jLabel152 = new javax.swing.JLabel();
        jTextFieldClimFile = new javax.swing.JTextField();
        jButtonClimFile = new javax.swing.JButton();
        jLabel153 = new javax.swing.JLabel();
        jComboBoxClimateSource = new javax.swing.JComboBox();
        jLabel154 = new javax.swing.JLabel();
        jLabel155 = new javax.swing.JLabel();
        jLabel164 = new javax.swing.JLabel();
        jLabel165 = new javax.swing.JLabel();
        jLabel111 = new javax.swing.JLabel();
        jTextFieldStormSpeed = new javax.swing.JTextField();
        jLabel187 = new javax.swing.JLabel();
        jTextFieldStormLength = new javax.swing.JTextField();
        jLabel188 = new javax.swing.JLabel();
        jTextFieldStormPrecip = new javax.swing.JTextField();
        jPanelLandEvol = new javax.swing.JPanel();
        jLabel121 = new javax.swing.JLabel();
        jTextYieldExpScalar1 = new javax.swing.JTextField();
        jLabel124 = new javax.swing.JLabel();
        jLabel64 = new javax.swing.JLabel();
        jLabel81 = new javax.swing.JLabel();
        jComboBoxSoilKMap1 = new javax.swing.JComboBox();
        jRadioButtonSoilKMap1 = new javax.swing.JRadioButton();
        jRadioButtonSoilKValue1 = new javax.swing.JRadioButton();
        jLabel86 = new javax.swing.JLabel();
        jTextSoilKValue1 = new javax.swing.JTextField();
        jComboBoxFrictionMap1 = new javax.swing.JComboBox();
        jRadioButtonFrictionMap1 = new javax.swing.JRadioButton();
        jRadioButtonFrictionValue1 = new javax.swing.JRadioButton();
        jLabel87 = new javax.swing.JLabel();
        jLabel88 = new javax.swing.JLabel();
        jComboBoxLandCover1 = new javax.swing.JComboBox();
        jTextFrictionValue1 = new javax.swing.JTextField();
        jComboBoxDEM1 = new javax.swing.JComboBox();
        jLabel89 = new javax.swing.JLabel();
        jLabel93 = new javax.swing.JLabel();
        jTextMapset1 = new javax.swing.JTextField();
        jButtonGenerateMapset1 = new javax.swing.JButton();
        jComboBoxLocation1 = new javax.swing.JComboBox();
        jTextGISDBASE1 = new javax.swing.JTextField();
        jButtonBrowseRootDir1 = new javax.swing.JButton();
        jLabel94 = new javax.swing.JLabel();
        jLabel95 = new javax.swing.JLabel();
        jLabel96 = new javax.swing.JLabel();
        jLabel97 = new javax.swing.JLabel();
        jLabel98 = new javax.swing.JLabel();
        jLabel101 = new javax.swing.JLabel();
        jComboBoxSoilKMap2 = new javax.swing.JComboBox();
        jRadioButtonSoilKMap2 = new javax.swing.JRadioButton();
        jRadioButtonSoilKValue2 = new javax.swing.JRadioButton();
        jLabel102 = new javax.swing.JLabel();
        jTextSoilKValue2 = new javax.swing.JTextField();
        jComboBoxFrictionMap2 = new javax.swing.JComboBox();
        jRadioButtonFrictionMap2 = new javax.swing.JRadioButton();
        jRadioButtonFrictionValue2 = new javax.swing.JRadioButton();
        jLabel103 = new javax.swing.JLabel();
        jLabel104 = new javax.swing.JLabel();
        jComboBoxLandCover2 = new javax.swing.JComboBox();
        jTextFrictionValue2 = new javax.swing.JTextField();
        jComboBoxDEM2 = new javax.swing.JComboBox();
        jLabel105 = new javax.swing.JLabel();
        jLabel106 = new javax.swing.JLabel();
        jTextMapset2 = new javax.swing.JTextField();
        jButtonGenerateMapset2 = new javax.swing.JButton();
        jComboBoxLocation2 = new javax.swing.JComboBox();
        jTextGISDBASE2 = new javax.swing.JTextField();
        jButtonBrowseRootDir2 = new javax.swing.JButton();
        jLabel107 = new javax.swing.JLabel();
        jLabel108 = new javax.swing.JLabel();
        jLabel109 = new javax.swing.JLabel();
        jLabel110 = new javax.swing.JLabel();
        jLabel90 = new javax.swing.JLabel();
        jLabel91 = new javax.swing.JLabel();
        jTextFieldsoilDensity = new javax.swing.JTextField();
        jLabel112 = new javax.swing.JLabel();
        jTextFieldstreamTrans = new javax.swing.JTextField();
        jLabel113 = new javax.swing.JLabel();
        jTextFieldloadExponent = new javax.swing.JTextField();
        jLabel115 = new javax.swing.JLabel();
        jTextFieldkappa = new javax.swing.JTextField();
        jLabel116 = new javax.swing.JLabel();
        jTextFieldcutoff1 = new javax.swing.JTextField();
        jTextFieldcutoff2 = new javax.swing.JTextField();
        jLabel117 = new javax.swing.JLabel();
        jTextFieldcutoff3 = new javax.swing.JTextField();
        jLabel118 = new javax.swing.JLabel();
        jLabel119 = new javax.swing.JLabel();
        jLabel114 = new javax.swing.JLabel();
        jComboBoxSmoothing = new javax.swing.JComboBox();
        jLabel131 = new javax.swing.JLabel();
        jLabel132 = new javax.swing.JLabel();
        jLabel133 = new javax.swing.JLabel();
        jTextFieldSoilMin = new javax.swing.JTextField();
        jTextFieldSoilMax = new javax.swing.JTextField();
        jLabel134 = new javax.swing.JLabel();
        jTextFieldSoilRecovery = new javax.swing.JTextField();
        jLabel135 = new javax.swing.JLabel();
        jTextFieldFertilityImpact = new javax.swing.JTextField();
        jPanelIM = new javax.swing.JPanel();
        jLabel52 = new javax.swing.JLabel();
        jLabel53 = new javax.swing.JLabel();
        jLabel54 = new javax.swing.JLabel();
        jComboBoxDateChangeDir = new javax.swing.JComboBox();
        jLabel55 = new javax.swing.JLabel();
        jLabel56 = new javax.swing.JLabel();
        jLabel57 = new javax.swing.JLabel();
        jLabelFullMapName = new javax.swing.JLabel();
        jTextDateChangeValue = new javax.swing.JTextField();
        jLabel59 = new javax.swing.JLabel();
        jTextDateEnd = new javax.swing.JTextField();
        jTextDatePrefix = new javax.swing.JTextField();
        jTextDateSuffix = new javax.swing.JTextField();
        jLabel58 = new javax.swing.JLabel();
        jComboBoxScale = new javax.swing.JComboBox();
        jComboBoxRatio = new javax.swing.JComboBox();
        jLabel149 = new javax.swing.JLabel();
        jTextDateStart = new javax.swing.JTextField();
        jLabel157 = new javax.swing.JLabel();
        jPanelData = new javax.swing.JPanel();
        jTextFieldAPSIM = new javax.swing.JTextField();
        jTextFieldGRASS = new javax.swing.JTextField();
        jTextFieldPython = new javax.swing.JTextField();
        jButtonAPSIM = new javax.swing.JButton();
        jButtonGRASS = new javax.swing.JButton();
        jButtonPython = new javax.swing.JButton();
        jLabel72 = new javax.swing.JLabel();
        jLabel77 = new javax.swing.JLabel();
        jLabel78 = new javax.swing.JLabel();
        jLabel79 = new javax.swing.JLabel();
        jLabel80 = new javax.swing.JLabel();
        jComboBoxWorldWind = new javax.swing.JComboBox();
        jLabel82 = new javax.swing.JLabel();
        jComboBoxHouseholdLog = new javax.swing.JComboBox();
        jLabel83 = new javax.swing.JLabel();
        jComboBoxHHEconLog = new javax.swing.JComboBox();
        jLabel84 = new javax.swing.JLabel();
        jComboBoxIMLog = new javax.swing.JComboBox();
        jLabel85 = new javax.swing.JLabel();
        jLabel61 = new javax.swing.JLabel();
        jLabel62 = new javax.swing.JLabel();
        jLabel63 = new javax.swing.JLabel();
        jTextFieldRC6 = new javax.swing.JTextField();
        jButtonRC6 = new javax.swing.JButton();
        jLabel128 = new javax.swing.JLabel();
        jLabel68 = new javax.swing.JLabel();
        jLabel129 = new javax.swing.JLabel();
        jLabel130 = new javax.swing.JLabel();
        jComboBoxTempMapFiles = new javax.swing.JComboBox();
        jLabel148 = new javax.swing.JLabel();
        jComboBoxHHPopEconLog = new javax.swing.JComboBox();
        jLabel158 = new javax.swing.JLabel();
        jComboBoxIMLogSave = new javax.swing.JComboBox();
        jLabel191 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jButtonConfigCancel = new javax.swing.JButton();
        jButtonConfigSave = new javax.swing.JButton();
        jButtonConfigLoad = new javax.swing.JButton();
        jLabel66 = new javax.swing.JLabel();
        jButtonConfigValidate = new javax.swing.JButton();
        jLabelVersion = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setName("agent_frame"); // NOI18N

        jButtonConfigInitialize.setText("Initialize");
        jButtonConfigInitialize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonConfigInitializeActionPerformed(evt);
            }
        });

        jTabbedPaneMain.setDoubleBuffered(true);
        jTabbedPaneMain.setName("tab_frame"); // NOI18N

        jLabel4.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel4.setLabelFor(jTableVillageData);
        jLabel4.setText("(Note: Village names and coordinate pair may not be duplicated.)");
        jLabel4.setFocusable(false);
        jLabel4.setRequestFocusEnabled(false);
        jLabel4.setVerifyInputWhenFocusTarget(false);

        jLabel27.setText("Initial Clearing:");

        jTextNumVillages.setEditable(false);
        jTextNumVillages.setFont(new java.awt.Font("Dialog", 1, 12));
        jTextNumVillages.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextNumVillages.setText("0");
        jTextNumVillages.setToolTipText("The total number of villages added to the environment");

        jLabel31.setText("%");

        jLabel3.setText("Coordinates (north-south, east-west):");

        jButtonVillageAdd.setText("Add Village");
        jButtonVillageAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonVillageAddActionPerformed(evt);
            }
        });

        jSliderVillagePctInitClearing.setMajorTickSpacing(25);
        jSliderVillagePctInitClearing.setPaintLabels(true);
        jSliderVillagePctInitClearing.setPaintTicks(true);
        jSliderVillagePctInitClearing.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderVillagePctInitClearingStateChanged(evt);
            }
        });

        jTextVillageName.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillageName.setText("Village_0");
        jTextVillageName.setToolTipText("Enter the number of starting villages");
        jTextVillageName.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTextVillageNameFocusGained(evt);
            }
        });

        jTextVillagePctInitClearing.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillagePctInitClearing.setText("50");
        jTextVillagePctInitClearing.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextVillagePctInitClearingActionPerformed(evt);
            }
        });

        jButtonVillageClearData.setText("Remove All");
        jButtonVillageClearData.setEnabled(false);
        jButtonVillageClearData.setName("remove_all_button"); // NOI18N
        jButtonVillageClearData.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonVillageClearDataActionPerformed(evt);
            }
        });

        jButtonVillageRemove.setText("Remove Selected Row");
        jButtonVillageRemove.setEnabled(false);
        jButtonVillageRemove.setName("remove_button"); // NOI18N
        jButtonVillageRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonVillageRemoveActionPerformed(evt);
            }
        });

        jLabel30.setText("of initial village area");

        jLabel5.setLabelFor(jTextNumVillages);
        jLabel5.setText("Number of VIllages:");

        jTextVillageNSCoord.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillageNSCoord.setText("0");
        jTextVillageNSCoord.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTextVillageNSCoordFocusGained(evt);
            }
        });

        jScrollPane4.setToolTipText("Enter data in form below and select 'Add Village'.");

        jTableVillageData.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Village Name", "NS-Coord", "EW-Coord", "Init Clearing", "Households", "Population/Hhd"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Float.class, java.lang.Float.class, java.lang.Integer.class, java.lang.Integer.class, java.lang.Integer.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jTableVillageData.setToolTipText("");
        jTableVillageData.setTableHeader(new javax.swing.table.JTableHeader(jTableVillageData.getColumnModel()));
        jScrollPane4.setViewportView(jTableVillageData);

        jTextVillageEWCoord.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillageEWCoord.setText("0");
        jTextVillageEWCoord.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                jTextVillageEWCoordFocusGained(evt);
            }
        });

        jLabel2.setText("Village Name:");

        jLabel1.setText("Coordinates");

        jCheckBoxEnvironmentCoordinates.setSelected(true);
        jCheckBoxEnvironmentCoordinates.setText("Use Environment Coordinates");
        jCheckBoxEnvironmentCoordinates.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBoxEnvironmentCoordinates.setEnabled(false);
        jCheckBoxEnvironmentCoordinates.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jCheckBoxEnvironmentCoordinates.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxEnvironmentCoordinatesActionPerformed(evt);
            }
        });

        jCheckBoxEnvironmentResolutions.setSelected(true);
        jCheckBoxEnvironmentResolutions.setText("Use Environment Resolutions");
        jCheckBoxEnvironmentResolutions.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBoxEnvironmentResolutions.setEnabled(false);
        jCheckBoxEnvironmentResolutions.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jCheckBoxEnvironmentResolutions.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxEnvironmentResolutionsActionPerformed(evt);
            }
        });

        jLabelHumanNCoord.setText("North:");
        jLabelHumanNCoord.setEnabled(false);

        jLabelHumanSCoord.setText("South:");
        jLabelHumanSCoord.setEnabled(false);

        jLabelHumanECoord.setText("East:");
        jLabelHumanECoord.setEnabled(false);

        jLabelHumanWCoord.setText("West:");
        jLabelHumanWCoord.setEnabled(false);

        jTextHumanNorthCoords.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextHumanNorthCoords.setEnabled(false);

        jTextHumanSouthCoords.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextHumanSouthCoords.setEnabled(false);

        jTextHumanEastCoords.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextHumanEastCoords.setEnabled(false);

        jTextHumanWestCoords.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextHumanWestCoords.setEnabled(false);

        jLabel6.setText("Number of Starting Households in Village:");

        jTextVillageNumHouseholds.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillageNumHouseholds.setText("3");
        jTextVillageNumHouseholds.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextVillageNumHouseholdsActionPerformed(evt);
            }
        });

        jLabel7.setText("Starting Population per Household:");

        jTextVillageHhStartPopulation.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextVillageHhStartPopulation.setText("6");
        jTextVillageHhStartPopulation.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextVillageHhStartPopulationActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanelVillagesLayout = new org.jdesktop.layout.GroupLayout(jPanelVillages);
        jPanelVillages.setLayout(jPanelVillagesLayout);
        jPanelVillagesLayout.setHorizontalGroup(
            jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelVillagesLayout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelVillagesLayout.createSequentialGroup()
                        .add(44, 44, 44)
                        .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 766, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanelVillagesLayout.createSequentialGroup()
                            .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 886, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .addContainerGap())
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelVillagesLayout.createSequentialGroup()
                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jPanelVillagesLayout.createSequentialGroup()
                                    .add(12, 12, 12)
                                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(jPanelVillagesLayout.createSequentialGroup()
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jLabelHumanNCoord)
                                                .add(jLabelHumanSCoord))
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jTextHumanSouthCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 157, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextHumanNorthCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 157, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                                        .add(jCheckBoxEnvironmentCoordinates))
                                    .add(57, 57, 57)
                                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(jCheckBoxEnvironmentResolutions)
                                        .add(jPanelVillagesLayout.createSequentialGroup()
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jLabelHumanECoord)
                                                .add(jLabelHumanWCoord))
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jTextHumanWestCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 157, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextHumanEastCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 157, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                                .add(jPanelVillagesLayout.createSequentialGroup()
                                    .add(jLabel5)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jTextNumVillages, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 34, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(98, 98, 98)
                                    .add(jLabel4))
                                .add(jLabel1)
                                .add(jPanelVillagesLayout.createSequentialGroup()
                                    .add(12, 12, 12)
                                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(jPanelVillagesLayout.createSequentialGroup()
                                            .add(jButtonVillageAdd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 124, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .add(233, 233, 233)
                                            .add(jButtonVillageRemove)
                                            .add(18, 18, 18)
                                            .add(jButtonVillageClearData))
                                        .add(jPanelVillagesLayout.createSequentialGroup()
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                                .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelVillagesLayout.createSequentialGroup()
                                                    .add(jLabel2)
                                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                    .add(jTextVillageName, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 404, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                                .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelVillagesLayout.createSequentialGroup()
                                                    .add(12, 12, 12)
                                                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                                            .add(org.jdesktop.layout.GroupLayout.LEADING, jSliderVillagePctInitClearing, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelVillagesLayout.createSequentialGroup()
                                                                .add(jLabel3)
                                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                                .add(jTextVillageNSCoord, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 115, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                                                        .add(jPanelVillagesLayout.createSequentialGroup()
                                                            .add(jLabel27, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 95, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                            .add(jTextVillagePctInitClearing, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 40, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                            .add(jLabel31)
                                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                            .add(jLabel30)))
                                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                                            .add(jLabel7)
                                                            .add(jLabel6))
                                                        .add(jTextVillageEWCoord, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                            .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jTextVillageHhStartPopulation, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextVillageNumHouseholds, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))))
                            .add(128, 128, 128)))))
        );

        jPanelVillagesLayout.linkSize(new java.awt.Component[] {jTextVillageHhStartPopulation, jTextVillageNumHouseholds}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanelVillagesLayout.linkSize(new java.awt.Component[] {jTextVillageEWCoord, jTextVillageNSCoord}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanelVillagesLayout.linkSize(new java.awt.Component[] {jButtonVillageAdd, jButtonVillageClearData, jButtonVillageRemove}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanelVillagesLayout.setVerticalGroup(
            jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelVillagesLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(jTextNumVillages, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel4))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 107, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButtonVillageAdd)
                    .add(jButtonVillageRemove)
                    .add(jButtonVillageClearData))
                .add(40, 40, 40)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(jTextVillageName, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(jTextVillageNSCoord, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextVillageEWCoord, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel27)
                    .add(jTextVillagePctInitClearing, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel30)
                    .add(jLabel31)
                    .add(jLabel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 15, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextVillageNumHouseholds, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel7)
                        .add(jTextVillageHhStartPopulation, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jSliderVillagePctInitClearing, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 49, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSeparator1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 19, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelVillagesLayout.createSequentialGroup()
                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabelHumanNCoord)
                            .add(jTextHumanNorthCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabelHumanSCoord)
                            .add(jTextHumanSouthCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanelVillagesLayout.createSequentialGroup()
                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabelHumanECoord)
                            .add(jTextHumanEastCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabelHumanWCoord)
                            .add(jTextHumanWestCoords, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .add(44, 44, 44)
                .add(jPanelVillagesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jCheckBoxEnvironmentCoordinates)
                    .add(jCheckBoxEnvironmentResolutions))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jTabbedPaneAgents.addTab("Villages", jPanelVillages);

        jPanelHouseholds.setToolTipText("Initialization human model variables");
        jPanelHouseholds.setName("agent_frame"); // NOI18N
        jPanelHouseholds.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                jPanelHouseholdsInputMethodTextChanged(evt);
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
        });

        jLabel8.setText("Initial Percent Probability:");

        jTextManDaysPerPerson.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextManDaysPerPerson.setText("300");
        jTextManDaysPerPerson.setToolTipText("Used to calculate land manageable per person and, thus, per household");
        jTextManDaysPerPerson.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextManDaysPerPersonActionPerformed(evt);
            }
        });

        jLabel9.setText("Initial Percent Probability:");

        jLabel10.setText("% per");

        jLabel11.setText("% per");

        jSpinnerBirthProbPerPeople.setModel(new SpinnerNumberModel(5, 1, 100, 1));

        jSpinnerDeathProbPerPeople.setModel(new SpinnerNumberModel(5, 1, 100, 1));

        jLabel12.setText("people per family");

        jLabel13.setText("people per family");

        jLabel14.setText("Percent Probability Delta:");

        jLabel15.setText("% (increase/decrease in a cycle)");

        jLabel16.setText("Labor provided:");

        jLabel23.setText("Percent Probability Delta:");

        jLabel24.setText("% (increase/decrease in a cycle)");

        jLabel25.setText("Minimum:");

        jLabel32.setText("Birth Factors:");

        jLabel33.setText("Death Factors:");

        jSpinnerBirthProbMin.setModel(new SpinnerNumberModel(0, 0, 100, 1));
        jSpinnerBirthProbMin.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerBirthProbMinStateChanged(evt);
            }
        });

        jLabel34.setText("Maximum:");

        jSpinnerBirthProbMax.setModel(new SpinnerNumberModel(100, 0, 100, 1));
        jSpinnerBirthProbMax.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerBirthProbMaxStateChanged(evt);
            }
        });

        jLabel35.setText("Minimum:");

        jLabel36.setText("Maximum:");

        jSpinnerDeathProbMin.setModel(new SpinnerNumberModel(0, 0, 100, 1));
        jSpinnerDeathProbMin.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerDeathProbMinStateChanged(evt);
            }
        });

        jSpinnerDeathProbMax.setModel(new SpinnerNumberModel(100, 0, 100, 1));
        jSpinnerDeathProbMax.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerDeathProbMaxStateChanged(evt);
            }
        });

        jLabel37.setText("%");

        jLabel39.setText("%");

        jLabel40.setText("%");

        jLabel28.setFont(new java.awt.Font("Dialog", 1, 10));
        jLabel28.setText("man-days / capita / year");

        jLabel29.setText("Yield Expectation Scalar:");

        jTextYieldExpScalar.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextYieldExpScalar.setText("75");
        jTextYieldExpScalar.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextYieldExpScalar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextYieldExpScalarActionPerformed(evt);
            }
        });

        jLabel41.setText("%");

        jLabel65.setText("Maximum distance cost to travel to farm:");
        jLabel65.setToolTipText("measured in seconds it takes to walk to a location");

        jTextMaxFarmDistance.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextMaxFarmDistance.setText("10800");
        jTextMaxFarmDistance.setToolTipText("Measured in Seconds Using R.Walk");
        jTextMaxFarmDistance.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextMaxFarmDistanceActionPerformed(evt);
            }
        });

        jLabel69.setText("%");

        jLabel20.setText("Percent of population providing labor:");

        jLabel70.setText("% (rounded up to whole person)");

        jSpinnerPopulationPctLabor.setModel(new SpinnerNumberModel(100, 1, 100, 1));

        jSliderYieldExpScalar.setMajorTickSpacing(25);
        jSliderYieldExpScalar.setMaximum(200);
        jSliderYieldExpScalar.setMinorTickSpacing(5);
        jSliderYieldExpScalar.setPaintLabels(true);
        jSliderYieldExpScalar.setPaintTicks(true);
        jSliderYieldExpScalar.setToolTipText("");
        jSliderYieldExpScalar.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderYieldExpScalarStateChanged(evt);
            }
        });

        jSpinnerBirthInitProb.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jSpinnerBirthInitProb.setText("6.6");
        jSpinnerBirthInitProb.setToolTipText("Birth Probability");
        jSpinnerBirthInitProb.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jSpinnerBirthInitProbActionPerformed(evt);
            }
        });

        jSpinnerBirthProbDelta.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jSpinnerBirthProbDelta.setText("1");
        jSpinnerBirthProbDelta.setToolTipText("Birth Probability Annual Change");
        jSpinnerBirthProbDelta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jSpinnerBirthProbDeltaActionPerformed(evt);
            }
        });

        jSpinnerDeathInitProb.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jSpinnerDeathInitProb.setText("5.2");
        jSpinnerDeathInitProb.setToolTipText("Death Probability");
        jSpinnerDeathInitProb.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jSpinnerDeathInitProbActionPerformed(evt);
            }
        });

        jSpinnerDeathProbDelta.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jSpinnerDeathProbDelta.setText("1");
        jSpinnerDeathProbDelta.setToolTipText("Death Probability");
        jSpinnerDeathProbDelta.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jSpinnerDeathProbDeltaActionPerformed(evt);
            }
        });

        jComboBoxHHSizeLimit.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "On", "Off" }));

        jLabel44.setText("Limit Household Size: ");

        jLabel192.setText("Household SizeLimit: ");

        jTextHHSizeLimit.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextHHSizeLimit.setText("75");
        jTextHHSizeLimit.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextHHSizeLimit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextHHSizeLimitActionPerformed(evt);
            }
        });

        jLabel42.setText("Agropastoral kcal / Year:");

        jTextAgpastKcal.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextAgpastKcal.setText("91250");
        jTextAgpastKcal.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextAgpastKcal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextAgpastKcalActionPerformed(evt);
            }
        });

        jTextAgriProportion.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextAgriProportion.setText("50");
        jTextAgriProportion.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextAgriProportion.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextAgriProportionActionPerformed(evt);
            }
        });

        jTextPastoralProportion.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextPastoralProportion.setText("50");
        jTextPastoralProportion.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextPastoralProportion.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextPastoralProportionActionPerformed(evt);
            }
        });

        jLabel46.setText("% from agriculture");

        jLabel170.setText("% from pastoralism");

        jLabel171.setFont(new java.awt.Font("Lucida Grande", 2, 12)); // NOI18N
        jLabel171.setText("(Above percentages must equal 100)");

        org.jdesktop.layout.GroupLayout jPanelHouseholdsLayout = new org.jdesktop.layout.GroupLayout(jPanelHouseholds);
        jPanelHouseholds.setLayout(jPanelHouseholdsLayout);
        jPanelHouseholdsLayout.setHorizontalGroup(
            jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelHouseholdsLayout.createSequentialGroup()
                .add(129, 129, 129)
                .add(jLabel35)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSpinnerDeathProbMin, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 47, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel37)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel36)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSpinnerDeathProbMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel69)
                .add(2879, 2879, 2879))
            .add(jPanelHouseholdsLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel20)
                .add(18, 18, 18)
                .add(jSpinnerPopulationPctLabor, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jLabel70)
                .addContainerGap(2745, Short.MAX_VALUE))
            .add(jPanelHouseholdsLayout.createSequentialGroup()
                .add(12, 12, 12)
                .add(jSliderYieldExpScalar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 826, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(2459, Short.MAX_VALUE))
            .add(jPanelHouseholdsLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(jLabel33)
                        .addContainerGap())
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(jLabel65)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextMaxFarmDistance, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(2955, 2955, 2955)
                                .add(jLabel41))
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelHouseholdsLayout.createSequentialGroup()
                                        .add(jLabel32)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 677, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelHouseholdsLayout.createSequentialGroup()
                                                .add(12, 12, 12)
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                                    .add(jLabel8)
                                                    .add(jLabel23)
                                                    .add(jLabel25))
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelHouseholdsLayout.createSequentialGroup()
                                                                .add(jSpinnerBirthInitProb, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                                .add(5, 5, 5)
                                                                .add(jLabel10)
                                                                .add(6, 6, 6)
                                                                .add(jSpinnerBirthProbPerPeople, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 45, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelHouseholdsLayout.createSequentialGroup()
                                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                                                    .add(jSpinnerBirthProbDelta, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                                        .add(jSpinnerBirthProbMin, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 47, Short.MAX_VALUE)
                                                                        .add(4, 4, 4)
                                                                        .add(jLabel39)))
                                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                                .add(jLabel34)
                                                                .add(11, 11, 11)))
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                                .add(jSpinnerBirthProbMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 46, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                                .add(jLabel40))
                                                            .add(jLabel12))
                                                        .add(9, 9, 9))
                                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(jLabel24)
                                                        .add(18, 18, 18)))
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(18, 18, 18)
                                                        .add(jLabel42)
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                        .add(jTextAgpastKcal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 69, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(83, 83, 83)
                                                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                                .add(jTextPastoralProportion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 55, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                                .add(jLabel170))
                                                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                                .add(jTextAgriProportion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 55, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                                .add(jLabel46))
                                                            .add(jLabel171)))))
                                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jLabel9)
                                                    .add(jLabel14))
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jSpinnerDeathInitProb, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                    .add(jSpinnerDeathProbDelta, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(10, 10, 10)
                                                        .add(jLabel11)
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                        .add(jSpinnerDeathProbPerPeople, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 44, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                        .add(jLabel13))
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                        .add(jLabel15)))
                                                .add(27, 27, 27)
                                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(jLabel44)
                                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                        .add(jComboBoxHHSizeLimit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                                                        .add(jLabel192)
                                                        .add(18, 18, 18)
                                                        .add(jTextHHSizeLimit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 55, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                                        .add(156, 156, 156)))
                                .add(691, 691, 691))
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(20, 20, 20)
                                .add(jLabel16)
                                .add(36, 36, 36)
                                .add(jTextManDaysPerPerson, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 63, Short.MAX_VALUE)
                                .add(18, 18, 18)
                                .add(jLabel28)
                                .add(46, 46, 46)
                                .add(jLabel29)
                                .add(18, 18, 18)
                                .add(jTextYieldExpScalar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 63, Short.MAX_VALUE)
                                .add(2649, 2649, 2649)))
                        .add(20, 20, 20))))
        );

        jPanelHouseholdsLayout.linkSize(new java.awt.Component[] {jSpinnerBirthProbMax, jSpinnerBirthProbMin, jSpinnerDeathProbMax, jSpinnerDeathProbMin}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanelHouseholdsLayout.linkSize(new java.awt.Component[] {jSpinnerPopulationPctLabor, jTextManDaysPerPerson, jTextMaxFarmDistance, jTextYieldExpScalar}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanelHouseholdsLayout.setVerticalGroup(
            jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelHouseholdsLayout.createSequentialGroup()
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(jLabel32)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jSpinnerBirthProbPerPeople, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel8, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 18, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel10)
                        .add(jSpinnerBirthInitProb, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel12)
                        .add(jLabel42, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 18, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jTextAgpastKcal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel23)
                            .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                .add(jLabel24)
                                .add(jSpinnerBirthProbDelta, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(18, 18, 18)
                                .add(jSpinnerBirthProbMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .add(jLabel33))
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(30, 30, 30)
                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jLabel34)
                                    .add(jLabel39)))
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(31, 31, 31)
                                .add(jLabel40))
                            .add(jPanelHouseholdsLayout.createSequentialGroup()
                                .add(20, 20, 20)
                                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jSpinnerBirthProbMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jLabel25)))))
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(18, 18, 18)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextAgriProportion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel46))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextPastoralProportion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel170))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel171)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jSpinnerDeathProbPerPeople, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jComboBoxHHSizeLimit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jLabel13)
                        .add(jLabel44))
                    .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel9)
                        .add(jSpinnerDeathInitProb, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jLabel11)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(8, 8, 8)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel14)
                            .add(jSpinnerDeathProbDelta, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel15)))
                    .add(jPanelHouseholdsLayout.createSequentialGroup()
                        .add(18, 18, 18)
                        .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel192)
                            .add(jTextHHSizeLimit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel35)
                    .add(jSpinnerDeathProbMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel37)
                    .add(jLabel36)
                    .add(jSpinnerDeathProbMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel69))
                .add(18, 18, 18)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jSpinnerPopulationPctLabor, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel20)
                    .add(jLabel70))
                .add(26, 26, 26)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel16)
                    .add(jTextManDaysPerPerson, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel28)
                    .add(jLabel29)
                    .add(jTextYieldExpScalar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(19, 19, 19)
                .add(jPanelHouseholdsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel41)
                    .add(jLabel65)
                    .add(jTextMaxFarmDistance, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSliderYieldExpScalar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(241, 241, 241))
        );

        jTabbedPaneAgents.addTab("Households", jPanelHouseholds);

        jPanelWood.setToolTipText("Initialization human model variables");
        jPanelWood.setName("agent_frame"); // NOI18N
        jPanelWood.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                jPanelWoodInputMethodTextChanged(evt);
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
        });

        jLabel186.setFont(new java.awt.Font("Dialog", 1, 10));
        jLabel186.setText("man-days / capita / year");

        jTextYieldExpScalar2.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextYieldExpScalar2.setText("75");
        jTextYieldExpScalar2.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextYieldExpScalar2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextYieldExpScalar2ActionPerformed(evt);
            }
        });

        jLabel189.setText("%");

        jLabel159.setFont(new java.awt.Font("Lucida Grande", 1, 14));
        jLabel159.setText("WOOD GATHERING PARAMETERS");

        jLabel160.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        jLabel160.setText("Annual Amount Of Wood Required Per Capita:");

        jTextWoodPc.setText("1662.98");

        jLabel161.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        jLabel161.setText("Kg");

        jLabel162.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        jLabel162.setText("Wood Gathering Intensity:");

        jTextFieldWoodIntensity.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldWoodIntensity.setText("0.08");

        jLabel163.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        jLabel163.setText("(kg of wood per square meter)");

        jLabel176.setFont(new java.awt.Font("Lucida Grande", 0, 12));
        jLabel176.setText("Wood Distance Weight");

        jTextFieldWoodDistanceWeight.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldWoodDistanceWeight.setText("3");
        jTextFieldWoodDistanceWeight.setToolTipText("Controls the importance of distance to the wood valuation equation, can be 0 or greater");

        org.jdesktop.layout.GroupLayout jPanelWoodLayout = new org.jdesktop.layout.GroupLayout(jPanelWood);
        jPanelWood.setLayout(jPanelWoodLayout);
        jPanelWoodLayout.setHorizontalGroup(
            jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelWoodLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelWoodLayout.createSequentialGroup()
                        .add(3288, 3288, 3288)
                        .add(jLabel189))
                    .add(jPanelWoodLayout.createSequentialGroup()
                        .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelWoodLayout.createSequentialGroup()
                                .add(31, 31, 31)
                                .add(jLabel159))
                            .add(jPanelWoodLayout.createSequentialGroup()
                                .add(88, 88, 88)
                                .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelWoodLayout.createSequentialGroup()
                                        .add(jLabel160)
                                        .add(18, 18, 18)
                                        .add(jTextWoodPc, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jLabel161))
                                    .add(jPanelWoodLayout.createSequentialGroup()
                                        .add(6, 6, 6)
                                        .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jLabel176)
                                            .add(jLabel162))
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jPanelWoodLayout.createSequentialGroup()
                                                .add(jTextFieldWoodIntensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 45, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(18, 18, 18)
                                                .add(jLabel163))
                                            .add(jTextFieldWoodDistanceWeight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 45, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))))
                        .add(1731, 1731, 1731)
                        .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel186)
                            .add(jPanelWoodLayout.createSequentialGroup()
                                .add(27, 27, 27)
                                .add(jTextYieldExpScalar2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 1045, Short.MAX_VALUE)))))
                .addContainerGap())
        );
        jPanelWoodLayout.setVerticalGroup(
            jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelWoodLayout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jTextFieldWoodIntensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jLabel162)
                        .add(jLabel163))
                    .add(jPanelWoodLayout.createSequentialGroup()
                        .add(jLabel186)
                        .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelWoodLayout.createSequentialGroup()
                                .add(8, 8, 8)
                                .add(jLabel159)
                                .add(18, 18, 18)
                                .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jLabel160)
                                    .add(jTextWoodPc, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jLabel161))
                                .add(22, 22, 22)
                                .add(jLabel189))
                            .add(jPanelWoodLayout.createSequentialGroup()
                                .add(19, 19, 19)
                                .add(jTextYieldExpScalar2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(25, 25, 25)))
                .add(18, 18, 18)
                .add(jPanelWoodLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel176)
                    .add(jTextFieldWoodDistanceWeight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(528, 528, 528))
        );

        jTabbedPaneAgents.addTab("Wood Gathering", jPanelWood);

        jLabel127.setFont(new java.awt.Font("Verdana", 0, 12));
        jLabel127.setText("To Goats :");

        jLabel122.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel122.setText("Number of Ovicaprids Per Person:");

        jLabel140.setText("kg");

        jLabel125.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel125.setText("Ovicaprid Grazing Impact Factor:");

        jLabel120.setFont(new java.awt.Font("Dialog", 1, 14));
        jLabel120.setText("OVICAPRID GRAZING PARAMETERS");

        jLabel142.setText("kcal");

        jLabel137.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel137.setText("Annual Sheep Fodder Requirement");

        jLabel139.setText("kg");

        jTextFieldgrazingDensity.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldgrazingDensity.setText("4");
        jTextFieldgrazingDensity.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldgrazingDensityActionPerformed(evt);
            }
        });

        jTextFieldGoatYield.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldGoatYield.setText("38560");

        jLabel138.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel138.setText("Annual Goat Fodder Requirement");

        jLabel136.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel136.setText("Fallow Field Grazing:");

        jTextFieldnumOfOvicaprids.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldnumOfOvicaprids.setText("17");
        jTextFieldnumOfOvicaprids.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldnumOfOvicapridsActionPerformed(evt);
            }
        });

        jLabel126.setFont(new java.awt.Font("Verdana", 0, 12));
        jLabel126.setText("Sheep :");

        jTextFieldGoatFodder.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldGoatFodder.setText("894");

        jTextFieldgoatRatio.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldgoatRatio.setText("2");
        jTextFieldgoatRatio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldgoatRatioActionPerformed(evt);
            }
        });

        jLabel144.setText("kcal");

        jLabel143.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel143.setText("Annual Caloric Yield per Goat");

        jLabel123.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel123.setText("Ratio of Sheep to Goats:");

        jTextFieldsheepRatio.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldsheepRatio.setText("1");
        jTextFieldsheepRatio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldsheepRatioActionPerformed(evt);
            }
        });

        jLabel141.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel141.setText("Annual Caloric Yield per Sheep");

        jComboBoxFallFieldGrz.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "ON", "OFF" }));
        jComboBoxFallFieldGrz.setToolTipText("Sheep and Goats are able to eat the stuble from agricultural fields");

        jTextFieldSheepFodder.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSheepFodder.setText("584");
        jTextFieldSheepFodder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldSheepFodderActionPerformed(evt);
            }
        });

        jTextFieldSheepYield.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSheepYield.setText("16520");
        jTextFieldSheepYield.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldSheepYieldActionPerformed(evt);
            }
        });

        jLabel172.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel172.setText("Initial Expected Grazing Return");

        jTextFieldInitGrazingReturn.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldInitGrazingReturn.setText("250");
        jTextFieldInitGrazingReturn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldInitGrazingReturnActionPerformed(evt);
            }
        });

        jLabel173.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel173.setText("kg / ha");

        jLabel174.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel174.setText("Proportion of Ovicaprid Diet from Barley ");

        jTextFieldPctBarleyFodder.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldPctBarleyFodder.setText("10");
        jTextFieldPctBarleyFodder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldPctBarleyFodderActionPerformed(evt);
            }
        });

        jLabel175.setText("%");

        jLabel177.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel177.setText("Grazing Distance Weight");

        jTextFieldGrazingDistWight.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldGrazingDistWight.setText("1");
        jTextFieldGrazingDistWight.setToolTipText("can be 0 or greater, used in the valuation of land for grazing");
        jTextFieldGrazingDistWight.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldGrazingDistWightActionPerformed(evt);
            }
        });

        jTextFieldLcovDistWight.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldLcovDistWight.setText("1");
        jTextFieldLcovDistWight.setToolTipText("can be 0 or greater, used in the valuation of land for grazing");
        jTextFieldLcovDistWight.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldLcovDistWightActionPerformed(evt);
            }
        });

        jLabel178.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel178.setText("Landcover Distance Weight");

        jLabel182.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel182.setText("Max Cost Distance to Graze");

        jTextMaxGrazeDistance.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextMaxGrazeDistance.setText("28800");
        jTextMaxGrazeDistance.setToolTipText("Measured in Seconds Using R.Walk");
        jTextMaxGrazeDistance.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextMaxGrazeDistanceActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanelResources1Layout = new org.jdesktop.layout.GroupLayout(jPanelResources1);
        jPanelResources1.setLayout(jPanelResources1Layout);
        jPanelResources1Layout.setHorizontalGroup(
            jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelResources1Layout.createSequentialGroup()
                .add(39, 39, 39)
                .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel178)
                    .add(jLabel120)
                    .add(jPanelResources1Layout.createSequentialGroup()
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResources1Layout.createSequentialGroup()
                                .add(jLabel122, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 265, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldnumOfOvicaprids, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResources1Layout.createSequentialGroup()
                                .add(jLabel123, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 219, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel126, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 49, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(18, 18, 18)
                                .add(jTextFieldsheepRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 30, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(18, 18, 18)
                                .add(jLabel127, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 71, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelResources1Layout.createSequentialGroup()
                                    .add(jLabel172)
                                    .add(74, 74, 74)
                                    .add(jLabel173, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelResources1Layout.createSequentialGroup()
                                    .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                        .add(jPanelResources1Layout.createSequentialGroup()
                                            .add(jLabel137)
                                            .add(18, 18, 18)
                                            .add(jTextFieldSheepFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                        .add(jPanelResources1Layout.createSequentialGroup()
                                            .add(jLabel141)
                                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                            .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                .add(jTextFieldInitGrazingReturn, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextFieldSheepYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 66, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextFieldGrazingDistWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(jTextFieldLcovDistWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                        .add(jLabel140)
                                        .add(jLabel142))))
                            .add(jLabel177))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResources1Layout.createSequentialGroup()
                                .add(jLabel182)
                                .add(18, 18, 18)
                                .add(jTextMaxGrazeDistance, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 62, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResources1Layout.createSequentialGroup()
                                .add(jLabel174)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldPctBarleyFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 43, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel175))
                            .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelResources1Layout.createSequentialGroup()
                                    .add(jLabel138)
                                    .add(18, 18, 18)
                                    .add(jTextFieldGoatFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(14, 14, 14)
                                    .add(jLabel139)
                                    .add(74, 74, 74))
                                .add(jPanelResources1Layout.createSequentialGroup()
                                    .add(jTextFieldgoatRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 30, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(31, 31, 31)
                                    .add(jLabel136, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 157, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jComboBoxFallFieldGrz, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .add(jPanelResources1Layout.createSequentialGroup()
                                    .add(jLabel143)
                                    .add(18, 18, 18)
                                    .add(jTextFieldGoatYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jLabel144)))
                            .add(jPanelResources1Layout.createSequentialGroup()
                                .add(jLabel125)
                                .add(18, 18, 18)
                                .add(jTextFieldgrazingDensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 49, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap())
        );
        jPanelResources1Layout.setVerticalGroup(
            jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelResources1Layout.createSequentialGroup()
                .add(63, 63, 63)
                .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jPanelResources1Layout.createSequentialGroup()
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel125)
                            .add(jTextFieldgrazingDensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldgoatRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jComboBoxFallFieldGrz, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel136))
                        .add(18, 18, 18)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel138)
                            .add(jTextFieldGoatFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel139))
                        .add(22, 22, 22)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel143)
                            .add(jTextFieldGoatYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel144)))
                    .add(jPanelResources1Layout.createSequentialGroup()
                        .add(jLabel120)
                        .add(27, 27, 27)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel122)
                            .add(jTextFieldnumOfOvicaprids, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel123)
                            .add(jLabel126)
                            .add(jTextFieldsheepRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel127))
                        .add(18, 18, 18)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel137)
                            .add(jTextFieldSheepFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel140))
                        .add(22, 22, 22)
                        .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel141)
                            .add(jTextFieldSheepYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel142))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel172)
                    .add(jLabel173)
                    .add(jLabel175)
                    .add(jLabel174)
                    .add(jTextFieldPctBarleyFodder, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextFieldInitGrazingReturn, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel177)
                    .add(jTextFieldGrazingDistWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel182)
                    .add(jTextMaxGrazeDistance, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelResources1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel178)
                    .add(jTextFieldLcovDistWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(133, Short.MAX_VALUE))
        );

        jTabbedPaneAgents.addTab("Grazing ", jPanelResources1);

        jLabel26.setFont(new java.awt.Font("Dialog", 1, 14));
        jLabel26.setText("FARMING PARAMETERS");

        jLabel38.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel38.setText("Labor Required");

        jLabel71.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel71.setText("Initial Expected Yield");

        jLabel73.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel73.setText("(man-days/ha/year)");

        jLabel74.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel74.setText("(kg/ha/year)");

        jTextWheatLaborRqd.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextWheatLaborRqd.setText("50");

        jTextWheatInitYield.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextWheatInitYield.setText("450");

        jLabel75.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel75.setText("Calories Provided");

        jLabel76.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel76.setText("(kcal/kg)");

        jTextWheatCalories.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextWheatCalories.setText("3540");

        jLabel145.setText("WHEAT");

        jLabel146.setText("BARLEY");

        jTextBarleyLaborRqd.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextBarleyLaborRqd.setText("50");

        jTextBarleyInitYield.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextBarleyInitYield.setText("450");

        jLabel166.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel166.setText("(Barley Is Only Consumed");

        jLabel167.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel167.setText("   By Ovicaprids)");

        jLabel168.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel168.setText("Maximum Yield");

        jLabel169.setFont(new java.awt.Font("Verdana", 0, 10));
        jLabel169.setText("(kg/ha)");

        jTextFieldMaxWheatYield.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldMaxWheatYield.setText("3500");

        jTextFieldMaxBarleyYield.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldMaxBarleyYield.setText("2500");

        jLabel179.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel179.setText("Soil Fertility Weight");

        jTextFieldSoilFtyWight.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSoilFtyWight.setText("1");
        jTextFieldSoilFtyWight.setToolTipText("can be 0 or greater, used in the valuation of land for grazing");
        jTextFieldSoilFtyWight.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldSoilFtyWightActionPerformed(evt);
            }
        });

        jLabel180.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel180.setText("Soil Depth Weight");

        jTextFieldDepthWght.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldDepthWght.setText("1");
        jTextFieldDepthWght.setToolTipText("can be 0 or greater, used in the valuation of land for grazing");
        jTextFieldDepthWght.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldDepthWghtActionPerformed(evt);
            }
        });

        jLabel181.setFont(new java.awt.Font("Verdana", 1, 12)); // NOI18N
        jLabel181.setText("Farming Plot Distance Weight");

        jTextFieldFarmDistWght.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldFarmDistWght.setText("1");
        jTextFieldFarmDistWght.setToolTipText("can be 0 or greater, used in the valuation of land for grazing");
        jTextFieldFarmDistWght.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldFarmDistWghtActionPerformed(evt);
            }
        });

        jLabel184.setFont(new java.awt.Font("Verdana", 1, 12));
        jLabel184.setText("Percentage for Seed");

        jTextFieldSeedPct.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSeedPct.setText("15");
        jTextFieldSeedPct.setToolTipText("Percentage of Returns which is needed for the following year's seed");
        jTextFieldSeedPct.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldSeedPctActionPerformed(evt);
            }
        });

        jLabel185.setText("%");

        jTextFieldBpointLC1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldBpointLC1.setText("30");

        jTextFieldBpointLC2.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldBpointLC2.setText("50");

        jTextFieldBpointDA1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldBpointDA1.setText("25");

        jTextFieldBpointDA2.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldBpointDA2.setText("90");

        jLabel183.setFont(new java.awt.Font("Lucida Grande", 1, 14)); // NOI18N
        jLabel183.setText("Existing Landcover Devaluation for Farm Plot Choice");

        jTextField5.setEditable(false);
        jTextField5.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField5.setText("5");

        jTextField6.setEditable(false);
        jTextField6.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField6.setText("0");

        jLabel193.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel193.setText("Landcover Value");

        jLabel194.setFont(new java.awt.Font("Dialog", 0, 12)); // NOI18N
        jLabel194.setText("Devaluation Amount");

        jLabel195.setText("Farming Value:");

        jLabel196.setText("First Breakpoint:");

        jLabel197.setText("Final Breakpoint:");

        jComboBoxLandTenure.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "On", "Off" }));
        jComboBoxLandTenure.setSelectedIndex(1);

        jLabel198.setText("Land Tenure:");

        org.jdesktop.layout.GroupLayout jPanelResourcesLayout = new org.jdesktop.layout.GroupLayout(jPanelResources);
        jPanelResources.setLayout(jPanelResourcesLayout);
        jPanelResourcesLayout.setHorizontalGroup(
            jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelResourcesLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel179)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 119, Short.MAX_VALUE)
                                .add(jTextFieldSoilFtyWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel181)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 52, Short.MAX_VALUE)
                                .add(jTextFieldFarmDistWght, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel184)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 116, Short.MAX_VALUE)
                                .add(jTextFieldSeedPct, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel180)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 131, Short.MAX_VALUE)
                                .add(jTextFieldDepthWght, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 53, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelResourcesLayout.createSequentialGroup()
                                .add(73, 73, 73)
                                .add(jLabel198)
                                .add(18, 18, 18)
                                .add(jComboBoxLandTenure, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel185)
                        .add(107, 107, 107)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel183)
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jPanelResourcesLayout.createSequentialGroup()
                                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jLabel195)
                                            .add(jLabel196)
                                            .add(jLabel197))
                                        .add(32, 32, 32)
                                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                            .add(jTextFieldBpointLC2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 61, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelResourcesLayout.createSequentialGroup()
                                                .add(1, 1, 1)
                                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jTextField5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 60, Short.MAX_VALUE)
                                                    .add(jTextFieldBpointLC1)))))
                                    .add(jLabel193))
                                .add(86, 86, 86)
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                    .add(jTextField6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 114, Short.MAX_VALUE)
                                    .add(jLabel194, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 123, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jTextFieldBpointDA1)
                                    .add(jTextFieldBpointDA2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 102, Short.MAX_VALUE))))
                        .addContainerGap())
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(jLabel26)
                        .add(690, 690, 690))
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel145)
                            .add(jLabel146))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jTextBarleyLaborRqd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 97, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel73)
                            .add(jTextWheatLaborRqd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 97, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel38))
                        .add(27, 27, 27)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(20, 20, 20)
                                .add(jLabel74))
                            .add(jLabel71)
                            .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jTextWheatInitYield)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jTextBarleyInitYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 98, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(29, 29, 29)
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                        .add(jTextWheatCalories)
                                        .add(jLabel75, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                                    .add(jPanelResourcesLayout.createSequentialGroup()
                                        .add(20, 20, 20)
                                        .add(jLabel76))
                                    .add(jLabel167)))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(11, 11, 11)
                                .add(jLabel166)))
                        .add(45, 45, 45)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel168, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 110, Short.MAX_VALUE)
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(20, 20, 20)
                                .add(jLabel169))
                            .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jTextFieldMaxWheatYield)
                                .add(org.jdesktop.layout.GroupLayout.LEADING, jTextFieldMaxBarleyYield, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 108, Short.MAX_VALUE)))
                        .add(252, 252, 252))))
        );
        jPanelResourcesLayout.setVerticalGroup(
            jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelResourcesLayout.createSequentialGroup()
                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(22, 22, 22)
                        .add(jLabel26)
                        .add(30, 30, 30)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, jLabel145)
                                .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelResourcesLayout.createSequentialGroup()
                                    .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                        .add(jLabel71)
                                        .add(jLabel75)
                                        .add(jLabel38))
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                        .add(jLabel74)
                                        .add(jLabel76)
                                        .add(jLabel73))
                                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                    .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                        .add(jTextWheatLaborRqd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(jTextWheatCalories, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(jTextWheatInitYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel168)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel169)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldMaxWheatYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(48, 48, 48)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel146)
                            .add(jTextBarleyInitYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jTextBarleyLaborRqd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel167)))
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(181, 181, 181)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel166)
                            .add(jTextFieldMaxBarleyYield, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .add(50, 50, 50)
                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel198)
                    .add(jComboBoxLandTenure, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(34, 34, 34)
                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelResourcesLayout.createSequentialGroup()
                        .add(8, 8, 8)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldFarmDistWght, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel181))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldSeedPct, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel184)
                            .add(jLabel185))
                        .add(18, 18, 18)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel179)
                            .add(jTextFieldSoilFtyWight, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(18, 18, 18)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldDepthWght, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel180))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelResourcesLayout.createSequentialGroup()
                        .add(jLabel183)
                        .add(21, 21, 21)
                        .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel194)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextField6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldBpointDA1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(18, 18, 18)
                                .add(jTextFieldBpointDA2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelResourcesLayout.createSequentialGroup()
                                .add(jLabel193)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jTextField5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jLabel195))
                                .add(8, 8, 8)
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jTextFieldBpointLC1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jLabel196))
                                .add(18, 18, 18)
                                .add(jPanelResourcesLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(jTextFieldBpointLC2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jLabel197))))))
                .add(55, 55, 55))
        );

        jTabbedPaneAgents.addTab("Farming", jPanelResources);

        jTabbedPaneMain.addTab("Agent", jTabbedPaneAgents);

        jPanelEnvironment.setToolTipText("Initialize environment model variables");
        jPanelEnvironment.setName("environment_frame"); // NOI18N

        jLabel21.setText("GRASS ENVIRONMENT SETUP");

        jLabel17.setText("Root Directory (GISDBASE):");

        jTextGISDBASE.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        jTextGISDBASE.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextGISDBASEActionPerformed(evt);
            }
        });

        jButtonBrowseRootDir.setText("Browse...");
        jButtonBrowseRootDir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBrowseRootDirActionPerformed(evt);
            }
        });

        jComboBoxLocation.setMaximumRowCount(20);
        jComboBoxLocation.setEnabled(false);
        jComboBoxLocation.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBoxLocationItemStateChanged(evt);
            }
        });

        jLabel18.setText("Location:");

        jTextMapset.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextMapset.setEnabled(false);

        jLabel19.setText("Mapset:");

        jButtonGenerateMapset.setText("Generate");
        jButtonGenerateMapset.setEnabled(false);
        jButtonGenerateMapset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGenerateMapsetActionPerformed(evt);
            }
        });

        jLabel45.setText("INITIALIZATION MAPS (in [Location]\\PERMANENT mapset)");

        jLabel22.setText("DEM:");

        jComboBoxDEM.setToolTipText("(Used for elevation and default region settings)");
        jComboBoxDEM.setEnabled(false);

        jLabel48.setText("Land Cover:");

        jComboBoxLandCover.setEnabled(false);

        jLabel51.setText("Friction (choose one):");

        jRadioButtonFrictionValue.setText("Constant Value");
        jRadioButtonFrictionValue.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionValue.setEnabled(false);
        jRadioButtonFrictionValue.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionValueActionPerformed(evt);
            }
        });

        jTextFrictionValue.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextFrictionValue.setText("1.0");
        jTextFrictionValue.setEnabled(false);

        jLabel47.setText("(Used for distance cost calculations)");

        jRadioButtonFrictionMap.setText("Map");
        jRadioButtonFrictionMap.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionMap.setEnabled(false);
        jRadioButtonFrictionMap.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionMap.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionMapActionPerformed(evt);
            }
        });

        jComboBoxFrictionMap.setEnabled(false);

        jLabel49.setText("Soil K-Factor (choose one):");

        jRadioButtonSoilKValue.setText("Constant Value");
        jRadioButtonSoilKValue.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKValue.setEnabled(false);
        jRadioButtonSoilKValue.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKValue.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKValueActionPerformed(evt);
            }
        });

        jTextSoilKValue.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextSoilKValue.setText("0.42");
        jTextSoilKValue.setEnabled(false);

        jLabel50.setText("(Used for erosion calculations)");

        jRadioButtonSoilKMap.setText("Map");
        jRadioButtonSoilKMap.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKMap.setEnabled(false);
        jRadioButtonSoilKMap.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKMap.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKMapActionPerformed(evt);
            }
        });

        jComboBoxSoilKMap.setEnabled(false);

        jLabel60.setText("NOTES");

        jLabel43.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel43.setText("     Region data will be read from DEM file.           Ensure that none of the maps above contains the phrase \"temp.\"");

        jLabel67.setText("Fertility:");

        jComboBoxFertility.setEnabled(false);

        jComboBoxBedrock.setToolTipText("If you do not select a bedrock map, one will be created for you using r.soildepth automatically");
        jComboBoxBedrock.setEnabled(false);

        jLabel147.setText("Bedrock (optional):");

        jTextFieldMaxLcov.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldMaxLcov.setText("50");
        jTextFieldMaxLcov.setToolTipText("Maximum Soil Depth used during the creation of a soil map in r.Soildepth");

        jLabel190.setText("Maximum Landcover Value");

        jComboBoxMaxLandCover.setToolTipText("If a map is selected, it will override the numerical value");
        jComboBoxMaxLandCover.setEnabled(false);

        jLabel156.setText("Maximum Landcover (optional):");

        org.jdesktop.layout.GroupLayout jPanelMapsLayout = new org.jdesktop.layout.GroupLayout(jPanelMaps);
        jPanelMaps.setLayout(jPanelMapsLayout);
        jPanelMapsLayout.setHorizontalGroup(
            jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelMapsLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .add(jLabel21)
                        .addContainerGap(637, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelMapsLayout.createSequentialGroup()
                        .add(24, 24, 24)
                        .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .add(jLabel17)
                                .add(10, 10, 10))
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jLabel19)
                                    .add(jLabel18))
                                .add(18, 18, 18)))
                        .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jComboBoxLocation, 0, 369, Short.MAX_VALUE)
                            .add(jTextGISDBASE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE)
                            .add(jTextMapset, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 369, Short.MAX_VALUE))
                        .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jButtonBrowseRootDir))
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .add(1, 1, 1)
                                .add(jButtonGenerateMapset)))
                        .add(137, 137, 137))))
            .add(jPanelMapsLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel45)
                .addContainerGap(455, Short.MAX_VALUE))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelMapsLayout.createSequentialGroup()
                .addContainerGap(128, Short.MAX_VALUE)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jLabel48)
                    .add(jLabel22))
                .add(18, 18, 18)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jComboBoxLandCover, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jComboBoxDEM, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 208, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(jLabel67))
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(jLabel147)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jComboBoxBedrock, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 228, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxFertility, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 228, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(40, 40, 40))
            .add(jPanelMapsLayout.createSequentialGroup()
                .add(30, 30, 30)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .add(jLabel190)
                        .add(18, 18, 18)
                        .add(jTextFieldMaxLcov, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 42, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel51)
                            .add(jLabel49))
                        .add(18, 18, 18)
                        .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .add(jRadioButtonSoilKValue)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jTextSoilKValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 78, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(27, 27, 27)
                                .add(jLabel50))
                            .add(jPanelMapsLayout.createSequentialGroup()
                                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                    .add(jRadioButtonFrictionMap, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .add(jRadioButtonFrictionValue, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, jRadioButtonSoilKMap))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelMapsLayout.createSequentialGroup()
                                        .add(jTextFrictionValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jLabel47))
                                    .add(jComboBoxSoilKMap, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 181, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jComboBoxFrictionMap, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 184, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jPanelMapsLayout.createSequentialGroup()
                                        .add(jLabel156)
                                        .add(29, 29, 29)
                                        .add(jComboBoxMaxLandCover, 0, 208, Short.MAX_VALUE))))))
                    .add(jPanelMapsLayout.createSequentialGroup()
                        .add(jLabel60)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel43)))
                .addContainerGap(57, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        jPanelMapsLayout.setVerticalGroup(
            jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelMapsLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel21)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel17)
                    .add(jTextGISDBASE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButtonBrowseRootDir))
                .add(18, 18, 18)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxLocation, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel18))
                .add(18, 18, 18)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel19)
                    .add(jTextMapset, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButtonGenerateMapset))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jLabel45)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel22)
                    .add(jComboBoxDEM, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxBedrock, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel147))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel48)
                    .add(jComboBoxLandCover, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel67)
                    .add(jComboBoxFertility, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel51)
                    .add(jRadioButtonFrictionValue)
                    .add(jTextFrictionValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel47))
                .add(18, 18, 18)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jRadioButtonFrictionMap)
                    .add(jComboBoxFrictionMap, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel50)
                    .add(jRadioButtonSoilKValue)
                    .add(jTextSoilKValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel49))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jRadioButtonSoilKMap)
                    .add(jComboBoxSoilKMap, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(9, 9, 9)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel190, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 16, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextFieldMaxLcov, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxMaxLandCover, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel156))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanelMapsLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel43)
                    .add(jLabel60))
                .addContainerGap())
        );

        jTabbedPaneEnv.addTab("Map Selection", jPanelMaps);

        jLabel150.setText("CLIMATE VALUES SETUP");

        jTextFieldrFactor.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldrFactor.setText("5.66");
        jTextFieldrFactor.setToolTipText("Rainfall (R factor) constant (AVERAGE FOR WHOLE MAP AREA). Must be between 1- 25.");

        jLabel92.setText("R Factor:");
        jLabel92.setToolTipText("Rainfall (R factor) constant (AVERAGE FOR WHOLE MAP AREA). Must be between 1- 25.");

        jLabel99.setText("Annual Precipitation in Meters:");
        jLabel99.setToolTipText("Anuall precip totals map or constant (in meters per year) Reasonable range 0-5 m.");

        jTextFieldrainValue.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldrainValue.setText("0.5");
        jTextFieldrainValue.setToolTipText("Anuall precip totals map or constant (in meters per year) Reasonable range 0-5 m.");

        jTextFieldStormEvents.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldStormEvents.setText("75");
        jTextFieldStormEvents.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jLabel100.setText("Storm Events per Year:");
        jLabel100.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jLabel151.setText("Static Climate Values:");

        jLabel152.setText("Climate File Location:");

        jTextFieldClimFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldClimFileActionPerformed(evt);
            }
        });

        jButtonClimFile.setText("Browse");
        jButtonClimFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonClimFileActionPerformed(evt);
            }
        });

        jLabel153.setText("Note: You may input climate values below that will be used in each year of the simulation, ");

        jComboBoxClimateSource.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Use Static Input Values", "Read Climate Variables From File" }));

        jLabel154.setText("or, you may choose a  *.csv file which contains unique values for each year the simulation.");

        jLabel155.setText("Dynamic Climate Values:");

        jLabel164.setText("Note That Values In The File Should Be In The Following Format:");

        jLabel165.setFont(new java.awt.Font("Lucida Grande", 2, 13));
        jLabel165.setText("Storm/Yr, R Factor, Annual Precip (m), Storm Speed, Storm Length, Storm Precip (mm)");

        jLabel111.setText("Storm Speed:");
        jLabel111.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jTextFieldStormSpeed.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldStormSpeed.setText("1.4");
        jTextFieldStormSpeed.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jLabel187.setText("Storm Length:");
        jLabel187.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jTextFieldStormLength.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldStormLength.setText("6.0");
        jTextFieldStormLength.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jLabel188.setText("Storm Precip in (mm):");
        jLabel188.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        jTextFieldStormPrecip.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldStormPrecip.setText("10.0");
        jTextFieldStormPrecip.setToolTipText("Number of days of rain per year (integer). Values must range between 0-365 days per year");

        org.jdesktop.layout.GroupLayout jPanelMaps1Layout = new org.jdesktop.layout.GroupLayout(jPanelMaps1);
        jPanelMaps1.setLayout(jPanelMaps1Layout);
        jPanelMaps1Layout.setHorizontalGroup(
            jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelMaps1Layout.createSequentialGroup()
                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelMaps1Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel150))
                    .add(jPanelMaps1Layout.createSequentialGroup()
                        .add(78, 78, 78)
                        .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jComboBoxClimateSource, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel154)
                            .add(jLabel153)))
                    .add(jPanelMaps1Layout.createSequentialGroup()
                        .add(88, 88, 88)
                        .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel164)
                            .add(jPanelMaps1Layout.createSequentialGroup()
                                .add(jLabel152)
                                .add(18, 18, 18)
                                .add(jTextFieldClimFile, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 309, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .add(18, 18, 18)
                                .add(jButtonClimFile))
                            .add(jPanelMaps1Layout.createSequentialGroup()
                                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jPanelMaps1Layout.createSequentialGroup()
                                        .add(jLabel111)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jTextFieldStormSpeed, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 82, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(jPanelMaps1Layout.createSequentialGroup()
                                        .add(jLabel100)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jTextFieldStormEvents, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 82, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                                .add(18, 18, 18)
                                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                    .add(jPanelMaps1Layout.createSequentialGroup()
                                        .add(jLabel92)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jTextFieldrFactor, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 58, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(35, 35, 35)
                                        .add(jLabel99)
                                        .add(18, 18, 18)
                                        .add(jTextFieldrainValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 57, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(jPanelMaps1Layout.createSequentialGroup()
                                        .add(jLabel187)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jTextFieldStormLength, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 82, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                        .add(jLabel188)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jTextFieldStormPrecip, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 82, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                            .add(jLabel165)))
                    .add(jPanelMaps1Layout.createSequentialGroup()
                        .add(54, 54, 54)
                        .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel151)
                            .add(jLabel155))))
                .addContainerGap(63, Short.MAX_VALUE))
        );
        jPanelMaps1Layout.setVerticalGroup(
            jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelMaps1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel150)
                .add(31, 31, 31)
                .add(jLabel153)
                .add(3, 3, 3)
                .add(jLabel154)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jComboBoxClimateSource, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(63, 63, 63)
                .add(jLabel151)
                .add(18, 18, 18)
                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel100)
                    .add(jTextFieldStormEvents, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel92)
                    .add(jTextFieldrFactor, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel99)
                    .add(jTextFieldrainValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel111)
                    .add(jTextFieldStormSpeed, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel187)
                    .add(jTextFieldStormLength, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextFieldStormPrecip, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel188))
                .add(39, 39, 39)
                .add(jLabel155)
                .add(18, 18, 18)
                .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelMaps1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jTextFieldClimFile, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jButtonClimFile))
                    .add(jPanelMaps1Layout.createSequentialGroup()
                        .add(7, 7, 7)
                        .add(jLabel152)))
                .add(33, 33, 33)
                .add(jLabel164)
                .add(18, 18, 18)
                .add(jLabel165)
                .add(45, 45, 45))
        );

        jTabbedPaneEnv.addTab("Climate Settings", jPanelMaps1);

        jPanelLandEvol.setToolTipText("Initialization human model variables");
        jPanelLandEvol.setName("agent_frame"); // NOI18N
        jPanelLandEvol.addInputMethodListener(new java.awt.event.InputMethodListener() {
            public void inputMethodTextChanged(java.awt.event.InputMethodEvent evt) {
                jPanelLandEvolInputMethodTextChanged(evt);
            }
            public void caretPositionChanged(java.awt.event.InputMethodEvent evt) {
            }
        });

        jLabel121.setFont(new java.awt.Font("Dialog", 1, 10));
        jLabel121.setText("man-days / capita / year");

        jTextYieldExpScalar1.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextYieldExpScalar1.setText("75");
        jTextYieldExpScalar1.setToolTipText("Estimation of next year's potential yield to current year's actual yield.");
        jTextYieldExpScalar1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextYieldExpScalar1ActionPerformed(evt);
            }
        });

        jLabel124.setText("%");

        jLabel64.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel64.setText("Region data will be read from DEM file.");

        jLabel81.setText("NOTES");

        jComboBoxSoilKMap1.setEnabled(false);

        jRadioButtonSoilKMap1.setText("Map");
        jRadioButtonSoilKMap1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKMap1.setEnabled(false);
        jRadioButtonSoilKMap1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKMap1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKMap1ActionPerformed(evt);
            }
        });

        jRadioButtonSoilKValue1.setText("Constant Value");
        jRadioButtonSoilKValue1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKValue1.setEnabled(false);
        jRadioButtonSoilKValue1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKValue1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKValue1ActionPerformed(evt);
            }
        });

        jLabel86.setText("Soil K-Factor (choose one):");

        jTextSoilKValue1.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextSoilKValue1.setText("0.25");
        jTextSoilKValue1.setEnabled(false);

        jComboBoxFrictionMap1.setEnabled(false);

        jRadioButtonFrictionMap1.setText("Map");
        jRadioButtonFrictionMap1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionMap1.setEnabled(false);
        jRadioButtonFrictionMap1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionMap1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionMap1ActionPerformed(evt);
            }
        });

        jRadioButtonFrictionValue1.setText("Constant Value");
        jRadioButtonFrictionValue1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionValue1.setEnabled(false);
        jRadioButtonFrictionValue1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionValue1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionValue1ActionPerformed(evt);
            }
        });

        jLabel87.setText("Friction (choose one):");

        jLabel88.setText("Land Cover:");

        jComboBoxLandCover1.setEnabled(false);

        jTextFrictionValue1.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextFrictionValue1.setText("1.0");
        jTextFrictionValue1.setEnabled(false);

        jComboBoxDEM1.setEnabled(false);

        jLabel89.setText("DEM:");

        jLabel93.setText("INITIALIZATION MAPS (in [Location]\\PERMANENT mapset)");

        jTextMapset1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextMapset1.setEnabled(false);

        jButtonGenerateMapset1.setText("Generate");
        jButtonGenerateMapset1.setEnabled(false);
        jButtonGenerateMapset1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGenerateMapset1ActionPerformed(evt);
            }
        });

        jComboBoxLocation1.setMaximumRowCount(20);
        jComboBoxLocation1.setEnabled(false);
        jComboBoxLocation1.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBoxLocation1ItemStateChanged(evt);
            }
        });

        jTextGISDBASE1.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        jTextGISDBASE1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextGISDBASE1ActionPerformed(evt);
            }
        });

        jButtonBrowseRootDir1.setText("Browse...");
        jButtonBrowseRootDir1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBrowseRootDir1ActionPerformed(evt);
            }
        });

        jLabel94.setText("Root Directory (GISDBASE):");

        jLabel95.setText("Location:");

        jLabel96.setText("Mapset:");

        jLabel97.setText("GRASS ENVIRONMENT SETUP");

        jLabel98.setFont(new java.awt.Font("Dialog", 0, 12));
        jLabel98.setText("Region data will be read from DEM file.");

        jLabel101.setText("NOTES");

        jComboBoxSoilKMap2.setEnabled(false);

        jRadioButtonSoilKMap2.setText("Map");
        jRadioButtonSoilKMap2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKMap2.setEnabled(false);
        jRadioButtonSoilKMap2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKMap2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKMap2ActionPerformed(evt);
            }
        });

        jRadioButtonSoilKValue2.setSelected(true);
        jRadioButtonSoilKValue2.setText("Constant Value");
        jRadioButtonSoilKValue2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonSoilKValue2.setEnabled(false);
        jRadioButtonSoilKValue2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonSoilKValue2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonSoilKValue2ActionPerformed(evt);
            }
        });

        jLabel102.setText("Soil K-Factor (choose one):");

        jTextSoilKValue2.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextSoilKValue2.setText("0.25");
        jTextSoilKValue2.setEnabled(false);

        jComboBoxFrictionMap2.setEnabled(false);

        jRadioButtonFrictionMap2.setText("Map");
        jRadioButtonFrictionMap2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionMap2.setEnabled(false);
        jRadioButtonFrictionMap2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionMap2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionMap2ActionPerformed(evt);
            }
        });

        jRadioButtonFrictionValue2.setSelected(true);
        jRadioButtonFrictionValue2.setText("Constant Value");
        jRadioButtonFrictionValue2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButtonFrictionValue2.setEnabled(false);
        jRadioButtonFrictionValue2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButtonFrictionValue2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButtonFrictionValue2ActionPerformed(evt);
            }
        });

        jLabel103.setText("Friction (choose one):");

        jLabel104.setText("Land Cover:");

        jComboBoxLandCover2.setEnabled(false);

        jTextFrictionValue2.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextFrictionValue2.setText("1.0");
        jTextFrictionValue2.setEnabled(false);

        jComboBoxDEM2.setEnabled(false);

        jLabel105.setText("DEM:");

        jLabel106.setText("INITIALIZATION MAPS (in [Location]\\PERMANENT mapset)");

        jTextMapset2.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextMapset2.setEnabled(false);

        jButtonGenerateMapset2.setText("Generate");
        jButtonGenerateMapset2.setEnabled(false);
        jButtonGenerateMapset2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGenerateMapset2ActionPerformed(evt);
            }
        });

        jComboBoxLocation2.setMaximumRowCount(20);
        jComboBoxLocation2.setEnabled(false);
        jComboBoxLocation2.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jComboBoxLocation2ItemStateChanged(evt);
            }
        });

        jTextGISDBASE2.setHorizontalAlignment(javax.swing.JTextField.TRAILING);
        jTextGISDBASE2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextGISDBASE2ActionPerformed(evt);
            }
        });

        jButtonBrowseRootDir2.setText("Browse...");
        jButtonBrowseRootDir2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonBrowseRootDir2ActionPerformed(evt);
            }
        });

        jLabel107.setText("Root Directory (GISDBASE):");

        jLabel108.setText("Location:");

        jLabel109.setText("Mapset:");

        jLabel110.setText("GRASS ENVIRONMENT SETUP");

        jLabel90.setText("LANDSCAPE EVOL PARAMATERS");

        jLabel91.setText("Soil Density");
        jLabel91.setToolTipText("Soil density constant (for conversion from mass to volume). Must be greater than 0, and less than 5.");

        jTextFieldsoilDensity.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldsoilDensity.setText("1.25");
        jTextFieldsoilDensity.setToolTipText("Soil density constant (for conversion from mass to volume). Must be greater than 0, and less than 5.");
        jTextFieldsoilDensity.setMaximumSize(new java.awt.Dimension(500, 500));

        jLabel112.setText("Stream Transport");
        jLabel112.setToolTipText("Stream transport efficiency variable (0.001 for a normal substrate (ie. sediment) to 0.000001 for a very hard substrate like bedrock) Value must be one of the following: 0.001, 0.0001, 0.00001, 0.00001, 0.000001.");

        jTextFieldstreamTrans.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldstreamTrans.setText("0.0001");
        jTextFieldstreamTrans.setToolTipText("Stream transport efficiency variable (0.001 for a normal substrate (ie. sediment) to 0.000001 for a very hard substrate like bedrock) Value must be one of the following: 0.001, 0.0001, 0.00001, 0.00001, 0.000001.");

        jLabel113.setText("Load Exponent");
        jLabel113.setToolTipText("Stream transport type varaible (1.5 for mainly bedload transport, 2.5 for mainly suspended load transport) ");

        jTextFieldloadExponent.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldloadExponent.setText("1.5");
        jTextFieldloadExponent.setToolTipText("Stream transport type varaible (1.5 for mainly bedload transport, 2.5 for mainly suspended load transport) ");
        jTextFieldloadExponent.setMaximumSize(new java.awt.Dimension(500, 500));

        jLabel115.setText("Kappa");
        jLabel115.setToolTipText("Hillslope diffusion (Kappa) rate map or constant (meters per kiloyear)  Range: 0-10");

        jTextFieldkappa.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldkappa.setText("1");
        jTextFieldkappa.setToolTipText("Hillslope diffusion (Kappa) rate map or constant (meters per kiloyear)  Range: 0-10");
        jTextFieldkappa.setMaximumSize(new java.awt.Dimension(500, 500));

        jLabel116.setText("cutoff 1");
        jLabel116.setToolTipText("Flow accumultion breakpoint value for shift from diffusion to overland flow (sq. m uplsope area) ");

        jTextFieldcutoff1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldcutoff1.setText("0.6");
        jTextFieldcutoff1.setToolTipText("Flow accumultion breakpoint value for shift from diffusion to overland flow (sq. m uplsope area) ");
        jTextFieldcutoff1.setMaximumSize(new java.awt.Dimension(500, 500));

        jTextFieldcutoff2.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldcutoff2.setText("2.5");
        jTextFieldcutoff2.setToolTipText("Flow accumultion breakpoint value for shift from overland flow to rill/gully flow (sq. m uplsope area) ");
        jTextFieldcutoff2.setMaximumSize(new java.awt.Dimension(500, 500));

        jLabel117.setText("cutoff 2");
        jLabel117.setToolTipText("Flow accumultion breakpoint value for shift from overland flow to rill/gully flow (sq. m uplsope area) ");

        jTextFieldcutoff3.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldcutoff3.setText("7");
        jTextFieldcutoff3.setToolTipText("Flow accumultion breakpoint value for shift from rill/gully flow to stream flow (sq. m uplsope area) ");
        jTextFieldcutoff3.setMaximumSize(new java.awt.Dimension(500, 500));

        jLabel118.setText("cutoff 3");
        jLabel118.setToolTipText("Flow accumultion breakpoint value for shift from rill/gully flow to stream flow (sq. m uplsope area) ");

        jLabel119.setText("Note: Allow your cursor to hover over any variable for more information");

        jLabel114.setText("Smoothing");

        jComboBoxSmoothing.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Low", "High", "None" }));
        jComboBoxSmoothing.setSelectedIndex(2);

        jLabel131.setText("LANDSCAPE PARAMETERS");

        jLabel132.setText("Soil Depth Minimum");

        jLabel133.setText("Soil Depth Maximum");

        jTextFieldSoilMin.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSoilMin.setText("0.01");
        jTextFieldSoilMin.setToolTipText("Minimum Soil Depth used during the creation of a soil map in r.Soildepth");
        jTextFieldSoilMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldSoilMinActionPerformed(evt);
            }
        });

        jTextFieldSoilMax.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSoilMax.setText("20");
        jTextFieldSoilMax.setToolTipText("Maximum Soil Depth used during the creation of a soil map in r.Soildepth");

        jLabel134.setText("Soil Fertiliy Recovery");

        jTextFieldSoilRecovery.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldSoilRecovery.setText("1");
        jTextFieldSoilRecovery.setToolTipText("Annual amount of fertility recovered");

        jLabel135.setText("Soil Fertility Impact");

        jTextFieldFertilityImpact.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextFieldFertilityImpact.setText("2");
        jTextFieldFertilityImpact.setToolTipText("Annual Impact of Farming On Soil fertility");

        org.jdesktop.layout.GroupLayout jPanelLandEvolLayout = new org.jdesktop.layout.GroupLayout(jPanelLandEvol);
        jPanelLandEvol.setLayout(jPanelLandEvolLayout);
        jPanelLandEvolLayout.setHorizontalGroup(
            jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelLandEvolLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelLandEvolLayout.createSequentialGroup()
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel90)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(77, 77, 77)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jLabel116)
                                    .add(jLabel91))
                                .add(8, 8, 8)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jTextFieldcutoff1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 54, Short.MAX_VALUE)
                                    .add(jTextFieldsoilDensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                        .add(14, 14, 14)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(jLabel117)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldcutoff2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jPanelLandEvolLayout.createSequentialGroup()
                                .add(jLabel113)
                                .add(2, 2, 2)
                                .add(jTextFieldloadExponent, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 55, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(27, 27, 27)
                                .add(jLabel118)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldcutoff3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 77, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(40, 40, 40)
                                .add(jLabel115)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldkappa, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jLabel112)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextFieldstreamTrans, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 58, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(40, 40, 40)
                                .add(jLabel114)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jComboBoxSmoothing, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(108, 108, 108)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 943, Short.MAX_VALUE)
                                .add(jLabel124))
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(1453, 1453, 1453)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jLabel121)
                                    .add(jPanelLandEvolLayout.createSequentialGroup()
                                        .add(27, 27, 27)
                                        .add(jTextYieldExpScalar1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 925, Short.MAX_VALUE))))))
                    .add(jLabel131)
                    .add(jPanelLandEvolLayout.createSequentialGroup()
                        .add(209, 209, 209)
                        .add(jTextFieldSoilMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(18, 18, 18)
                        .add(jLabel133)
                        .add(18, 18, 18)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jTextFieldSoilRecovery, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jTextFieldSoilMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 42, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanelLandEvolLayout.createSequentialGroup()
                        .add(67, 67, 67)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel132)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(jLabel135)
                                .add(18, 18, 18)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelLandEvolLayout.createSequentialGroup()
                                        .add(jTextFieldFertilityImpact, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 48, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(18, 18, 18)
                                        .add(jLabel134))
                                    .add(jLabel119))))))
                .addContainerGap())
            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(jPanelLandEvolLayout.createSequentialGroup()
                    .add(1347, 1347, 1347)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanelLandEvolLayout.createSequentialGroup()
                            .add(jLabel97)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 423, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelLandEvolLayout.createSequentialGroup()
                                        .add(22, 22, 22)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(jLabel95)
                                            .add(jLabel96)
                                            .add(jLabel94))
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                            .add(jTextGISDBASE1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE)
                                            .add(jComboBoxLocation1, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                            .add(jTextMapset1)))
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(jLabel88)
                                            .add(jLabel89)
                                            .add(jLabel87)
                                            .add(jLabel86))
                                        .add(32, 32, 32)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jComboBoxDEM1, 0, 291, Short.MAX_VALUE)
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonSoilKMap1)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jComboBoxSoilKMap1, 0, 239, Short.MAX_VALUE))
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonSoilKValue1)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 96, Short.MAX_VALUE)
                                                .add(jTextSoilKValue1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 78, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonFrictionMap1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jComboBoxFrictionMap1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 239, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonFrictionValue1)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 95, Short.MAX_VALUE)
                                                .add(jTextFrictionValue1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jComboBoxLandCover1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 290, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jButtonBrowseRootDir1)
                                    .add(jButtonGenerateMapset1)))
                            .add(jLabel93))
                        .add(jPanelLandEvolLayout.createSequentialGroup()
                            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jLabel81)
                                .add(jPanelLandEvolLayout.createSequentialGroup()
                                    .add(12, 12, 12)
                                    .add(jLabel64)))
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 363, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(1348, 1348, 1348)))
            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                    .add(1357, 1357, 1357)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanelLandEvolLayout.createSequentialGroup()
                            .add(jLabel110)
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 423, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelLandEvolLayout.createSequentialGroup()
                                        .add(22, 22, 22)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(jLabel108)
                                            .add(jLabel109)
                                            .add(jLabel107))
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                            .add(jTextGISDBASE2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE)
                                            .add(jComboBoxLocation2, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                            .add(jTextMapset2)))
                                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(jLabel104)
                                            .add(jLabel105)
                                            .add(jLabel103)
                                            .add(jLabel102))
                                        .add(32, 32, 32)
                                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jComboBoxDEM2, 0, 291, Short.MAX_VALUE)
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonSoilKMap2)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jComboBoxSoilKMap2, 0, 239, Short.MAX_VALUE))
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonSoilKValue2)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 96, Short.MAX_VALUE)
                                                .add(jTextSoilKValue2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 78, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonFrictionMap2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jComboBoxFrictionMap2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 239, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                                .add(jRadioButtonFrictionValue2)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 95, Short.MAX_VALUE)
                                                .add(jTextFrictionValue2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jComboBoxLandCover2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 290, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jButtonBrowseRootDir2)
                                    .add(jButtonGenerateMapset2)))
                            .add(jLabel106))
                        .add(jPanelLandEvolLayout.createSequentialGroup()
                            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jLabel101)
                                .add(jPanelLandEvolLayout.createSequentialGroup()
                                    .add(12, 12, 12)
                                    .add(jLabel98)))
                            .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 363, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(1338, 1338, 1338)))
        );
        jPanelLandEvolLayout.setVerticalGroup(
            jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelLandEvolLayout.createSequentialGroup()
                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelLandEvolLayout.createSequentialGroup()
                        .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(jLabel121)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(80, 80, 80)
                                .add(jLabel124))
                            .add(jPanelLandEvolLayout.createSequentialGroup()
                                .add(19, 19, 19)
                                .add(jTextYieldExpScalar1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(151, 151, 151))
                    .add(jPanelLandEvolLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel90)
                        .add(19, 19, 19)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel91)
                            .add(jTextFieldsoilDensity, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jTextFieldloadExponent, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel114)
                            .add(jLabel113)
                            .add(jTextFieldkappa, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel115)
                            .add(jComboBoxSmoothing, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(29, 29, 29)
                        .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel116)
                            .add(jTextFieldcutoff1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel117)
                            .add(jTextFieldcutoff2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jTextFieldcutoff3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jTextFieldstreamTrans, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel118)
                            .add(jLabel112))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel131)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel132)
                    .add(jTextFieldSoilMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel133, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 16, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jTextFieldSoilMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(18, 18, 18)
                .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel135)
                    .add(jTextFieldFertilityImpact, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel134)
                    .add(jTextFieldSoilRecovery, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(41, 41, 41)
                .add(jLabel119)
                .add(180, 180, 180))
            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(jPanelLandEvolLayout.createSequentialGroup()
                    .add(129, 129, 129)
                    .add(jLabel97)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel94)
                        .add(jTextGISDBASE1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jButtonBrowseRootDir1))
                    .add(6, 6, 6)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jLabel95)
                        .add(jComboBoxLocation1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel96)
                        .add(jTextMapset1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jButtonGenerateMapset1))
                    .add(40, 40, 40)
                    .add(jLabel93)
                    .add(6, 6, 6)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel89)
                        .add(jComboBoxDEM1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel88)
                        .add(jComboBoxLandCover1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel87)
                        .add(jRadioButtonFrictionValue1)
                        .add(jTextFrictionValue1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jComboBoxFrictionMap1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jRadioButtonFrictionMap1))
                    .add(11, 11, 11)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel86)
                        .add(jRadioButtonSoilKValue1)
                        .add(jTextSoilKValue1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jRadioButtonSoilKMap1)
                        .add(jComboBoxSoilKMap1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(23, 23, 23)
                    .add(jLabel81)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jLabel64)
                    .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
            .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelLandEvolLayout.createSequentialGroup()
                    .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jLabel110)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel107)
                        .add(jTextGISDBASE2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jButtonBrowseRootDir2))
                    .add(6, 6, 6)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jLabel108)
                        .add(jComboBoxLocation2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel109)
                        .add(jTextMapset2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jButtonGenerateMapset2))
                    .add(40, 40, 40)
                    .add(jLabel106)
                    .add(6, 6, 6)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel105)
                        .add(jComboBoxDEM2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel104)
                        .add(jComboBoxLandCover2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel103)
                        .add(jRadioButtonFrictionValue2)
                        .add(jTextFrictionValue2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jComboBoxFrictionMap2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jRadioButtonFrictionMap2))
                    .add(11, 11, 11)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel102)
                        .add(jRadioButtonSoilKValue2)
                        .add(jTextSoilKValue2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jPanelLandEvolLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jRadioButtonSoilKMap2)
                        .add(jComboBoxSoilKMap2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(23, 23, 23)
                    .add(jLabel101)
                    .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                    .add(jLabel98)
                    .add(120, 120, 120)))
        );

        jTabbedPaneEnv.addTab("Landscape Values", jPanelLandEvol);

        org.jdesktop.layout.GroupLayout jPanelEnvironmentLayout = new org.jdesktop.layout.GroupLayout(jPanelEnvironment);
        jPanelEnvironment.setLayout(jPanelEnvironmentLayout);
        jPanelEnvironmentLayout.setHorizontalGroup(
            jPanelEnvironmentLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelEnvironmentLayout.createSequentialGroup()
                .addContainerGap()
                .add(jTabbedPaneEnv, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 857, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(29, Short.MAX_VALUE))
        );
        jPanelEnvironmentLayout.setVerticalGroup(
            jPanelEnvironmentLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanelEnvironmentLayout.createSequentialGroup()
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(jTabbedPaneEnv, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 557, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(57, 57, 57))
        );

        jTabbedPaneEnv.getAccessibleContext().setAccessibleName("Climate Settings");

        jTabbedPaneMain.addTab("Environment Model", jPanelEnvironment);

        jPanelIM.setToolTipText("Initialize interaction model parameters");
        jPanelIM.setName("interaction_frame"); // NOI18N

        jLabel52.setText("TIMING");

        jLabel53.setText("Start Date:");

        jLabel54.setText("time by");

        jComboBoxDateChangeDir.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Increment", "Decrement" }));

        jLabel55.setText("End Date:");

        jLabel56.setText("Date Prefix:");

        jLabel57.setText("Date Suffix:");

        jLabelFullMapName.setText("Full map name will be y###_[MAP NAME]");

        jTextDateChangeValue.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextDateChangeValue.setText("1");

        jLabel59.setText("each simulation cycle");

        jTextDateEnd.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextDateEnd.setText("501");
        jTextDateEnd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextDateEndActionPerformed(evt);
            }
        });

        jTextDatePrefix.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextDatePrefix.setText("y");
        jTextDatePrefix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextDatePrefixActionPerformed(evt);
            }
        });

        jTextDateSuffix.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextDateSuffix.setText("_");
        jTextDateSuffix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextDateSuffixActionPerformed(evt);
            }
        });

        jLabel58.setText("SCALE (region grid resolution)");

        jComboBoxScale.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "1 : 1", "1 : 4", "1 : 9" }));
        jComboBoxScale.setToolTipText("Specify the scale ratio.");
        jComboBoxScale.setEnabled(false);

        jComboBoxRatio.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Agent : Environment", "Environment : Agent" }));
        jComboBoxRatio.setToolTipText("Specify whether the scale ratio is agent-to-environment or environment-to-agent.");
        jComboBoxRatio.setEnabled(false);

        jLabel149.setText("The start date is always 0.");

        jTextDateStart.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextDateStart.setText("0");
        jTextDateStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextDateStartActionPerformed(evt);
            }
        });

        jLabel157.setText("Start Date:");

        org.jdesktop.layout.GroupLayout jPanelIMLayout = new org.jdesktop.layout.GroupLayout(jPanelIM);
        jPanelIM.setLayout(jPanelIMLayout);
        jPanelIMLayout.setHorizontalGroup(
            jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelIMLayout.createSequentialGroup()
                .addContainerGap()
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel52)
                    .add(jPanelIMLayout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jPanelIMLayout.createSequentialGroup()
                                .add(jComboBoxDateChangeDir, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel54)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jTextDateChangeValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel59))
                            .add(jPanelIMLayout.createSequentialGroup()
                                .add(jLabel53)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .add(jLabel149)
                                .add(137, 137, 137))
                            .add(jPanelIMLayout.createSequentialGroup()
                                .add(12, 12, 12)
                                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jLabelFullMapName)
                                    .add(jPanelIMLayout.createSequentialGroup()
                                        .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                            .add(jLabel56)
                                            .add(jLabel57)
                                            .add(jPanelIMLayout.createSequentialGroup()
                                                .add(jLabel157)
                                                .add(7, 7, 7))
                                            .add(jPanelIMLayout.createSequentialGroup()
                                                .add(jLabel55)
                                                .add(13, 13, 13)))
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                                            .add(jTextDateStart)
                                            .add(jTextDateSuffix, 0, 0, Short.MAX_VALUE)
                                            .add(jTextDatePrefix, 0, 0, Short.MAX_VALUE)
                                            .add(jTextDateEnd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 59, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 87, Short.MAX_VALUE))
                            .add(jLabel58)
                            .add(jPanelIMLayout.createSequentialGroup()
                                .add(12, 12, 12)
                                .add(jComboBoxScale, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jComboBoxRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 164, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap(492, Short.MAX_VALUE))
        );
        jPanelIMLayout.setVerticalGroup(
            jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelIMLayout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel52)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel53)
                    .add(jLabel149))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxDateChangeDir, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel54)
                    .add(jLabel59)
                    .add(jTextDateChangeValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 36, Short.MAX_VALUE)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel157)
                    .add(jTextDateStart, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextDateEnd, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel55, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextDatePrefix, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel56))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextDateSuffix, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel57))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabelFullMapName)
                .add(20, 20, 20)
                .add(jLabel58)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelIMLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxScale, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxRatio, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(208, Short.MAX_VALUE))
        );

        jTabbedPaneMain.addTab("Interaction Model", jPanelIM);

        jPanelData.setEnabled(false);
        jPanelData.setName("data_capture_frame"); // NOI18N

        jTextFieldAPSIM.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldAPSIMActionPerformed(evt);
            }
        });

        jTextFieldGRASS.setToolTipText("The User must select GRASS 6.4. The model may not function properly with other versions of GRASS.");
        jTextFieldGRASS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldGRASSActionPerformed(evt);
            }
        });

        jTextFieldPython.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldPythonActionPerformed(evt);
            }
        });

        jButtonAPSIM.setText("Browse");
        jButtonAPSIM.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAPSIMActionPerformed(evt);
            }
        });

        jButtonGRASS.setText("Browse");
        jButtonGRASS.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGRASSActionPerformed(evt);
            }
        });

        jButtonPython.setText("Browse");
        jButtonPython.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonPythonActionPerformed(evt);
            }
        });

        jLabel72.setText("GRASS ENVIRONMENT SETUP");

        jLabel77.setText("APSIM Location:");

        jLabel78.setText("GRASS 6.4:");

        jLabel79.setText("Python");

        jLabel80.setText("WorldWind Visualization");

        jComboBoxWorldWind.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "ON", "OFF" }));
        jComboBoxWorldWind.setSelectedIndex(1);

        jLabel82.setText("Household Info Log");

        jComboBoxHouseholdLog.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Keep & Show", "Delete & Hide" }));
        jComboBoxHouseholdLog.setSelectedIndex(1);

        jLabel83.setText("Model Log Settings");

        jComboBoxHHEconLog.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Keep", "Delete" }));
        jComboBoxHHEconLog.setSelectedIndex(1);

        jLabel84.setText("Village Log");

        jComboBoxIMLog.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Show", "Hide" }));
        jComboBoxIMLog.setSelectedIndex(1);

        jLabel85.setText("Interaction Model Log");

        jLabel61.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel61.setText("(The installation location of the APSIM model)");

        jLabel62.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel62.setText("(The folder which contains the GRASS Bin folder)");

        jLabel63.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel63.setText("(Usually located in /usr/bin)");

        jTextFieldRC6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldRC6ActionPerformed(evt);
            }
        });

        jButtonRC6.setText("Browse");
        jButtonRC6.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRC6ActionPerformed(evt);
            }
        });

        jLabel128.setText("GRASS .rc6 Location");

        jLabel68.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel68.setText("(This file is usually located in the User's home directory.");

        jLabel129.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel129.setText("Please note that the file may be hidden)");

        jLabel130.setText("Temporary Map Files");

        jComboBoxTempMapFiles.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Keep", "Delete" }));
        jComboBoxTempMapFiles.setSelectedIndex(1);

        jLabel148.setFont(new java.awt.Font("Lucida Grande", 0, 11));
        jLabel148.setText("(Please note that these file will be stored in the associated GRASS mapset folder)");

        jComboBoxHHPopEconLog.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Keep", "Delete" }));

        jLabel158.setText("Household Economy and Population Logs");

        jComboBoxIMLogSave.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Keep", "Delete" }));
        jComboBoxIMLogSave.setSelectedIndex(1);

        jLabel191.setText("Interaction Model Log");

        org.jdesktop.layout.GroupLayout jPanelDataLayout = new org.jdesktop.layout.GroupLayout(jPanelData);
        jPanelData.setLayout(jPanelDataLayout);
        jPanelDataLayout.setHorizontalGroup(
            jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelDataLayout.createSequentialGroup()
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelDataLayout.createSequentialGroup()
                        .add(39, 39, 39)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel72)
                            .add(jPanelDataLayout.createSequentialGroup()
                                .add(10, 10, 10)
                                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanelDataLayout.createSequentialGroup()
                                        .add(10, 10, 10)
                                        .add(jComboBoxWorldWind, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 525, Short.MAX_VALUE)
                                        .add(jComboBoxTempMapFiles, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 92, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .add(113, 113, 113))
                                    .add(jPanelDataLayout.createSequentialGroup()
                                        .add(jLabel80)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 442, Short.MAX_VALUE)
                                        .add(jLabel130)
                                        .add(96, 96, 96))
                                    .add(jLabel83)
                                    .add(jPanelDataLayout.createSequentialGroup()
                                        .add(12, 12, 12)
                                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jPanelDataLayout.createSequentialGroup()
                                                .add(jComboBoxHouseholdLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jLabel82))
                                            .add(jPanelDataLayout.createSequentialGroup()
                                                .add(jComboBoxIMLogSave, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(18, 18, 18)
                                                .add(jLabel191)))
                                        .add(18, 18, 18)
                                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jPanelDataLayout.createSequentialGroup()
                                                .add(jComboBoxHHEconLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                                .add(jLabel84)
                                                .add(31, 31, 31)
                                                .add(jComboBoxIMLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .add(18, 18, 18)
                                                .add(jLabel85))
                                            .add(jPanelDataLayout.createSequentialGroup()
                                                .add(jComboBoxHHPopEconLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                                .add(jLabel158)))
                                        .add(68, 68, 68))))))
                    .add(jPanelDataLayout.createSequentialGroup()
                        .add(139, 139, 139)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel77)
                            .add(jLabel78)
                            .add(jLabel79)
                            .add(jLabel128))
                        .add(18, 18, 18)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jTextFieldPython)
                            .add(jTextFieldGRASS)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel61)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel62, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel63)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jTextFieldAPSIM)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jTextFieldRC6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 258, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jLabel68))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jButtonRC6)
                            .add(jButtonAPSIM)
                            .add(jButtonGRASS)
                            .add(jButtonPython)
                            .add(jLabel129))))
                .addContainerGap(30, Short.MAX_VALUE))
            .add(jPanelDataLayout.createSequentialGroup()
                .add(195, 195, 195)
                .add(jLabel148)
                .addContainerGap(279, Short.MAX_VALUE))
        );
        jPanelDataLayout.setVerticalGroup(
            jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanelDataLayout.createSequentialGroup()
                .add(30, 30, 30)
                .add(jLabel72)
                .add(18, 18, 18)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanelDataLayout.createSequentialGroup()
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldAPSIM, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jButtonAPSIM))
                        .add(5, 5, 5)
                        .add(jLabel61)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldGRASS, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jButtonGRASS)
                            .add(jLabel78))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel62)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jTextFieldPython, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jButtonPython)
                            .add(jLabel79)))
                    .add(jPanelDataLayout.createSequentialGroup()
                        .add(7, 7, 7)
                        .add(jLabel77)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel63)
                .add(18, 18, 18)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jTextFieldRC6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButtonRC6)
                    .add(jLabel128))
                .add(18, 18, 18)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel68)
                    .add(jLabel129))
                .add(26, 26, 26)
                .add(jLabel83)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxHouseholdLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel82)
                    .add(jComboBoxHHEconLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxIMLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel85)
                    .add(jLabel84))
                .add(18, 18, 18)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxIMLogSave, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel191)
                    .add(jComboBoxHHPopEconLog, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel158))
                .add(18, 18, 18)
                .add(jLabel148)
                .add(25, 25, 25)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel80)
                    .add(jLabel130))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanelDataLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBoxWorldWind, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBoxTempMapFiles, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(31, Short.MAX_VALUE))
        );

        jTabbedPaneMain.addTab("System Settings", jPanelData);

        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Lucida Grande", 0, 14)); // NOI18N
        jTextArea1.setRows(5);
        jTextArea1.setText("\n\n\n\n For More Information: http://shesc.asu.edu/medland/\n\n\n Publications: http://shesc.asu.edu/medland/papers");
        jScrollPane1.setViewportView(jTextArea1);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 859, Short.MAX_VALUE)
                .add(228, 228, 228))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 552, Short.MAX_VALUE)
                .addContainerGap())
        );

        jTabbedPaneMain.addTab("More Information", jPanel1);

        jButtonConfigCancel.setText("Cancel");
        jButtonConfigCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonConfigCancelActionPerformed(evt);
            }
        });

        jButtonConfigSave.setText("Save Configuration");
        jButtonConfigSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonConfigSaveActionPerformed(evt);
            }
        });

        jButtonConfigLoad.setText("Load Configuration");
        jButtonConfigLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonConfigLoadActionPerformed(evt);
            }
        });

        jLabel66.setFont(new java.awt.Font("Times New Roman", 0, 12));
        jLabel66.setText("GUI version ");

        jButtonConfigValidate.setText("Validate");
        jButtonConfigValidate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonConfigValidateActionPerformed(evt);
            }
        });

        jLabelVersion.setText(" ");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jTabbedPaneMain, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 920, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(layout.createSequentialGroup()
                        .add(jLabel66)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabelVersion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 127, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 33, Short.MAX_VALUE)
                        .add(jButtonConfigSave)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButtonConfigLoad)
                        .add(63, 63, 63)
                        .add(jButtonConfigValidate)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButtonConfigInitialize)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButtonConfigCancel)
                        .addContainerGap(41, Short.MAX_VALUE))))
        );

        layout.linkSize(new java.awt.Component[] {jButtonConfigLoad, jButtonConfigSave}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(jTabbedPaneMain, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 618, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel66)
                    .add(jButtonConfigCancel)
                    .add(jButtonConfigInitialize)
                    .add(jButtonConfigValidate)
                    .add(jButtonConfigSave)
                    .add(jButtonConfigLoad)
                    .add(jLabelVersion, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 19, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    
	private void jButtonConfigCancelActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonCancelActionPerformed
	    this.dispose();
	    
	}//GEN-LAST:event_jButtonCancelActionPerformed

	private void jButtonConfigInitializeActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonInitializeActionPerformed
		
        ArrayList errors = ValidateEntries();
        
        if ( errors.isEmpty() )
        {
            System.out.println("\n-> Current configuration appears valid. Initializing model parameters...");
        }
        
        else
        {
            //System.err.println();
            //System.err.println("INVALID CONFIGURATION!  " + errors.size() + " errors were encountered before validation ceased.");
            Iterator it = errors.iterator();
            
        	String error = "There was an error loading the data: ";
            while ( it.hasNext() )
            {
                 error = error + "\n\n" + (String) it.next();
                //System.err.println("\t" + error);
                it.remove();
            }
            
            
            javax.swing.JOptionPane.showMessageDialog(null, error,
                    "Error Loading Data", JOptionPane.WARNING_MESSAGE);
            
            return;
        }
        
        // save data to parameters object
        // IP!
        ip = new InitParameters();
        
        // Set Village data
        int num_rows = jTableVillageData.getRowCount();
        for ( int r = 0; r < num_rows; r++ )
        {
            ip.AddVillage(r, (String) jTableVillageData.getValueAt(r,0),
                    ((Float) jTableVillageData.getValueAt(r, 1)).floatValue(),
                    ((Float) jTableVillageData.getValueAt(r, 2)).floatValue(),
                    ((Integer) jTableVillageData.getValueAt(r, 3)).intValue(),
                    ((Integer) jTableVillageData.getValueAt(r, 4)).intValue(),
                    ((Integer) jTableVillageData.getValueAt(r, 5)).intValue());
        }
        
        ip.pct_clearing = (Integer.valueOf(this.jTextVillagePctInitClearing.getText())).intValue();
        // Get agent coordinates and resolution
        // Assume validation will insert landscape coords, and res if checked.
        //ip.agent_n = (Float.valueOf(this.jTextHumanNorthCoords.getText())).floatValue();
        //ip.agent_s = (Float.valueOf(this.jTextHumanSouthCoords.getText())).floatValue();
        //ip.agent_e = (Float.valueOf(this.jTextHumanEastCoords.getText())).floatValue();
        //ip.agent_w = (Float.valueOf(this.jTextHumanWestCoords.getText())).floatValue();
        // HOUSEHOLD PARAMETERS
        ip.birth_init_prob = (Double.valueOf(this.jSpinnerBirthInitProb.getText())).doubleValue();
        ip.birth_prob_group_size = ((Integer) this.jSpinnerBirthProbPerPeople.getValue()).intValue();
        ip.birth_prob_delta = (Double.valueOf(this.jSpinnerBirthProbDelta.getText())).doubleValue();
        ip.birth_prob_min = ((Integer) this.jSpinnerBirthProbMin.getValue()).intValue();
        ip.birth_prob_max = ((Integer) this.jSpinnerBirthProbMax.getValue()).intValue();
        ip.death_init_prob = (Double.valueOf(this.jSpinnerDeathInitProb.getText())).doubleValue();
        ip.death_prob_group_size = ((Integer) this.jSpinnerDeathProbPerPeople.getValue()).intValue();
        ip.death_prob_delta = (Double.valueOf(this.jSpinnerDeathProbDelta.getText())).doubleValue();
        ip.death_prob_min = ((Integer) this.jSpinnerDeathProbMin.getValue()).intValue();
        ip.death_prob_max = ((Integer) this.jSpinnerDeathProbMax.getValue()).intValue();
        ip.labor_pop_pct = ((Integer) this.jSpinnerPopulationPctLabor.getValue()).intValue();        
        ip.man_days_per_capita = (Integer.valueOf(this.jTextManDaysPerPerson.getText())).intValue();
       // ip.meat_milk_calories_pc = (Double.valueOf(this.jTextMeatMilkRqdPerPerson.getText())).doubleValue(); //
        //ip.wheat_reqd_per_capita = (Double.valueOf(this.jTextWheatRqdPerPerson.getText())).doubleValue(); //
        double totalkcal = (Double.valueOf(this.jTextAgpastKcal.getText())).doubleValue();
        double agriProportion = (Double.valueOf(this.jTextAgpastKcal.getText())).doubleValue() / 100;
        double pastproportion = (Double.valueOf(this.jTextPastoralProportion.getText())).doubleValue() / 100;
        ip.meat_milk_calories_pc = totalkcal* pastproportion;//
        ip.wheat_reqd_per_capita = agriProportion * totalkcal; //
        ip.maxfarmcost = (Integer.valueOf(this.jTextMaxFarmDistance.getText())).intValue();
        ip.maxgrazecost = (Integer.valueOf(this.jTextMaxGrazeDistance.getText())).intValue();
        ip.yield_expecation_scalar = (Integer.valueOf(this.jTextYieldExpScalar.getText())).intValue();

        //RESOURCE PARAMETERS
        ip.wheat_labor_reqd = (Float.valueOf(this.jTextWheatLaborRqd.getText())).floatValue();
        ip.wheat_init_expected_yield = (Float.valueOf(this.jTextWheatInitYield.getText())).floatValue();
        ip.wheat_calories_provided = (Float.valueOf(this.jTextWheatCalories.getText())).floatValue();
        ip.barley_labor_reqd = (Double.valueOf(this.jTextBarleyLaborRqd.getText())).doubleValue();
        ip.barley_init_expected_yield = (Double.valueOf(this.jTextBarleyInitYield.getText())).doubleValue();
        ip.maximum_wheat_yield = (Double.valueOf(this.jTextFieldMaxWheatYield.getText())).doubleValue();
        ip.maximum_barley_yield = (Double.valueOf(this.jTextFieldMaxBarleyYield.getText())).doubleValue();
        ip.wood_pc = (Double.valueOf(this.jTextWoodPc.getText())).doubleValue();
        ip.wood_intensity = (Double.valueOf(this.jTextFieldWoodIntensity.getText())).doubleValue();
        ip.wooddistweight = (Double.valueOf(this.jTextFieldWoodDistanceWeight.getText()));
        ip.lcovweight = (Double.valueOf(this.jTextFieldLcovDistWight.getText()));
        ip.fdistweight = (Double.valueOf(this.jTextFieldFarmDistWght.getText()));
        ip.sfertilweight = (Double.valueOf(this.jTextFieldSoilFtyWight.getText()));
        ip.sdepthweight = (Double.valueOf(this.jTextFieldDepthWght.getText()));
        ip.degrade_rate = (Double.valueOf(this.jTextFieldFertilityImpact.getText()));
        ip.maxlcov = (Integer.valueOf( this.jTextFieldMaxLcov.getText())).intValue();
        ip.seed_percentage = (Double.valueOf(this.jTextFieldSeedPct.getText()))/100;

        ip.lcBreakPoint1= (Integer.valueOf(this.jTextFieldBpointLC1.getText()));
        ip.lcBreakPoint2= (Integer.valueOf(this.jTextFieldBpointLC2.getText()));
        ip.daBpoint1= (Integer.valueOf(this.jTextFieldBpointDA1.getText()));
        ip.daBpoint2= (Integer.valueOf(this.jTextFieldBpointDA2.getText()));
        
        if ( this.jComboBoxLandTenure.getSelectedIndex() != 1 ){
    	ip.landTenure = true;
        }
        else{   	ip.landTenure = false;
        }

        if ( this.jComboBoxHHSizeLimit.getSelectedIndex() != 1 ){
    	ip.hhSizeLimit = true;
    	ip.hhSizeExtreme = (Integer.valueOf(this.jTextHHSizeLimit.getText()));
        }
        else{   	ip.hhSizeLimit = false;
        }        

        //OVICAPRID PARAMETERS
        ip.ovicaprid_density = (Double.valueOf(this.jTextFieldgrazingDensity.getText())).doubleValue();
        ip.sheep_ratio =  (Integer.valueOf( this.jTextFieldsheepRatio.getText()));
        ip.goat_ratio = (Integer.valueOf( this.jTextFieldgoatRatio.getText()));
        ip.ovicaprid_per_person = (Integer.valueOf( this.jTextFieldnumOfOvicaprids.getText())).intValue();
        ip.init_grazing_expectation = (Double.valueOf(this.jTextFieldInitGrazingReturn.getText())).doubleValue();
        ip.barley_fodder_percentage = (Double.valueOf(this.jTextFieldPctBarleyFodder.getText())).doubleValue();
        ip.sheep_return = (Double.valueOf(this.jTextFieldSheepYield.getText())).doubleValue();
        ip.goat_return = (Double.valueOf(this.jTextFieldGoatYield.getText())).doubleValue();
        ip.gdistweight = (Double.valueOf(this.jTextFieldGrazingDistWight.getText()));
        ip.sheep_fodder = (Integer.valueOf(this.jTextFieldSheepFodder.getText()));
        ip.goat_fodder = (Integer.valueOf(this.jTextFieldGoatFodder.getText()));
        
        if ( this.jComboBoxFallFieldGrz.getSelectedIndex() == 0 ){
        	ip.fallow_field_grazing = true;
        }
        else if (this.jComboBoxFallFieldGrz.getSelectedIndex() == 1 ){
        	ip.fallow_field_grazing = false;
        }
        
        // GRASS ENVIRIONMENT DATA
        ip.APSIMPath = this.jTextFieldAPSIM.getText();
        ip.grassrc6loc = this.jTextFieldRC6.getText();
        ip.GISDBASE = this.jTextGISDBASE.getText();
        ip.location = this.jComboBoxLocation.getSelectedItem().toString();
        ip.mapset = this.jTextMapset.getText();
        ip.DEM = this.jComboBoxDEM.getSelectedItem().toString();
        ip.land_cover = this.jComboBoxLandCover.getSelectedItem().toString();
        ip.fertility_map = this.jComboBoxFertility.getSelectedItem().toString();
        
        if ( this.jComboBoxBedrock.getSelectedIndex() > 0 ){
        	ip.bedrock_map =  this.jComboBoxBedrock.getSelectedItem().toString();
        }
        if ( this.jComboBoxMaxLandCover.getSelectedIndex() > 0 ){
        	ip.max_landcover_map =  this.jComboBoxMaxLandCover.getSelectedItem().toString();
        }
        
        if ( this.jComboBoxSmoothing.getSelectedIndex() == 0 ){
        	ip.evolSmoothing  = "low";
        }
        else if (this.jComboBoxSmoothing.getSelectedIndex() == 1 ){
        	ip.evolSmoothing  = "high";
        } 
        else if (this.jComboBoxSmoothing.getSelectedIndex() == 2 ){
        	ip.evolSmoothing  = "no";
        } 
       

        if ( this.jRadioButtonFrictionMap.isSelected() ){
        	ip.friction_map = this.jComboBoxFrictionMap.getSelectedItem().toString();
        }
        else{
        	ip.friction_value = (Float.valueOf(this.jTextFrictionValue.getText())).floatValue();
        }
        if ( this.jRadioButtonSoilKMap.isSelected() ){
        	ip.soilK_map = this.jComboBoxSoilKMap.getSelectedItem().toString();
        }
        else{
        	ip.soilK_value = (Float.valueOf(this.jTextSoilKValue.getText())).floatValue();
        }
        
        // Set Interaction data
        ip.date_end = (Integer.valueOf(this.jTextDateEnd.getText())).intValue();
        ip.date_start = (Integer.valueOf(this.jTextDateStart.getText())).intValue();
        ip.date_change_dir = this.jComboBoxDateChangeDir.getSelectedIndex();
        ip.date_prefix = this.jTextDatePrefix.getText();
        ip.date_suffix = this.jTextDateSuffix.getText(); 
        ip.date_delta = (Integer.valueOf(this.jTextDateChangeValue.getText()));

        //Set System Settings, Paths and Log Information
        String RulesPath = ip.APSIMPath+ "Rules/";
        ip.landcover_reclass_rules = RulesPath + "luse_reclass_rules.txt";
    	ip.landcover_color_rules = RulesPath + "luse_colors.txt";
    	ip.cfactor_reclass_rules = RulesPath + "cfactor_recode_rules.txt";
    	ip.cfactor_color_rules = RulesPath + "cfactor_colors.txt";
    	ip.impacts_color_rules = RulesPath + "impacts_colors.txt";
    	ip.fertility_color_rules = RulesPath + "sfertil_colors.txt";
        ip.GrassPath = this.jTextFieldGRASS.getText();
        ip.PythonPath = this.jTextFieldPython.getText() + "/";
        
        if ( this.jComboBoxWorldWind.getSelectedIndex() != 1 ){
    	ip.WorldWindDisplayOn = true;
        }
        else{   	ip.WorldWindDisplayOn = false;
        } 
        if ( this.jComboBoxHouseholdLog.getSelectedIndex() != 1 ){
        ip.HouseholdLogOn = true;
        }
        else{        ip.HouseholdLogOn = false;
        }
        if ( this.jComboBoxHHEconLog.getSelectedIndex() != 1 ){
        ip.VillageLogOn = true;
        }
        else{        ip.VillageLogOn = false;
        }
        if ( this.jComboBoxIMLog.getSelectedIndex() != 1 ){
        ip.IMLogShow = true;
        }
        else{        ip.IMLogShow = false;
        }
        if ( this.jComboBoxTempMapFiles.getSelectedIndex() != 1 ){
        ip.KeepRasterMaps = false;
        }
        else{        ip.KeepRasterMaps = true;
        }
        if ( this.jComboBoxHHPopEconLog.getSelectedIndex() != 1 ){
        ip.popeconlogs = true;
        }
        else{        ip.popeconlogs = false;
        }
        if ( this.jComboBoxIMLogSave.getSelectedIndex() != 1 ){
        ip.IMLogOn = true;
        }
        else{        ip.IMLogOn = false;
        }

        
        // LANDSCAPE EVOL PARAMETERS
        ip.soil_density = (Double.valueOf(this.jTextFieldsoilDensity.getText()));
        ip.stream_transport = this.jTextFieldstreamTrans.getText();    
        ip.load_exponent = (Double.valueOf(this.jTextFieldloadExponent.getText()));
        ip.kappa = (Integer.valueOf( this.jTextFieldkappa.getText())).intValue();
        ip.cutoff1 = (Double.valueOf( this.jTextFieldcutoff1.getText()));
        ip.cutoff2 = (Double.valueOf( this.jTextFieldcutoff2.getText()));
        ip.cutoff3 = (Double.valueOf( this.jTextFieldcutoff3.getText()));
        
    	//OTHER LANDSCAPE PARAMETERS
        ip.soil_depth_min = (Double.valueOf(this.jTextFieldSoilMin.getText()));//
        ip.soil_depth_max =(Double.valueOf(this.jTextFieldSoilMax.getText()));//
        ip.soil_recovery = (Integer.valueOf(this.jTextFieldSoilRecovery.getText())).intValue();//
        
        // CLIMATE PARAMETERS
        ip.r_factor = (Double.valueOf(this.jTextFieldrFactor.getText()));
        ip.storms = (Integer.valueOf(this.jTextFieldStormEvents.getText())).intValue();
        ip.annual_precip = (Double.valueOf(this.jTextFieldrainValue.getText()));
        ip.storm_speed  = (Double.valueOf(this.jTextFieldStormSpeed.getText()));
        ip.storm_length  = (Double.valueOf(this.jTextFieldStormLength.getText()));
        ip.storm_precip = (Double.valueOf(this.jTextFieldStormPrecip.getText()));
        
        ip.climateDataPath = this.jTextFieldClimFile.getText();
        if ( this.jComboBoxClimateSource.getSelectedIndex() != 1 ){
            // static
        		ip.climateDataSeries = false;
            }
        else{ // dynamic
            	ip.climateDataSeries = true;
            }
        

	        
	}//GEN-LAST:event_jButtonInitializeActionPerformed

	private void jButtonConfigLoadActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonLoadConfigActionPerformed
	        this.setCursor(new java.awt.Cursor(java.awt.Cursor.WAIT_CURSOR));

	        // request the user provide the configuration file name
	        javax.swing.JFileChooser LoadDialog = new javax.swing.JFileChooser();
	        int returnVal = LoadDialog.showOpenDialog(this);
	        File file = LoadDialog.getSelectedFile();
	        
	        // write the variable values to their respective components
	        if ( file != null )
	            ConfigurationLoad(file);

	        // change the cursor back to default
	        this.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
	        
	}//GEN-LAST:event_jButtonLoadConfigActionPerformed

	private void jButtonConfigSaveActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonSaveConfigActionPerformed
	        this.setCursor(new java.awt.Cursor(java.awt.Cursor.WAIT_CURSOR));

	        // request the user provide the configuration file name
	        javax.swing.JFileChooser SaveDialog = new javax.swing.JFileChooser();
	        int returnVal = SaveDialog.showSaveDialog(this);
	        File file = SaveDialog.getSelectedFile();
	        
	        // write the current variable values to the requested file.
	        if ( file != null )
	            ConfigurationSave(file);

	        // change the cursor back to default
	        this.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
	        
	}//GEN-LAST:event_jButtonSaveConfigActionPerformed

	private void jButtonConfigValidateActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonValidateActionPerformed
        ArrayList errors = ValidateEntries();
        
        if ( errors.isEmpty() )
        {
        	javax.swing.JOptionPane.showMessageDialog(null, "Current configuration appears valid. \n\n Select [Save Configuration] to save now or [Initialize] to start the simulation.",
                    "Data Load Successful", JOptionPane.WARNING_MESSAGE);
        }
        
        else
        {
            System.err.println();
            System.err.println("INVALID CONFIGURATION!  " + errors.size() + " errors were encountered before validation ceased.");
            Iterator it = errors.iterator();
            String error = "There was an error loading the data: ";
            while ( it.hasNext() )
            {
                 error = error + "\n\n" +(String) it.next();
                //System.err.println("\t" + error);
                it.remove();
            }
            
            
            javax.swing.JOptionPane.showMessageDialog(null, error, "Error Loading Data", JOptionPane.WARNING_MESSAGE);
        }
        
    }//GEN-LAST:event_jButtonValidateActionPerformed

   
	private void jButtonVillageAddActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonAddVillageActionPerformed
	    try
	    {
	        int num_rows = jTableVillageData.getRowCount();
	        String curr_name = "";
	        Float curr_ns = null, curr_ew = null;
//            Integer num_hh = null, hh_pop = null;
	        boolean unique_village = true;
	        
	        // Check if a village already exists with that name or at those coordinates
	        for ( int r = 0; r < num_rows; r++ )
	        {
	            curr_name = (String) jTableVillageData.getValueAt(r,0);
	            curr_ns = (Float) jTableVillageData.getValueAt(r, 1);
	            curr_ew = (Float) jTableVillageData.getValueAt(r, 2);
	            
	            // same name
	            if ( curr_name == jTextVillageName.getText() )
	                unique_village = false;
	            
	            // same coordinates
	            if ( (curr_ns.floatValue() == Float.parseFloat(jTextVillageNSCoord.getText())) &&
	                 (curr_ew.floatValue() == Float.parseFloat(jTextVillageEWCoord.getText())) )
	                unique_village = false;
	            
	            if ( !unique_village )
	                break;
	        }
	              
	        if ( unique_village )
	        {
	            // Add the village data to the table
	            Object [] row_data = {jTextVillageName.getText(),
	                                  Float.valueOf(jTextVillageNSCoord.getText()),
	                                  Float.valueOf(jTextVillageEWCoord.getText()),
	                                  Integer.valueOf(jTextVillagePctInitClearing.getText()),
                                          Integer.valueOf(jTextVillageNumHouseholds.getText()),
                                          Integer.valueOf(jTextVillageHhStartPopulation.getText())};
	
	            VillageAddRowData(row_data);
	
	            num_rows = jTableVillageData.getRowCount();
	            jTextNumVillages.setText(new Integer(num_rows).toString());
	            jButtonVillageRemove.setEnabled(true);
	            jButtonVillageClearData.setEnabled(true);
	                        
	            // Update default name for next village
	            jTextVillageName.setText("Village_" + num_rows);
	        }
	        
	        else
	        {
	            temp_name = jTextVillageName.getText();
	            jTextVillageName.setText("ERROR: Duplicate village.");
	        }
	    }
	    
	    catch (NumberFormatException nfe)
	    {
	        temp_name = jTextVillageName.getText();
	        jTextVillageName.setText("ERROR. Check number values.");
	    }
	    
	}//GEN-LAST:event_jButtonAddVillageActionPerformed

	private void jButtonVillageClearDataActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonClearVillageDataActionPerformed
	    VillageClearAllData();
	    
	    jTextNumVillages.setText("0");
	    jTextVillageName.setText("Village_0");
	    
	    jButtonVillageRemove.setEnabled(false);
	    jButtonVillageClearData.setEnabled(false);
	    
	}//GEN-LAST:event_jButtonClearVillageDataActionPerformed

	private void jButtonVillageRemoveActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jButtonRemoveVillageActionPerformed
	    int row = jTableVillageData.getSelectedRow();
	    
	    // Remove the village data row from the table
	    if ( row != -1 )
	    {
	        javax.swing.table.DefaultTableModel model = (javax.swing.table.DefaultTableModel) jTableVillageData.getModel();
	        model.removeRow(row);
	        
	        int num_rows = jTableVillageData.getRowCount();
	        jTextNumVillages.setText(new Integer(num_rows).toString());
	        
	        if ( num_rows == 0 )
	        {
	            jButtonVillageRemove.setEnabled(false);
	            jButtonVillageClearData.setEnabled(false);
	        }
	    }
	    
	}//GEN-LAST:event_jButtonRemoveVillageActionPerformed

	private void jCheckBoxEnvironmentCoordinatesActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jCheckBoxEnvironmentCoordinatesActionPerformed
	    boolean manual_entry = !this.jCheckBoxEnvironmentCoordinates.isSelected();
	    
	    this.jLabelHumanNCoord.setEnabled(manual_entry);
	    this.jLabelHumanSCoord.setEnabled(manual_entry);
	    this.jLabelHumanECoord.setEnabled(manual_entry);
	    this.jLabelHumanWCoord.setEnabled(manual_entry);
	    this.jTextHumanNorthCoords.setEnabled(manual_entry);
	    this.jTextHumanSouthCoords.setEnabled(manual_entry);
	    this.jTextHumanEastCoords.setEnabled(manual_entry);
	    this.jTextHumanWestCoords.setEnabled(manual_entry);
	    
	}//GEN-LAST:event_jCheckBoxEnvironmentCoordinatesActionPerformed

	private void jCheckBoxEnvironmentResolutionsActionPerformed(java.awt.event.ActionEvent evt)
	{//GEN-FIRST:event_jCheckBoxEnvironmentResolutionsActionPerformed
	    boolean manual_entry = !this.jCheckBoxEnvironmentResolutions.isSelected();
	    
	    //this.jLabelHumanNSRes.setEnabled(manual_entry);
	    //this.jLabelHumanEWRes.setEnabled(manual_entry);
	    //this.jTextHumanNSResolution.setEnabled(manual_entry);
	    //this.jTextHumanEWResolution.setEnabled(manual_entry);
	    
	}//GEN-LAST:event_jCheckBoxEnvironmentResolutionsActionPerformed


        private void jSliderVillagePctInitClearingStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderVillagePctInitClearingStateChanged
	    jTextVillagePctInitClearing.setText(String.valueOf(jSliderVillagePctInitClearing.getValue()));
	    
        }//GEN-LAST:event_jSliderVillagePctInitClearingStateChanged

                                            

	private void jTextDateEndActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextEndDateActionPerformed
	    try
	    {
	        // Check if the date is valid
	        Integer validate = Integer.valueOf(jTextDateEnd.getText());
	    }
	    
	    catch (NumberFormatException nfe)
	    {
	        // Restore text box to default value
	        jTextDateEnd.setText("250");
	    }
	    
	    finally
	    {
	        // Update label
	        num_date_placeholders = Math.max(0, jTextDateEnd.getText().length()); // changed because of new nonzero change
	        this.UpdateFullMapNameLabel();
	    }
	    
	}//GEN-LAST:event_jTextEndDateActionPerformed

	private void jTextDatePrefixActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextDatePrefixActionPerformed
	    this.UpdateFullMapNameLabel();
	    
	}//GEN-LAST:event_jTextDatePrefixActionPerformed


	private void jTextDateSuffixActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextDateSuffixActionPerformed
	    this.UpdateFullMapNameLabel();
	    
	}//GEN-LAST:event_jTextDateSuffixActionPerformed

    private void jTextVillageNumHouseholdsActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_jTextVillageNumHouseholdsActionPerformed
	    String num_hh_str = jTextVillageNumHouseholds.getText();
	    
	    if ( num_hh_str.compareTo("") == 0 )
	        jTextVillageNumHouseholds.setText("0");
	    
	    else
	    {
	        try
	        {
	            int num_hh = Integer.parseInt(num_hh_str);
	            
	            if ( num_hh < 0 )
	                jTextVillageNumHouseholds.setText("0");
	        }
	        
	        catch (NumberFormatException nfe)
	        {
	            jTextVillageNumHouseholds.setText("0");
	        }
	    }
	    
    }//GEN-LAST:event_jTextVillageNumHouseholdsActionPerformed

    private void jTextVillageHhStartPopulationActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_jTextVillageHhStartPopulationActionPerformed
        String start_pop_str = jTextVillageHhStartPopulation.getText();
        
        if ( start_pop_str.compareTo("") == 0 )
            jTextVillageHhStartPopulation.setText("0");
        
        else
        {
            try
            {
                int start_pop = Integer.parseInt(start_pop_str);
                
                if ( start_pop < 0 )
                    jTextVillageHhStartPopulation.setText("0");
            }
            
            catch (NumberFormatException nfe)
            {
                jTextVillageHhStartPopulation.setText("0");
            }
        }
        
        }//GEN-LAST:event_jTextVillageHhStartPopulationActionPerformed

    private void jTextVillageEWCoordFocusGained(java.awt.event.FocusEvent evt)
    {//GEN-FIRST:event_jTextEWCoordFocusGained
        if ( temp_name != "" )
        {
            jTextVillageName.setText(temp_name);
            temp_name = "";
        }
        
    }//GEN-LAST:event_jTextEWCoordFocusGained

    private void jTextVillageNSCoordFocusGained(java.awt.event.FocusEvent evt)
    {//GEN-FIRST:event_jTextNSCoordFocusGained
        if ( temp_name != "" )
        {
            jTextVillageName.setText(temp_name);
            temp_name = "";
        }
        
    }//GEN-LAST:event_jTextNSCoordFocusGained

    private void jTextVillageNameFocusGained(java.awt.event.FocusEvent evt)
    {//GEN-FIRST:event_jTextVillageNameFocusGained
        if ( temp_name != "" )
        {
            jTextVillageName.setText(temp_name);
            temp_name = "";
        }
        
    }//GEN-LAST:event_jTextVillageNameFocusGained

    private void jTextVillagePctInitClearingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextPctInitClearingActionPerformed
        float currValue = 0.0f;
        
        try
        {
            currValue = Float.parseFloat(jTextVillagePctInitClearing.getText());
            
            if ( currValue < 0.0f )
                currValue = 0.0f;
            
            else if ( currValue > 100.0f )
                currValue = 100.0f;
        }
        
        catch (NumberFormatException nfe)
        {
            currValue = 50.0f;
        }
        
        finally
        {
            jSliderVillagePctInitClearing.setValue((int) Math.floor(currValue));
        }
    }//GEN-LAST:event_jTextPctInitClearingActionPerformed

    private void jTextFieldAPSIMActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldAPSIMActionPerformed
       
    	
    	
    }//GEN-LAST:event_jTextFieldAPSIMActionPerformed

    private void jTextFieldGRASSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldGRASSActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldGRASSActionPerformed

    private void jButtonAPSIMActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAPSIMActionPerformed
    	try
	    {
	        int result = jFileChooserAPSIM.showOpenDialog(this);
	
	        if ( result == jFileChooserAPSIM.APPROVE_OPTION )
	        {
	            String path = jFileChooserAPSIM.getSelectedFile().getCanonicalPath();
	            jTextFieldAPSIM.setText(path);
	            File dir = new File(path);
	           
	        }
	    }
	    
	    catch (java.io.IOException ioe)
	    {
	        
	    }
    }//GEN-LAST:event_jButtonAPSIMActionPerformed

    private void jButtonGRASSActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGRASSActionPerformed
    	try
	    {
	        int result = jFileChooserGRASS.showOpenDialog(this);
	
	        if ( result == jFileChooserGRASS.APPROVE_OPTION )
	        {
	            String path = jFileChooserGRASS.getSelectedFile().getCanonicalPath();
	            jTextFieldGRASS.setText(path);
	            File dir = new File(path);
	           
	        }
	    }
	    
	    catch (java.io.IOException ioe)
	    {
	        
	    }
    }//GEN-LAST:event_jButtonGRASSActionPerformed

    private void jButtonPythonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonPythonActionPerformed
    	try
	    {
	        int result = jFileChooserPython.showOpenDialog(this);
	
	        if ( result == jFileChooserPython.APPROVE_OPTION )
	        {
	            String path = jFileChooserPython.getSelectedFile().getCanonicalPath();
	            jTextFieldPython.setText(path);
	            File dir = new File(path);
	
	            	           
	        }
	    }
	    
	    catch (java.io.IOException ioe)
	    {
	        
	    }
    }//GEN-LAST:event_jButtonPythonActionPerformed

    private void jTextFieldPythonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldPythonActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldPythonActionPerformed

    private void jTextYieldExpScalar1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextYieldExpScalar1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jTextYieldExpScalar1ActionPerformed

    private void jRadioButtonSoilKMap1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKMap1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonSoilKMap1ActionPerformed

    private void jRadioButtonSoilKValue1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKValue1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonSoilKValue1ActionPerformed

    private void jRadioButtonFrictionMap1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionMap1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonFrictionMap1ActionPerformed

    private void jRadioButtonFrictionValue1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionValue1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonFrictionValue1ActionPerformed

    private void jButtonGenerateMapset1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGenerateMapset1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jButtonGenerateMapset1ActionPerformed

    private void jComboBoxLocation1ItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jComboBoxLocation1ItemStateChanged
        // TODO add your handling code here:
}//GEN-LAST:event_jComboBoxLocation1ItemStateChanged

    private void jTextGISDBASE1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextGISDBASE1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jTextGISDBASE1ActionPerformed

    private void jButtonBrowseRootDir1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBrowseRootDir1ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jButtonBrowseRootDir1ActionPerformed

    private void jRadioButtonSoilKMap2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKMap2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonSoilKMap2ActionPerformed

    private void jRadioButtonSoilKValue2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKValue2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonSoilKValue2ActionPerformed

    private void jRadioButtonFrictionMap2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionMap2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonFrictionMap2ActionPerformed

    private void jRadioButtonFrictionValue2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionValue2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jRadioButtonFrictionValue2ActionPerformed

    private void jButtonGenerateMapset2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGenerateMapset2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jButtonGenerateMapset2ActionPerformed

    private void jComboBoxLocation2ItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jComboBoxLocation2ItemStateChanged
        // TODO add your handling code here:
}//GEN-LAST:event_jComboBoxLocation2ItemStateChanged

    private void jTextGISDBASE2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextGISDBASE2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jTextGISDBASE2ActionPerformed

    private void jButtonBrowseRootDir2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBrowseRootDir2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jButtonBrowseRootDir2ActionPerformed

    private void jPanelLandEvolInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_jPanelLandEvolInputMethodTextChanged
        // TODO add your handling code here:
}//GEN-LAST:event_jPanelLandEvolInputMethodTextChanged

    private void jTextGISDBASEActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextGISDBASEActionPerformed
        try {
            if ( jTextGISDBASE.getText().compareTo("") != 0 ) {
                String path = jTextGISDBASE.getText();
                File dir = new File(path);

                // This filter only returns directories
                FileFilter dirFilter = new FileFilter() {
                    public boolean accept(File dir) {
                        return dir.isDirectory();
                    }
                };

                File[] files = dir.listFiles(dirFilter);
                jComboBoxLocation.removeAllItems();
                jComboBoxLocation.addItem(makeObj("[Select a location]"));

                for ( int f = 0; f < files.length; f++ ) {
                    jComboBoxLocation.addItem(makeObj(files[f].getName()));
                }

                jComboBoxLocation.setSelectedIndex(0);
                jComboBoxLocation.setEnabled(true);
                jTextMapset.setEnabled(false);
                jButtonGenerateMapset.setEnabled(false);
            }

            else {
                jComboBoxLocation.setEnabled(false);
                jComboBoxLocation.removeAllItems();
                jComboBoxLocation.setSelectedIndex(-1);
                jTextMapset.setEnabled(false);
                jButtonGenerateMapset.setEnabled(false);
            }
        }

        catch (NullPointerException npe) {

        }
}//GEN-LAST:event_jTextGISDBASEActionPerformed

    private void jButtonBrowseRootDirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonBrowseRootDirActionPerformed
        try {
            int result = jFileChooserGISDBASE.showOpenDialog(this);

            if ( result == jFileChooserGISDBASE.APPROVE_OPTION ) {
                String path = jFileChooserGISDBASE.getSelectedFile().getCanonicalPath();
                jTextGISDBASE.setText(path);
                File dir = new File(path);

                // This filter only returns directories
                FileFilter dirFilter = new FileFilter() {
                    public boolean accept(File dir) {
                        return dir.isDirectory();
                    }
                };

                File[] files = dir.listFiles(dirFilter);
                jComboBoxLocation.removeAllItems();
                jComboBoxLocation.addItem(makeObj("[Select a location]"));

                for ( int f = 0; f < files.length; f++ ) {
                    jComboBoxLocation.addItem(makeObj(files[f].getName()));
                }

                jComboBoxLocation.setSelectedIndex(0);
                jComboBoxLocation.setEnabled(true);
                jTextMapset.setEnabled(false);
                jButtonGenerateMapset.setEnabled(false);
            }
        }

        catch (java.io.IOException ioe) {

        }
}//GEN-LAST:event_jButtonBrowseRootDirActionPerformed

    private void jComboBoxLocationItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jComboBoxLocationItemStateChanged
        try {
            if ( jComboBoxLocation.getSelectedIndex() > 0 ) {
                // get grasspath and add PERMANENT and cell header to file directory
                String path = jTextGISDBASE.getText() + File.separator + jComboBoxLocation.getSelectedItem().toString();
                path = path + File.separator + "PERMANENT" + File.separator + "cellhd" + File.separator;
                File dir = new File(path);

                // This filter only returns files
                FileFilter dirFilter = new FileFilter() {
                    public boolean accept(File dir) {
                        return dir.isFile();
                    }
                };

                File[] files = dir.listFiles(dirFilter);
                java.util.Arrays.sort(files);
                jComboBoxDEM.removeAllItems();
                jComboBoxDEM.addItem(makeObj("[Select the DEM]"));
                jComboBoxMaxLandCover.removeAllItems();
                jComboBoxMaxLandCover.addItem(makeObj("[Optional]"));
                jComboBoxBedrock.removeAllItems();
                jComboBoxBedrock.addItem(makeObj("[Optional]"));
                jComboBoxLandCover.removeAllItems();
                jComboBoxLandCover.addItem(makeObj("[Select the Land Cover map]"));
                jComboBoxFrictionMap.removeAllItems();
                jComboBoxFrictionMap.addItem(makeObj("[Select the Friction map]"));
                jComboBoxSoilKMap.removeAllItems();
                jComboBoxSoilKMap.addItem(makeObj("[Select the Soil K-Factor map]"));
                jComboBoxFertility.removeAllItems();
                jComboBoxFertility.addItem(makeObj("[Select the Soil Fertility map]"));

                for ( int f = 0; f < files.length; f++ ) {
                    jComboBoxDEM.addItem(makeObj(files[f].getName()));
                    jComboBoxMaxLandCover.addItem(makeObj(files[f].getName()));
                    jComboBoxBedrock.addItem(makeObj(files[f].getName()));
                    jComboBoxLandCover.addItem(makeObj(files[f].getName()));
                    jComboBoxFrictionMap.addItem(makeObj(files[f].getName()));
                    jComboBoxSoilKMap.addItem(makeObj(files[f].getName()));
                    jComboBoxFertility.addItem(makeObj(files[f].getName()));
                }

                jComboBoxDEM.setEnabled(true);
                jComboBoxLandCover.setEnabled(true);
                jComboBoxFertility.setEnabled(true);
                jRadioButtonFrictionValue.setEnabled(true);
                jRadioButtonFrictionMap.setEnabled(true);
                jRadioButtonFrictionMap.setSelected(true);
                jRadioButtonSoilKValue.setEnabled(true);
                jRadioButtonSoilKMap.setEnabled(true);
                jRadioButtonSoilKMap.setSelected(true);
                jComboBoxMaxLandCover.setEnabled(true);
                jComboBoxBedrock.setEnabled(true);

                if ( jRadioButtonFrictionValue.isSelected() ) {
                    jTextFrictionValue.setEnabled(true);
                    jComboBoxFrictionMap.setEnabled(false);
                }

                else if ( jRadioButtonFrictionMap.isSelected() ) {
                    jTextFrictionValue.setEnabled(false);
                    jComboBoxFrictionMap.setEnabled(true);
                }

                if ( jRadioButtonSoilKValue.isSelected() ) {
                    jTextSoilKValue.setEnabled(true);
                    jComboBoxSoilKMap.setEnabled(false);
                }

                else if ( jRadioButtonSoilKMap.isSelected() ) {
                    jTextSoilKValue.setEnabled(false);
                    jComboBoxSoilKMap.setEnabled(true);
                }

                jTextMapset.setEnabled(true);
                jButtonGenerateMapset.setEnabled(true);
            }

            else {
                jComboBoxDEM.setEnabled(false);
                jComboBoxDEM.removeAllItems();
                jComboBoxDEM.setSelectedIndex(-1);
                jComboBoxLandCover.setEnabled(false);
                jComboBoxLandCover.removeAllItems();
                jComboBoxLandCover.setSelectedIndex(-1);
                jComboBoxFertility.setEnabled(false);
                jComboBoxFertility.removeAllItems();
                jComboBoxFertility.setSelectedIndex(-1);
                jRadioButtonFrictionValue.setEnabled(false);
                jTextFrictionValue.setEnabled(false);
                jRadioButtonFrictionMap.setEnabled(false);
                jComboBoxFrictionMap.setEnabled(false);
                jComboBoxFrictionMap.removeAllItems();
                jComboBoxFrictionMap.setSelectedIndex(-1);
                jRadioButtonSoilKValue.setEnabled(false);
                jTextSoilKValue.setEnabled(false);
                jRadioButtonSoilKMap.setEnabled(false);
                jComboBoxSoilKMap.setEnabled(false);
                jComboBoxSoilKMap.removeAllItems();
                jComboBoxSoilKMap.setSelectedIndex(-1);

                jTextMapset.setEnabled(false);
                jButtonGenerateMapset.setEnabled(false);
            }
        }

        catch (NullPointerException npe) {
            // invalid path and no files/directories available. Gracefully ignore.
            jComboBoxDEM.removeAllItems();
            jComboBoxDEM.addItem(makeObj("[Invalid Directory]"));
            jComboBoxLandCover.removeAllItems();
            jComboBoxLandCover.addItem(makeObj("[Invalid Directory]"));
            jComboBoxFrictionMap.removeAllItems();
            jComboBoxFrictionMap.addItem(makeObj("[Invalid Directory]"));
            jComboBoxSoilKMap.removeAllItems();
            jComboBoxSoilKMap.addItem(makeObj("[Invalid Directory]"));
        }
}//GEN-LAST:event_jComboBoxLocationItemStateChanged

    private void jButtonGenerateMapsetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGenerateMapsetActionPerformed
        // find next directory of form YYYYMMDD_Run# and enter into the mapset text box.
        String new_mapset = "";
        String path = jTextGISDBASE.getText() + File.separator + jComboBoxLocation.getSelectedItem().toString()  + File.separator;
        int run_num = 1;

        Calendar today = Calendar.getInstance();
        String year = Integer.toString(today.get(Calendar.YEAR));
        String month = Integer.toString(today.get(Calendar.MONTH) + 1);     // 0-based set
        if ( month.length() == 1 )
            month = "0" + month;
        String day = Integer.toString(today.get(Calendar.DAY_OF_MONTH));
        if ( day.length() == 1 )
            day = "0" + day;
        String run = Integer.toString(run_num);

        new_mapset = year + month + day + "_Run" + run;
        File dir = new File(path + new_mapset);

        while ( dir.exists() ) {
        	
            run_num++;
            run = Integer.toString(run_num);
            new_mapset = year + month + day + "_Run" + run;
            dir = new File(path + new_mapset);
        }
       

        this.jTextMapset.setText(new_mapset);
}//GEN-LAST:event_jButtonGenerateMapsetActionPerformed

    private void jRadioButtonFrictionValueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionValueActionPerformed
        jComboBoxFrictionMap.setEnabled(false);
        jTextFrictionValue.setEnabled(true);
}//GEN-LAST:event_jRadioButtonFrictionValueActionPerformed

    private void jRadioButtonFrictionMapActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonFrictionMapActionPerformed
        jTextFrictionValue.setEnabled(false);
        jComboBoxFrictionMap.setEnabled(true);
}//GEN-LAST:event_jRadioButtonFrictionMapActionPerformed

    private void jRadioButtonSoilKValueActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKValueActionPerformed
        jComboBoxSoilKMap.setEnabled(false);
        jTextSoilKValue.setEnabled(true);
}//GEN-LAST:event_jRadioButtonSoilKValueActionPerformed

    private void jRadioButtonSoilKMapActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jRadioButtonSoilKMapActionPerformed
        jTextSoilKValue.setEnabled(false);
        jComboBoxSoilKMap.setEnabled(true);
}//GEN-LAST:event_jRadioButtonSoilKMapActionPerformed

    private void jTextFieldnumOfOvicapridsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldnumOfOvicapridsActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldnumOfOvicapridsActionPerformed

    private void jTextFieldsheepRatioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldsheepRatioActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldsheepRatioActionPerformed

    private void jTextFieldgoatRatioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldgoatRatioActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldgoatRatioActionPerformed

    private void jTextFieldgrazingDensityActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldgrazingDensityActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldgrazingDensityActionPerformed

    private void jTextFieldRC6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldRC6ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldRC6ActionPerformed

    private void jButtonRC6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRC6ActionPerformed
    	try
	    {
	        int result = jFileChooserRC6.showOpenDialog(this);
	
	        if ( result == jFileChooserRC6.APPROVE_OPTION )
	        {
	            String path = jFileChooserRC6.getSelectedFile().getCanonicalPath();
	            jTextFieldRC6.setText(path);
	            File dir = new File(path);
	           
	        }
	    }
	    
	    catch (java.io.IOException ioe)
	    {
	        
	    }
    	
    }//GEN-LAST:event_jButtonRC6ActionPerformed

    private void jTextFieldSoilMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldSoilMinActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldSoilMinActionPerformed

    private void jTextFieldSheepFodderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldSheepFodderActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldSheepFodderActionPerformed

    private void jTextFieldSheepYieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldSheepYieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldSheepYieldActionPerformed

    private void jTextFieldClimFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldClimFileActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldClimFileActionPerformed

    private void jButtonClimFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonClimFileActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jButtonClimFileActionPerformed

    private void jPanelHouseholdsInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_jPanelHouseholdsInputMethodTextChanged
        // TODO add your handling code here:
    }//GEN-LAST:event_jPanelHouseholdsInputMethodTextChanged

    private void jSliderYieldExpScalarStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderYieldExpScalarStateChanged
        jTextYieldExpScalar.setText(String.valueOf(jSliderYieldExpScalar.getValue()));
    }//GEN-LAST:event_jSliderYieldExpScalarStateChanged

    private void jTextMaxFarmDistanceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextMaxFarmDistanceActionPerformed
        String max_dist_str = jTextMaxFarmDistance.getText();

        if ( max_dist_str.compareTo("") == 0 )
            jTextMaxFarmDistance.setText("1");

        else {
            try {
                int max_dist = Integer.parseInt(max_dist_str);

                if ( max_dist < 1 )
                    jTextMaxFarmDistance.setText("1");
            }

            catch (NumberFormatException nfe) {
                jTextMaxFarmDistance.setText("1");
            }
        }
    }//GEN-LAST:event_jTextMaxFarmDistanceActionPerformed

    private void jTextYieldExpScalarActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextYieldExpScalarActionPerformed
        float currValue = 0.0f;

        try {
            currValue = Float.parseFloat(jTextYieldExpScalar.getText());

            if ( currValue < 0.0f )
                currValue = 0.0f;

            else if ( currValue > 200.0f )
                currValue = 200.0f;
        }

        catch (NumberFormatException nfe) {
            currValue = 75.0f;
        }

        finally {
            jSliderYieldExpScalar.setValue((int) Math.floor(currValue));
        }
    }//GEN-LAST:event_jTextYieldExpScalarActionPerformed

    private void jSpinnerDeathProbMaxStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerDeathProbMaxStateChanged
        //SpinnerNumberModel initModel = (SpinnerNumberModel) jSpinnerDeathInitProb.getModel();
        SpinnerNumberModel minModel = (SpinnerNumberModel) jSpinnerDeathProbMin.getModel();
        SpinnerNumberModel maxModel = (SpinnerNumberModel) jSpinnerDeathProbMax.getModel();

       /* if ( maxModel.getNumber().intValue() < initModel.getNumber().intValue() )
            initModel.setValue(maxModel.getNumber());

        else if ( maxModel.getNumber().intValue() < minModel.getNumber().intValue() )
            minModel.setValue(maxModel.getNumber());
            */
    }//GEN-LAST:event_jSpinnerDeathProbMaxStateChanged

    private void jSpinnerDeathProbMinStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerDeathProbMinStateChanged
        //SpinnerNumberModel initModel = (SpinnerNumberModel) jSpinnerDeathInitProb.getModel();
        SpinnerNumberModel minModel = (SpinnerNumberModel) jSpinnerDeathProbMin.getModel();
        SpinnerNumberModel maxModel = (SpinnerNumberModel) jSpinnerDeathProbMax.getModel();
/*
        if ( minModel.getNumber().intValue() > initModel.getNumber().intValue() )
            initModel.setValue(minModel.getNumber());

        else if ( minModel.getNumber().intValue() > maxModel.getNumber().intValue() )
            maxModel.setValue(minModel.getNumber());
            */
    }//GEN-LAST:event_jSpinnerDeathProbMinStateChanged

    private void jSpinnerBirthProbMaxStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerBirthProbMaxStateChanged
        //SpinnerNumberModel initModel = (SpinnerNumberModel) jSpinnerBirthInitProb.getModel();
        SpinnerNumberModel minModel = (SpinnerNumberModel) jSpinnerBirthProbMin.getModel();
        SpinnerNumberModel maxModel = (SpinnerNumberModel) jSpinnerBirthProbMax.getModel();
/*
        if ( maxModel.getNumber().intValue() < initModel.getNumber().intValue() )
            initModel.setValue(maxModel.getNumber());

        else if ( maxModel.getNumber().intValue() < minModel.getNumber().intValue() )
            minModel.setValue(maxModel.getNumber());
            */
    }//GEN-LAST:event_jSpinnerBirthProbMaxStateChanged

    private void jTextManDaysPerPersonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextManDaysPerPersonActionPerformed
        String mdpp_str = jTextManDaysPerPerson.getText();

        if ( mdpp_str.compareTo("") == 0 )
            jTextManDaysPerPerson.setText("1");

        else {
            try {
                int mdpp = Integer.parseInt(mdpp_str);

                if ( mdpp < 1 )
                    jTextManDaysPerPerson.setText("1");
            }

            catch (NumberFormatException nfe) {
                jTextManDaysPerPerson.setText("1");
            }
        }
    }//GEN-LAST:event_jTextManDaysPerPersonActionPerformed

    private void jPanelWoodInputMethodTextChanged(java.awt.event.InputMethodEvent evt) {//GEN-FIRST:event_jPanelWoodInputMethodTextChanged
        // TODO add your handling code here:
}//GEN-LAST:event_jPanelWoodInputMethodTextChanged

    private void jTextYieldExpScalar2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextYieldExpScalar2ActionPerformed
        // TODO add your handling code here:
}//GEN-LAST:event_jTextYieldExpScalar2ActionPerformed

    private void jTextFieldInitGrazingReturnActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldInitGrazingReturnActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldInitGrazingReturnActionPerformed

    private void jTextFieldPctBarleyFodderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldPctBarleyFodderActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldPctBarleyFodderActionPerformed

    private void jTextFieldGrazingDistWightActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldGrazingDistWightActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldGrazingDistWightActionPerformed

    private void jTextFieldLcovDistWightActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldLcovDistWightActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldLcovDistWightActionPerformed

    private void jTextFieldSoilFtyWightActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldSoilFtyWightActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldSoilFtyWightActionPerformed

    private void jTextFieldDepthWghtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldDepthWghtActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldDepthWghtActionPerformed

    private void jTextFieldFarmDistWghtActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldFarmDistWghtActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldFarmDistWghtActionPerformed

    private void jTextMaxGrazeDistanceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextMaxGrazeDistanceActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextMaxGrazeDistanceActionPerformed

    private void jTextFieldSeedPctActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldSeedPctActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldSeedPctActionPerformed

    private void jSpinnerBirthInitProbActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jSpinnerBirthInitProbActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jSpinnerBirthInitProbActionPerformed

    private void jSpinnerBirthProbDeltaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jSpinnerBirthProbDeltaActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jSpinnerBirthProbDeltaActionPerformed

    private void jSpinnerBirthProbMinStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerBirthProbMinStateChanged
        //SpinnerNumberModel initModel = (SpinnerNumberModel) jSpinnerBirthInitProb.getModel();
        SpinnerNumberModel minModel = (SpinnerNumberModel) jSpinnerBirthProbMin.getModel();
        SpinnerNumberModel maxModel = (SpinnerNumberModel) jSpinnerBirthProbMax.getModel();
/*
        if ( minModel.getNumber().intValue() > initModel.getNumber().intValue() )
            initModel.setValue(minModel.getNumber());

        else if ( minModel.getNumber().intValue() > maxModel.getNumber().intValue() )
            maxModel.setValue(minModel.getNumber());
            */
}//GEN-LAST:event_jSpinnerBirthProbMinStateChanged

    private void jSpinnerDeathInitProbActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jSpinnerDeathInitProbActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jSpinnerDeathInitProbActionPerformed

    private void jSpinnerDeathProbDeltaActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jSpinnerDeathProbDeltaActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jSpinnerDeathProbDeltaActionPerformed

    private void jTextDateStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextDateStartActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextDateStartActionPerformed

    private void jTextHHSizeLimitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextHHSizeLimitActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextHHSizeLimitActionPerformed

    private void jTextAgpastKcalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextAgpastKcalActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextAgpastKcalActionPerformed

    private void jTextAgriProportionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextAgriProportionActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextAgriProportionActionPerformed

    private void jTextPastoralProportionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextPastoralProportionActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextPastoralProportionActionPerformed

    /**
	 * @param args the command line arguments
	 */
/*	public static void main(String args[]) {
	    java.awt.EventQueue.invokeLater(new Runnable() {
	        public void run() {
	            new ModelInitialization().setVisible(true);
	        }
	    });
	}
*/
    private Object makeObj ( final String item )
    {
        return new Object() { public String toString() { return item; } };
        
    }

    private void ConfigurationLoad(File file)
    {// LOAD!
      boolean load_error = false;
      String data = "";
      String[] sdata = (String[])null;
      try
      {
        BufferedReader in = new BufferedReader(new FileReader(file));

        data = in.readLine();
        while ((data != null) && (data.compareTo("[VERSION]") != 0)) {
          data = in.readLine();
        }
        if (data == null)
          throw new IllegalArgumentException("Version data expected but not found.\n" + data.toString());
        data = in.readLine();
        sdata = data.split("\\|\\|");

        if (sdata.length != 2) {
          throw new IllegalArgumentException("Invalid version data.");
        }
        if ((sdata[0].compareTo("5.25") != 0) || (sdata[1].compareTo("May 2012") != 0)) {
          throw new IllegalArgumentException("Invalid version data.");
        }
        data = in.readLine();
        if (data.compareTo("[VILLAGES]") != 0) {
          throw new IllegalArgumentException("Village data expected but not found.\n" + data.toString());
        }

        data = in.readLine();
        int num_villages = Integer.parseInt(data);
        VillageClearAllData();
        this.jTextNumVillages.setText(data);

        for (int v = 0; v < num_villages; v++)
        {
          data = in.readLine();
          sdata = data.split("\\|\\|");

          if (sdata.length != 6) {
            throw new IllegalArgumentException("Invalid village data row.");
          }
          Object[] row_data = { sdata[0], 
            Float.valueOf(sdata[1]), 
            Float.valueOf(sdata[2]), 
            Integer.valueOf(sdata[3]), 
            Integer.valueOf(sdata[4]), 
            Integer.valueOf(sdata[5]) };

          VillageAddRowData(row_data);
        }

        if (num_villages > 0)
        {
          this.jButtonVillageRemove.setEnabled(true);
          this.jButtonVillageClearData.setEnabled(true);
        }
        else
        {
          this.jButtonVillageRemove.setEnabled(false);
          this.jButtonVillageClearData.setEnabled(false);
        }

        data = in.readLine();
        sdata = data.split("\\|\\|");

        if (sdata.length != 3) {
          throw new IllegalArgumentException("Invalid agent model NS coordinate/resolution data. " + sdata.length);
        }
        for (int d = 0; d < 3; d++) {
          if (sdata[d].compareTo(" ") == 0)
            sdata[d] = "";
        }
        this.jTextHumanNorthCoords.setText(sdata[0]);
        this.jTextHumanSouthCoords.setText(sdata[1]);

        data = in.readLine();
        sdata = data.split("\\|\\|");

        if (sdata.length != 3) {
          throw new IllegalArgumentException("Invalid agent model EW coordinate/resolution data.");
        }
        for (int d = 0; d < 3; d++) {
          if (sdata[d].compareTo(" ") == 0)
            sdata[d] = "";
        }
        this.jTextHumanEastCoords.setText(sdata[0]);
        this.jTextHumanWestCoords.setText(sdata[1]);

        data = in.readLine();
        sdata = data.split("\\|\\|");

        if (sdata.length != 2) {
          throw new IllegalArgumentException("Invalid agent model checkbox data.");
        }
        this.jCheckBoxEnvironmentCoordinates.setSelected(Boolean.valueOf(sdata[0].toLowerCase()).booleanValue());
        this.jCheckBoxEnvironmentResolutions.setSelected(Boolean.valueOf(sdata[1].toLowerCase()).booleanValue());

        boolean enable = !this.jCheckBoxEnvironmentCoordinates.isSelected();
        this.jLabelHumanNCoord.setEnabled(enable);
        this.jTextHumanNorthCoords.setEnabled(enable);
        this.jLabelHumanSCoord.setEnabled(enable);
        this.jTextHumanSouthCoords.setEnabled(enable);
        this.jLabelHumanECoord.setEnabled(enable);
        this.jTextHumanEastCoords.setEnabled(enable);
        this.jLabelHumanWCoord.setEnabled(enable);
        this.jTextHumanWestCoords.setEnabled(enable);

        enable = !this.jCheckBoxEnvironmentResolutions.isSelected();

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextVillageNumHouseholds.setText(sdata[0]);
        this.jTextVillageHhStartPopulation.setText(sdata[1]);

        data = in.readLine();

        if (data.compareTo("[HOUSEHOLDS]") != 0) {
          throw new IllegalArgumentException("Household data expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jSpinnerBirthInitProb.setText(sdata[0]);
        SetSpinnerValue(this.jSpinnerBirthProbPerPeople, Integer.valueOf(sdata[1]));

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jSpinnerBirthProbDelta.setText(sdata[0]);
        SetSpinnerValue(this.jSpinnerBirthProbMin, Integer.valueOf(sdata[1]));
        SetSpinnerValue(this.jSpinnerBirthProbMax, Integer.valueOf(sdata[2]));

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jSpinnerDeathInitProb.setText(sdata[0]);
        SetSpinnerValue(this.jSpinnerDeathProbPerPeople, Integer.valueOf(sdata[1]));

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jSpinnerDeathProbDelta.setText(sdata[0]);
        SetSpinnerValue(this.jSpinnerDeathProbMin, Integer.valueOf(sdata[1]));
        SetSpinnerValue(this.jSpinnerDeathProbMax, Integer.valueOf(sdata[2]));

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextManDaysPerPerson.setText(sdata[0]);
        this.jTextMaxFarmDistance.setText(sdata[1]);
        this.jTextYieldExpScalar.setText(sdata[2]);
        SetSpinnerValue(this.jSpinnerPopulationPctLabor, Integer.valueOf(sdata[3]));
        this.jTextVillagePctInitClearing.setText(sdata[4]);
        this.jComboBoxHHSizeLimit.setSelectedIndex(Integer.valueOf(sdata[5]));
        this.jTextHHSizeLimit.setText(sdata[6]);
        jTextYieldExpScalarActionPerformed(null);

        data = in.readLine();

        if (data.compareTo("[RESOURCES]") != 0) {
          throw new IllegalArgumentException("GRASS resource data expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");

        this.jTextWheatLaborRqd.setText(sdata[0]);
        this.jTextWheatInitYield.setText(sdata[1]);
        this.jTextWheatCalories.setText(sdata[2]);

        data = in.readLine();
        sdata = data.split("\\|\\|");

        this.jTextBarleyLaborRqd.setText(sdata[0]);
        this.jTextBarleyInitYield.setText(sdata[1]);

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldMaxWheatYield.setText(sdata[0]);
        this.jTextFieldMaxBarleyYield.setText(sdata[1]);
        
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldFarmDistWght.setText(sdata[0]);
        this.jTextFieldSoilFtyWight.setText(sdata[1]);
        this.jTextFieldDepthWght.setText(sdata[2]);
        this.jTextFieldSeedPct.setText(sdata[3]);
        
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextAgpastKcal.setText(sdata[0]);
        this.jTextAgriProportion.setText(sdata[1]);
        this.jTextPastoralProportion.setText(sdata[2]);
        this.jComboBoxLandTenure.setSelectedIndex(Integer.valueOf(sdata[3]));
        
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldBpointLC1.setText(sdata[0]);
        this.jTextFieldBpointLC2.setText(sdata[1]);
        this.jTextFieldBpointDA1.setText(sdata[2]);
        this.jTextFieldBpointDA2.setText(sdata[3]);
        
        data = in.readLine();
        if (data.compareTo("[GRASS ENVIRONMENT]") != 0) {
          throw new IllegalArgumentException("GRASS environment data expected but not found.\n" + data.toString());
        }
        String gisdata = in.readLine();
        data = in.readLine();

        if (data.compareTo("[ENVIRONMENT INITIALIZATION MAPS]") != 0) {
          throw new IllegalArgumentException("GRASS initialization maps expected but not found.\n" + data.toString());
        }
        String mapsdata = in.readLine();
        String moremaps = in.readLine();
        String maps3 = in.readLine();

        data = in.readLine();

        if (data.compareTo("[INTERACTION TIMING]") != 0) {
          throw new IllegalArgumentException("GRASS initialization maps expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");

        this.jComboBoxDateChangeDir.setSelectedIndex(Integer.parseInt(sdata[1]));
        this.jTextDateChangeValue.setText(sdata[2]);
        this.jTextDateEnd.setText(sdata[3]);
        this.jTextDatePrefix.setText(sdata[4]);
        this.jTextDateSuffix.setText(sdata[5]);
        this.jTextDateStart.setText(sdata[6]);

        data = in.readLine();

        if (data.compareTo("[SYSTEM SETTINGS]") != 0) {
          throw new IllegalArgumentException("GRASS SYSTEM SETTINGS expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");

        this.jTextFieldAPSIM.setText(sdata[0]);
        this.jTextFieldGRASS.setText(sdata[1]);
        this.jTextFieldPython.setText(sdata[2]);
        this.jTextFieldRC6.setText(sdata[3]);
        this.jComboBoxWorldWind.setSelectedIndex(Integer.parseInt(sdata[4]));
        this.jComboBoxHouseholdLog.setSelectedIndex(Integer.parseInt(sdata[5]));
        this.jComboBoxHHEconLog.setSelectedIndex(Integer.parseInt(sdata[6]));
        this.jComboBoxIMLog.setSelectedIndex(Integer.parseInt(sdata[7]));
        this.jComboBoxTempMapFiles.setSelectedIndex(Integer.parseInt(sdata[8]));
        this.jComboBoxHHPopEconLog.setSelectedIndex(Integer.parseInt(sdata[9]));
        this.jComboBoxIMLogSave.setSelectedIndex(Integer.parseInt(sdata[10]));

        data = in.readLine();

        if (data.compareTo("[GRASS EVOL SETTINGS]") != 0) {
          throw new IllegalArgumentException("GRASS Evol Settings expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");
    
        this.jTextFieldsoilDensity.setText(sdata[0]);
        this.jTextFieldrFactor.setText(sdata[1]);
        this.jTextFieldrainValue.setText(sdata[2]);
        this.jTextFieldstreamTrans.setText(sdata[3]);
        this.jTextFieldloadExponent.setText(sdata[4]);
        this.jComboBoxSmoothing.setSelectedIndex(Integer.parseInt(sdata[5]));
        this.jTextFieldkappa.setText(sdata[6]);
        this.jTextFieldcutoff1.setText(sdata[7]);
        this.jTextFieldcutoff2.setText(sdata[8]);
        this.jTextFieldcutoff3.setText(sdata[9]);

        data = in.readLine();
        if (data.compareTo("[GRASS LANDSCAPE SETTINGS]") != 0) {
          throw new IllegalArgumentException("GRASS landscape settings expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldSoilMin.setText(sdata[0]);
        this.jTextFieldSoilMax.setText(sdata[1]);
        this.jTextFieldSoilRecovery.setText(sdata[2]);
        this.jTextFieldFertilityImpact.setText(sdata[3]);
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldMaxLcov.setText(sdata[0]);

        data = in.readLine();
        if (data.compareTo("[OVICAPRID VALUES]") != 0) {
          throw new IllegalArgumentException("GRASS ovicparid values expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");

        this.jTextFieldnumOfOvicaprids.setText(sdata[0]);
        this.jTextFieldsheepRatio.setText(sdata[1]);
        this.jTextFieldgoatRatio.setText(sdata[2]);
        this.jTextFieldgrazingDensity.setText(sdata[3]);
        this.jTextFieldGoatFodder.setText(sdata[4]);
        this.jTextFieldGoatYield.setText(sdata[5]);
        this.jTextFieldSheepFodder.setText(sdata[6]);
        this.jTextFieldSheepYield.setText(sdata[7]);
        this.jComboBoxFallFieldGrz.setSelectedIndex(Integer.parseInt(sdata[8]));

        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextFieldInitGrazingReturn.setText(sdata[0]);
        this.jTextFieldPctBarleyFodder.setText(sdata[1]);
        //this.jTextFieldPercentLactate.setText(sdata[2]);
        //this.jTextFieldPercentMeat.setText(sdata[3]);
        this.jTextFieldGrazingDistWight.setText(sdata[4]);
        this.jTextFieldLcovDistWight.setText(sdata[5]);

        data = in.readLine();
        if (data.compareTo("[CLIMATE AND WOOD GATHERING]") != 0) {
          throw new IllegalArgumentException("GRASS climate and wood gathering values expected but not found.\n" + data.toString());
        }
        data = in.readLine();
        sdata = data.split("\\|\\|");
        this.jTextWoodPc.setText(sdata[0]);
        this.jTextFieldWoodIntensity.setText(sdata[1]);
        this.jTextFieldClimFile.setText(sdata[2]);
        this.jComboBoxClimateSource.setSelectedIndex(Integer.parseInt(sdata[3]));
        this.jTextFieldStormEvents.setText(sdata[4]);
        this.jTextFieldStormSpeed.setText(sdata[5]);
        this.jTextFieldStormLength.setText(sdata[6]);
        this.jTextFieldStormPrecip.setText(sdata[7]);
        this.jTextFieldWoodDistanceWeight.setText(sdata[8]);
        
        //MOVED HERE
        
        sdata = gisdata.split("\\|\\|");
        this.jTextGISDBASE.setText(sdata[0]);

        if (this.jTextGISDBASE.getText().compareTo("") != 0)
        {
          String path = this.jTextGISDBASE.getText();
          File dir = new File(path);

          FileFilter dirFilter = new FileFilter()
          {
            public boolean accept(File dir) {
              return dir.isDirectory();
            }
          };
          File[] files = dir.listFiles(dirFilter);
          this.jComboBoxLocation.removeAllItems();
          this.jComboBoxLocation.addItem(makeObj("[Select a location]"));
          int index = 0;
          for (int f = 0; f < files.length; f++)
          {
            this.jComboBoxLocation.addItem(makeObj(files[f].getName()));
            if (files[f].getName().equals(sdata[1])) {
              index = this.jComboBoxLocation.getItemCount();
            }
          }

          this.jComboBoxLocation.setSelectedIndex(index - 1);
          this.jComboBoxLocation.setEnabled(true);
          this.jTextMapset.setEnabled(true);
          jComboBoxLocationItemStateChanged(null);
        }
        else
        {
          this.jComboBoxLocation.setEnabled(false);
          this.jComboBoxLocation.setSelectedIndex(-1);
          this.jTextMapset.setEnabled(false);
        }

        if (sdata.length == 3) {
          this.jTextMapset.setText(sdata[2]);
        }
        else {
          this.jTextMapset.setText("");
        }
        
        sdata = mapsdata.split("\\|\\|");

        if (this.jComboBoxLocation.getSelectedIndex() > 0)
        {
          int newIndex = 0;
          for (int index = 0; index < this.jComboBoxDEM.getItemCount(); index++) {
            String map = this.jComboBoxDEM.getItemAt(index).toString();
            if (sdata[0].equals(map)) {
              newIndex = index;
            }
          }
          this.jComboBoxDEM.setSelectedIndex(newIndex);
          newIndex = 0;
          System.out.println("Loaded map: "+sdata[1]);
          for (int index = 0; index < this.jComboBoxLandCover.getItemCount(); index++) {
            String map = this.jComboBoxLandCover.getItemAt(index).toString();
            System.out.println("LC map: "+map+" index"+index);
            if (sdata[1].equals(map)) {
              System.out.println("Sdata equaled map");
              newIndex = index;
            }
          }
          this.jComboBoxLandCover.setSelectedIndex(newIndex);
          newIndex = 0;

          for (int index = 0; index < this.jComboBoxFertility.getItemCount(); index++) {
            String map = this.jComboBoxFertility.getItemAt(index).toString();
            if (sdata[2].equals(map)) {
              newIndex = index;
            }
          }
          this.jComboBoxFertility.setSelectedIndex(newIndex);
        }
        
        sdata = moremaps.split("\\|\\|");

        if (sdata[0].compareTo("value") == 0)
        {
          this.jRadioButtonFrictionValue.setSelected(true);
          this.jRadioButtonFrictionMap.setSelected(false);
          this.jTextFrictionValue.setText(sdata[1]);
          this.jComboBoxFrictionMap.setSelectedIndex(-1);
          this.jComboBoxFrictionMap.setEnabled(false);

          if ((this.jComboBoxLocation.isEnabled()) && (this.jComboBoxLocation.getSelectedIndex() != 0)) {
            this.jTextFrictionValue.setEnabled(true);
          }
        }
        else if (sdata[0].compareTo("map") == 0)
        {
          this.jRadioButtonFrictionMap.setSelected(true);
          this.jRadioButtonFrictionValue.setSelected(false);

          int newIndex = 0;

          for (int index = 0; index < this.jComboBoxFrictionMap.getItemCount() - 1; index++) {
            String map = this.jComboBoxFrictionMap.getItemAt(index).toString();
            if (sdata[1].equals(map)) {
              newIndex = index;
            }
          }
          this.jComboBoxFrictionMap.setSelectedIndex(newIndex);
          this.jComboBoxFrictionMap.setEnabled(true);
          this.jTextFrictionValue.setText("1.0");
          this.jTextFrictionValue.setEnabled(false);

          if ((this.jComboBoxLocation.isEnabled()) && (this.jComboBoxLocation.getSelectedIndex() != 0))
            this.jComboBoxFrictionMap.setEnabled(true);
        }
        else
        {
          throw new IllegalArgumentException("Invalid radio button type.");
        }
        sdata = maps3.split("\\|\\|");

        if (sdata[0].compareTo("value") == 0)
        {
          this.jRadioButtonSoilKValue.setSelected(true);
          this.jRadioButtonSoilKMap.setSelected(false);
          this.jTextSoilKValue.setText(sdata[1]);
          this.jComboBoxSoilKMap.setSelectedIndex(-1);
          this.jComboBoxSoilKMap.setEnabled(false);

          if ((this.jComboBoxLocation.isEnabled()) && (this.jComboBoxLocation.getSelectedIndex() != 0)) {
            this.jTextSoilKValue.setEnabled(true);
          }
        }
        else if (sdata[0].compareTo("map") == 0)
        {
          this.jRadioButtonSoilKMap.setSelected(true);
          this.jRadioButtonSoilKValue.setSelected(false);

          int newIndex = 0;

          for (int index = 0; index < this.jComboBoxSoilKMap.getItemCount() - 1; index++) {
            String map = this.jComboBoxSoilKMap.getItemAt(index).toString();
            if (sdata[1].equals(map)) {
              newIndex = index;
            }
          }
          this.jComboBoxSoilKMap.setSelectedIndex(newIndex);

          this.jTextSoilKValue.setText("0.25");
          this.jTextSoilKValue.setEnabled(false);

          if ((this.jComboBoxLocation.isEnabled()) && (this.jComboBoxLocation.getSelectedIndex() != 0))
            this.jComboBoxSoilKMap.setEnabled(true);
        }
        else
        {
          throw new IllegalArgumentException("Invalid radio button type.");
        }
        
      }
      catch (Exception exc)
      {
        load_error = true;
        System.err.println(exc.toString());
      }
      finally
      {
        if (load_error)
          LoadError();
      }
    }

    private void ConfigurationSave(File file)
    {
      try
      {
        BufferedWriter out = new BufferedWriter(new FileWriter(file));
        out.write("********************************************************************"); out.newLine();
        out.write("**                                                                **"); out.newLine();
        out.write("** Save File for Model Initialization Configuration               **"); out.newLine();
        out.write("**                                                                **"); out.newLine();
        out.write("** Notes:                                                         **"); out.newLine();
        out.write("**                                                                **"); out.newLine();
        out.write("**                                                                **"); out.newLine();
        out.write("** DO NOT MODIFY THIS FILE MANUALLY BELOW THIS BOX;               **"); out.newLine();
        out.write("** DOING SO MAY AFFECT THE ABILITY TO RELOAD THE DATA.            **"); out.newLine();
        out.write("**                                                                **"); out.newLine();
        out.write("********************************************************************"); out.newLine();

        out.write("[VERSION]");
        out.newLine();
        out.write("5.25" + "||" + "May 2012");
        out.newLine();

        out.write("[VILLAGES]");
        out.newLine();
        int num_rows = this.jTableVillageData.getRowCount();
        String curr_name = "";
        Float curr_ns = null; Float curr_ew = null;
        Integer curr_clearing = null; Integer curr_hh = null; Integer curr_pop = null;

        out.write(String.valueOf(num_rows));
        out.newLine();

        for (int r = 0; r < num_rows; r++)
        {
          curr_name = (String)this.jTableVillageData.getValueAt(r, 0);
          curr_ns = (Float)this.jTableVillageData.getValueAt(r, 1);
          curr_ew = (Float)this.jTableVillageData.getValueAt(r, 2);
          curr_clearing = (Integer)this.jTableVillageData.getValueAt(r, 3);
          curr_hh = (Integer)this.jTableVillageData.getValueAt(r, 4);
          curr_pop = (Integer)this.jTableVillageData.getValueAt(r, 5);

          out.write(curr_name + "||" + curr_ns.floatValue() + "||" + curr_ew.floatValue() + "||" + curr_clearing.intValue());
          out.write("||" + curr_hh.intValue() + "||" + curr_pop.intValue());
          out.newLine();
        }

        String ns_data = "";

        if (this.jTextHumanNorthCoords.getText().compareTo("") != 0) {
          ns_data = ns_data + this.jTextHumanNorthCoords.getText();
        }
        else {
          ns_data = ns_data + " ";
        }
        if (this.jTextHumanSouthCoords.getText().compareTo("") != 0) {
          ns_data = ns_data + "||" + this.jTextHumanSouthCoords.getText();
        }
        else {
          ns_data = ns_data + "|| ";
        }

        ns_data = ns_data + "|| ";

        out.write(ns_data);
        out.newLine();

        String ew_data = "";

        if (this.jTextHumanEastCoords.getText().compareTo("") != 0) {
          ew_data = ew_data + this.jTextHumanEastCoords.getText();
        }
        else {
          ew_data = ew_data + " ";
        }
        if (this.jTextHumanWestCoords.getText().compareTo("") != 0) {
          ew_data = ew_data + "||" + this.jTextHumanWestCoords.getText();
        }
        else {
          ew_data = ew_data + "|| ";
        }

        ew_data = ew_data + "|| ";

        out.write(ew_data);
        out.newLine();

        boolean use_coords = this.jCheckBoxEnvironmentCoordinates.isSelected();
        boolean use_res = this.jCheckBoxEnvironmentResolutions.isSelected();
        out.write(String.valueOf(use_coords) + "||" + String.valueOf(use_res));
        out.newLine();
        out.write(this.jTextVillageNumHouseholds.getText() + "||" + this.jTextVillageHhStartPopulation.getText());
        out.newLine();

        out.write("[HOUSEHOLDS]");
        out.newLine();
        out.write(this.jSpinnerBirthInitProb.getText() + "||" + this.jSpinnerBirthProbPerPeople.getValue());
        out.newLine();
        out.write(this.jSpinnerBirthProbDelta.getText() + "||" + this.jSpinnerBirthProbMin.getValue() + "||" + this.jSpinnerBirthProbMax.getValue());
        out.newLine();
        out.write(this.jSpinnerDeathInitProb.getText() + "||" + this.jSpinnerDeathProbPerPeople.getValue());
        out.newLine();
        out.write(this.jSpinnerDeathProbDelta.getText() + "||" + this.jSpinnerDeathProbMin.getValue() + "||" + this.jSpinnerDeathProbMax.getValue());
        out.newLine();
        out.write(this.jTextManDaysPerPerson.getText() +  "||" + this.jTextMaxFarmDistance.getText() + "||" + this.jTextYieldExpScalar.getText() + 
        		"||" + this.jSpinnerPopulationPctLabor.getValue() + "||" + this.jTextVillagePctInitClearing.getText() + "||" + 
        		this.jComboBoxHHSizeLimit.getSelectedIndex() + "||" +this.jTextHHSizeLimit.getText());

        out.newLine();

        out.write("[RESOURCES]");
        out.newLine();
        out.write(this.jTextWheatLaborRqd.getText() + "||" + this.jTextWheatInitYield.getText() + "||" + this.jTextWheatCalories.getText());
        out.newLine();
        out.write(this.jTextBarleyLaborRqd.getText() + "||" + this.jTextBarleyInitYield.getText());
        out.newLine();
        out.write(this.jTextFieldMaxWheatYield.getText() + "||" + this.jTextFieldMaxBarleyYield.getText());
        out.newLine();
        out.write(this.jTextFieldFarmDistWght.getText() + "||" + this.jTextFieldSoilFtyWight.getText()+ "||" + this.jTextFieldDepthWght.getText()+ "||" + this.jTextFieldSeedPct.getText());
        out.newLine();
        out.write(this.jTextAgpastKcal.getText() + "||" + this.jTextAgriProportion.getText() + "||" + this.jTextPastoralProportion.getText()+ "||" + this.jComboBoxLandTenure.getSelectedIndex() );
        out.newLine();
        out.write(this.jTextFieldBpointLC1.getText() + "||" + this.jTextFieldBpointLC2.getText() + "||" + this.jTextFieldBpointDA1.getText()+ "||" + this.jTextFieldBpointDA2.getText());
        out.newLine();

        out.write("[GRASS ENVIRONMENT]");
        out.newLine();
        out.write(this.jTextGISDBASE.getText() + "||" + this.jComboBoxLocation.getSelectedItem().toString() + "||" + this.jTextMapset.getText());
        out.newLine();
        out.write("[ENVIRONMENT INITIALIZATION MAPS]");
        out.newLine();
        out.write(this.jComboBoxDEM.getSelectedItem().toString() + "||" + this.jComboBoxLandCover.getSelectedItem().toString() + "||" + this.jComboBoxFertility.getSelectedItem().toString());
        out.newLine();

        if (this.jRadioButtonFrictionValue.isSelected())
          out.write("value||" + this.jTextFrictionValue.getText());
        else {
          out.write("map||" + this.jComboBoxFrictionMap.getSelectedItem().toString());
        }
        out.newLine();

        if (this.jRadioButtonSoilKValue.isSelected())
          out.write("value||" + this.jTextSoilKValue.getText());
        else {
          out.write("map||" + this.jComboBoxSoilKMap.getSelectedItem().toString());
        }

        out.newLine();
        out.write("[INTERACTION TIMING]");
        out.newLine();
        out.write("0||" + this.jComboBoxDateChangeDir.getSelectedIndex() + "||" + this.jTextDateChangeValue.getText() + 
          "||" + this.jTextDateEnd.getText() + "||" + this.jTextDatePrefix.getText() + "||" + this.jTextDateSuffix.getText() +"||" + this.jTextDateStart.getText());
        out.newLine();

        out.write("[SYSTEM SETTINGS]");
        out.newLine();
        out.write(this.jTextFieldAPSIM.getText() + "||" + this.jTextFieldGRASS.getText() + "||" + this.jTextFieldPython.getText() + "||" + this.jTextFieldRC6.getText() + 
          "||" + this.jComboBoxWorldWind.getSelectedIndex() + "||" + this.jComboBoxHouseholdLog.getSelectedIndex() + "||" + 
          this.jComboBoxHHEconLog.getSelectedIndex() + "||" + this.jComboBoxIMLog.getSelectedIndex() + "||" + this.jComboBoxTempMapFiles.getSelectedIndex() + "||"+
          this.jComboBoxHHPopEconLog.getSelectedIndex() + "||" + this.jComboBoxIMLogSave.getSelectedIndex());
        out.newLine();

        out.write("[GRASS EVOL SETTINGS]");
        out.newLine();
        out.write(this.jTextFieldsoilDensity.getText() + "||" + this.jTextFieldrFactor.getText() + "||" + this.jTextFieldrainValue.getText() + 
          "||" + this.jTextFieldstreamTrans.getText() + "||" + this.jTextFieldloadExponent.getText() + "||" + this.jComboBoxSmoothing.getSelectedIndex() + 
          "||" + this.jTextFieldkappa.getText() + "||" + this.jTextFieldcutoff1.getText() + "||" + this.jTextFieldcutoff2.getText() + "||" + 
          this.jTextFieldcutoff3.getText());
        out.newLine();

        out.write("[GRASS LANDSCAPE SETTINGS]");
        out.newLine();
        out.write(this.jTextFieldSoilMin.getText() + "||" + this.jTextFieldSoilMax.getText() + "||" + this.jTextFieldSoilRecovery.getText() + "||" + this.jTextFieldFertilityImpact.getText());
        out.newLine();
        out.write(this.jTextFieldMaxLcov.getText());
        out.newLine();

        out.write("[OVICAPRID VALUES]");
        out.newLine();
        out.write(this.jTextFieldnumOfOvicaprids.getText() + "||" + this.jTextFieldsheepRatio.getText() + "||" + this.jTextFieldgoatRatio.getText() + 
          "||" + this.jTextFieldgrazingDensity.getText() + "||" + this.jTextFieldGoatFodder.getText() + "||" + this.jTextFieldGoatYield.getText() + 
          "||" + this.jTextFieldSheepFodder.getText() + "||" + this.jTextFieldSheepYield.getText() + "||" + this.jComboBoxFallFieldGrz.getSelectedIndex());
        out.newLine();
        out.write(this.jTextFieldInitGrazingReturn.getText() + "||" + this.jTextFieldPctBarleyFodder.getText() + "||" + 
          "unused" + "||" + "unused" 
          //***************NEW***********************
          + "||" + this.jTextFieldGrazingDistWight.getText() + "||" + this.jTextFieldLcovDistWight.getText()	);
        out.newLine();

        out.write("[CLIMATE AND WOOD GATHERING]");
        out.newLine();
        out.write(this.jTextWoodPc.getText() + "||" + this.jTextFieldWoodIntensity.getText() + "||" + this.jTextFieldClimFile.getText() + 
          "||" + this.jComboBoxClimateSource.getSelectedIndex() + "||" + this.jTextFieldStormEvents.getText() + "||" + this.jTextFieldStormSpeed.getText() 
          + "||" + this.jTextFieldStormLength.getText()+ "||" + this.jTextFieldStormPrecip.getText()+ "||" + this.jTextFieldWoodDistanceWeight.getText());
        out.newLine();

        out.write("********************************************************************"); out.newLine();
        out.write("** END                                                            **"); out.newLine();
        out.write("********************************************************************"); out.newLine();

        out.close();
      }
      catch (IOException ioe)
      {
        file.delete();
      }
    }


    // returns the float value of a string or evalue if the 
    //  string can not be converted to a float
    private float GetFloat ( String svalue, float evalue )
    {
        float fvalue = evalue;

        try
        {
            fvalue = Float.valueOf(svalue).floatValue();
        }

        catch (NumberFormatException nfe)
        {
            fvalue = evalue;
        }

        return fvalue;

    }

    // returns the integer value of a string or evalue if the 
    //  string can not be converted to an integer
    private int GetInteger ( String svalue, int evalue )
    {
        int ivalue = evalue;

        try
        {
            ivalue = Integer.valueOf(svalue).intValue();
        }

        catch (NumberFormatException nfe)
        {
            ivalue = evalue;
        }

        return ivalue;

    }

    private void LoadError ()
    {
        javax.swing.JOptionPane.showMessageDialog(null, "There was an error loading the data. Some values may have been left unmodified or restored to default.\nCheck all values before executing the simulation.",
                "Error Loading Data", JOptionPane.WARNING_MESSAGE);

    }

    private void SetSpinnerValue ( javax.swing.JSpinner spinner, Number value )
    {
        SpinnerNumberModel spnModel = (SpinnerNumberModel) spinner.getModel();
        spnModel.setValue(value);

    }

    private void UpdateFullMapNameLabel ()
    {
        String fullname = "Full map name will be " + jTextDatePrefix.getText();

        for ( int ph = 0; ph < num_date_placeholders; ph++ )
            fullname += "#";

        fullname += jTextDateSuffix.getText() + "[MAP NAME]";

        jLabelFullMapName.setText(fullname);

    }

    private ArrayList ValidateEntries()
    {
      ArrayList errors = new ArrayList();

      String gisdbase = null;
      String location = null;
      String env_script = null;
      String dem = null;

      String grassloc = null;
      String pythonloc = null;
      String userpath = null;
      String gisbase = null;
      String grassrc6loc = null;
      String grasspath = null;

      if ((Double.parseDouble(this.jTextFieldsoilDensity.getText()) < 0.0D) || (Double.parseDouble(this.jTextFieldsoilDensity.getText()) > 5.0D)) {
        errors.add(new String("LANDSCAPE EVOLUTION SETTINGS: The soil density must be greater than 0 and less than or equal to 5."));
      }

      if ((Double.parseDouble(this.jTextFieldstreamTrans.getText()) != 0.001D) && (Double.parseDouble(this.jTextFieldstreamTrans.getText()) != 0.0001D) && 
        (Double.parseDouble(this.jTextFieldstreamTrans.getText()) != 1.E-05D) && (Double.parseDouble(this.jTextFieldstreamTrans.getText()) != 1.0E-06D))
      {
        errors.add(new String("LANDSCAPE EVOLUTION SETTINGS: infiltration must be 0.001, 0.0001, 0.00001, or 0.000001."));
      }
      if ((Double.parseDouble(this.jTextFieldloadExponent.getText()) != 1.5D) && (Double.parseDouble(this.jTextFieldloadExponent.getText()) != 2.5D))
      {
        errors.add(new String("LANDSCAPE EVOLUTION SETTINGS: Load Exponent  must be 1.5 or 2.5."));
      }

      if ((Double.parseDouble(this.jTextFieldkappa.getText()) < 0.0D) || (Double.parseDouble(this.jTextFieldkappa.getText()) > 10.0D)) {
        errors.add(new String("LANDSCAPE EVOLUTION SETTINGS: alpha must be between 0 and 10."));
      }
      if ((Double.parseDouble(this.jTextFieldcutoff1.getText()) > Double.parseDouble(this.jTextFieldcutoff2.getText())) || 
        (Double.parseDouble(this.jTextFieldcutoff2.getText()) > Double.parseDouble(this.jTextFieldcutoff3.getText()))) {
        errors.add(new String("LANDSCAPE EVOLUTION SETTINGS:Cutoff 1 must be less than cutoff 2 which must be less than cutoff 3."));
      }
      
      if (this.jComboBoxHHSizeLimit.getSelectedIndex() == 0) {
    	  if (this.jTextHHSizeLimit.getText().length() == 0) {
    		  errors.add(new String("HH SETTINGS: Household size limit must be greater than 0."));
          }
          else if (Double.parseDouble(this.jTextHHSizeLimit.getText()) <= 0.0D) {
            errors.add(new String("HH SETTINGS: THousehold size limit must be greater than 0."));}
      }
      
      if (this.jTextAgpastKcal.getText().length() == 0) {
          errors.add(new String("Resource SETTINGS: The amount of agropastoral kcal be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextAgpastKcal.getText()) <= 0.0D) {
          errors.add(new String("Resource SETTINGS: The amount of agropastoral kcal must be greater than 0."));
        }
      if (this.jTextPastoralProportion.getText().length() == 0) {
          errors.add(new String("Resource SETTINGS: The proportion of pastoral kcal be entered."));
        }
        else if (this.jTextAgriProportion.getText().length() == 0) {
          errors.add(new String("Resource SETTINGS: The proportion of agricultural kcal must be entered."));
        }
        else if (Integer.parseInt(this.jTextAgriProportion.getText()) + Integer.parseInt(this.jTextPastoralProportion.getText()) != 100 ) {
            errors.add(new String("Resource SETTINGS: The proportion of agricultural kcal and pastoral kcal must be equal to 100."));
          } 
      
      if (this.jTextFieldBpointLC1.getText().length() == 0) {
          errors.add(new String("FARMING SETTINGS: landcover breakpoint must be entered."));
        }
        else if (Double.parseDouble(this.jTextFieldBpointLC1.getText()) <= 5) {
          errors.add(new String("FARMING SETTINGS: The landcover breakpoint must be greater than 5."));
        }
      if (this.jTextFieldBpointLC2.getText().length() == 0) {
          errors.add(new String("FARMING SETTINGS: The second landcover breakpoint must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldBpointLC2.getText()) <= Double.parseDouble(this.jTextFieldBpointLC1.getText()) ) {
          errors.add(new String("FARMING SETTINGS: The second landcover breakpoint  must be greater than the first breakpoint."));
        }
      
      if (this.jTextFieldBpointDA1.getText().length() == 0) {
          errors.add(new String("FARMING SETTINGS: landcover breakpoint devaluation must be entered."));
        }
        else if (Double.parseDouble(this.jTextFieldBpointDA1.getText()) < 0 || Double.parseDouble(this.jTextFieldBpointDA1.getText()) > 100) {
          errors.add(new String("FARMING SETTINGS: The landcover breakpoint devaluation must be between 0 and 100."));
        }
      if (this.jTextFieldBpointDA2.getText().length() == 0) {
          errors.add(new String("FARMING SETTINGS: landcover breakpoint must be entered."));
        }
        else if (Double.parseDouble(this.jTextFieldBpointDA2.getText()) < 0 || Double.parseDouble(this.jTextFieldBpointDA2.getText()) > 100) {
          errors.add(new String("FARMING SETTINGS: The landcover breakpoint devaluation must be between 0 and 100."));
        } 
      
      if (this.jTextBarleyLaborRqd.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The amount of labor to farm barley must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextBarleyLaborRqd.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The amount of labor to farm barley  must be greater than 0."));
      }

      if (this.jTextBarleyInitYield.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The initial barley yield must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextBarleyInitYield.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The initial barley yield must be greater than 0."));
      }

      if (this.jTextWheatCalories.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The wheat calorie count must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextWheatCalories.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The wheat calorie count must be greater than 0."));
      }

      if (this.jTextWheatInitYield.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The initial wheat yield must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextWheatInitYield.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The initial wheat yield must be greater than 0."));
      }

      if (this.jTextWheatLaborRqd.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The WHEAT labor requiremenet must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextWheatLaborRqd.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The WHEAT labor requiremenet must be greater than 0."));
      }

      if (this.jTextFieldMaxBarleyYield.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The max barley yield must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldMaxBarleyYield.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The max barley yield must be greater than 0."));
      }

      if (this.jTextFieldMaxWheatYield.getText().length() == 0) {
        errors.add(new String("FARMING SETTINGS: The max wheat yield must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldMaxWheatYield.getText()) <= 0.0D) {
        errors.add(new String("FARMING SETTINGS: The max wheat yield must be greater than 0."));
      }

      if (this.jTextFieldLcovDistWight.getText().length() == 0) {
          errors.add(new String("landcover distance weight must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldLcovDistWight.getText()) <= 0.0D) {
          errors.add(new String("landcover distance weight must be greater than 0."));
        }
      if (this.jTextFieldFarmDistWght.getText().length() == 0) {
          errors.add(new String("farming distance weight must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldFarmDistWght.getText()) <= 0.0D) {
          errors.add(new String("farming distance weight must be greater than 0."));
        } 
      
      if (this.jTextFieldSoilFtyWight.getText().length() == 0) {
          errors.add(new String("soil fertility weight must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldSoilFtyWight.getText()) <= 0.0D) {
          errors.add(new String("soil fertility weight must be greater than 0."));
        } 
      if (this.jTextFieldDepthWght.getText().length() == 0) {
          errors.add(new String("soil field depth weight must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldDepthWght.getText()) <= 0.0D) {
          errors.add(new String("soil field depth weight must be greater than 0."));
        } 
      if (this.jTextMaxGrazeDistance.getText().length() == 0) {
          errors.add(new String("maximum grazing distance must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextMaxGrazeDistance.getText()) <= 0.0D) {
          errors.add(new String("maximum grazing distance must be greater than 0."));
        } 
      if (this.jTextFieldFertilityImpact.getText().length() == 0) {
          errors.add(new String("a value must be entered for the field fertility impact."));
        } 
      if (this.jTextFieldSoilRecovery.getText().length() == 0) {
          errors.add(new String("a value must be entered for the field soil recovery value."));
        } 
      if (this.jTextFieldSeedPct.getText().length() == 0) {
          errors.add(new String("field seed percentage must have a value entered 0."));
        }
        else if (Double.parseDouble(this.jTextFieldSeedPct.getText()) <= 0.0D) {
          errors.add(new String("field seed percentage must be greater than 0."));
        } 
      
      if (this.jTextFieldnumOfOvicaprids.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The number of ovicaprids per person must be greater than or equal to 0."));
      }
      else if (Double.parseDouble(this.jTextFieldnumOfOvicaprids.getText()) < 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The number of ovicaprids per person must be greater than or equal to 0."));
      }

      if (this.jTextFieldgoatRatio.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The number of goats in the sheep:goat ratio must be greater than or equal to 0."));
      }
      else if (Double.parseDouble(this.jTextFieldgoatRatio.getText()) < 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The number of goats in the sheep:goat ratio must be greater than or equal to 0."));
      }

      if (this.jTextFieldsheepRatio.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The number of sheep in the sheep:goat ratio must be greater than or equal to 0."));
      }
      else if (Double.parseDouble(this.jTextFieldsheepRatio.getText()) < 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The number of sheep in the sheep:goat ratio must be greater than or equal to 0."));
      }

      if (this.jTextFieldgrazingDensity.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The ovicaprid density must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldgrazingDensity.getText()) <= 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The ovicaprid density must be greater than 0."));
      }

      if (this.jTextFieldPctBarleyFodder.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The proportion of barley that makes up ovicaptid diet must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldPctBarleyFodder.getText()) <= 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The proportion of barley that makes up ovicaptid diet must be greater than 0."));
      }

      if (this.jTextFieldInitGrazingReturn.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The Initial Grazing Return must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldInitGrazingReturn.getText()) <= 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The Initial Grazing Return must be greater than 0."));
      }

      if (this.jTextFieldSheepYield.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The return from sheep must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldSheepYield.getText()) <= 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The return from sheep must be greater than 0."));
      }
      
      if (this.jTextFieldSheepYield.getText().length() == 0) {
          errors.add(new String("OVICAPRID SETTINGS: The return from sheep must be greater than 0."));
        }
        else if (Double.parseDouble(this.jTextFieldSheepYield.getText()) <= 0.0D) {
          errors.add(new String("OVICAPRID SETTINGS: The return from sheep must be greater than 0."));
        }

      if (this.jTextFieldGoatYield.getText().length() == 0) {
        errors.add(new String("OVICAPRID SETTINGS: The return from goats must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldGoatYield.getText()) <= 0.0D) {
        errors.add(new String("OVICAPRID SETTINGS: The return from goats must be greater than 0."));
      }

      if (this.jTextFieldWoodIntensity.getText().length() == 0) {
        errors.add(new String("WOOD GATHERING SETTINGS: Wood gathering intensity must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldWoodIntensity.getText()) <= 0.0D) {
        errors.add(new String("WOOD GATHERING SETTINGS: Wood gathering intensity must be greater than 0."));
      }
      if (this.jTextWoodPc.getText().length() == 0) {
        errors.add(new String("WOOD GATHERING SETTINGS: Wood gathering per capita must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextWoodPc.getText()) <= 0.0D) {
        errors.add(new String("WOOD GATHERING SETTINGS: Wood gathering per capita must be greater than 0."));
      }
      else if (Double.parseDouble(this.jTextFieldWoodDistanceWeight.getText()) <= 0.0D) {
          errors.add(new String("WOOD GATHERING SETTINGS: Wood distance weight must be greater than 0."));
        }

      if (this.jComboBoxClimateSource.getSelectedIndex() != 1) {
        if ((Double.parseDouble(this.jTextFieldrFactor.getText()) < 1.0D) || (Double.parseDouble(this.jTextFieldrFactor.getText()) > 25.0D)) {
          errors.add(new String("CLIMATE SETTINGS: Rfactor must be between 1 and 25."));
        }
        if (Double.parseDouble(this.jTextFieldStormEvents.getText()) > 365.0D) {
          errors.add(new String("CLIMATE SETTINGS: Storm Events per year must be less than 366. That would be very rainy..."));
        }
        if (Double.parseDouble(this.jTextFieldStormEvents.getText()) <= 0.0D) {
          errors.add(new String("CLIMATE SETTINGS: Rain days per year must be more than 0. That would be very dry..."));
        }
        if (Double.parseDouble(this.jTextFieldrainValue.getText()) <= 0.0D) {
          errors.add(new String("CLIMATE SETTINGS: Annual precipitation must be greater than 0"));
        }
        if (Double.parseDouble(this.jTextFieldStormLength.getText()) <= 0.0D) {
            errors.add(new String("CLIMATE SETTINGS: storm length must be greater than 0"));
          }
        if (Double.parseDouble(this.jTextFieldStormPrecip.getText()) <= 0.0D) {
            errors.add(new String("CLIMATE SETTINGS: storm precipitation must be greater than 0"));
          }

      }
      else if (this.jTextFieldClimFile.getText().length() == 0) {
        errors.add(new String("CLIMATE SETTINGS: The climate file must be specified."));
      }

      if (this.jComboBoxLocation.getSelectedItem().toString().length() == 0) {
        errors.add(new String("SYSTEM SETTINGS: The grass location must be specified."));
      }
      else
      {
        grassloc = this.jComboBoxLocation.getSelectedItem().toString();
      }

      if (this.jTextFieldPython.getText().length() == 0) {
        errors.add(new String("SYSTEM SETTINGS: The python directory must be specified."));
      }
      else
      {
        pythonloc = this.jTextFieldPython.getText();
        if (!pythonloc.endsWith(File.separator))
        {
          pythonloc = pythonloc.concat(File.separator);
          this.jTextFieldPython.setText(pythonloc);
        }
        File dir = new File(pythonloc);
        if (!dir.exists())
        {
          errors.add(new String("SYSTEM SETTINGS: The python directory does not exist."));
          pythonloc = null;
        }

      }

      if (this.jTextFieldAPSIM.getText().length() == 0) {
        errors.add(new String("SYSTEM SETTINGS: The USER directory must be specified."));
      }
      else
      {
        userpath = this.jTextFieldAPSIM.getText();
        if (!userpath.endsWith(File.separator))
        {
          userpath = userpath.concat(File.separator);
          this.jTextFieldAPSIM.setText(userpath);
        }
        File dir = new File(userpath);
        if (!dir.exists())
        {
          errors.add(new String("SYSTEM SETTINGS: The userpath directory does not exist."));
          userpath = null;
        }
      }

      if (this.jTextFieldRC6.getText().length() == 0) {
        errors.add(new String("SYSTEM SETTINGS: The grass .RC6 file could not be found."));
      }
      else
      {
        grassrc6loc = this.jTextFieldRC6.getText();
        if (!grassrc6loc.endsWith(File.separator))
        {
          grassrc6loc = grassrc6loc.concat(File.separator);
          this.jTextFieldRC6.setText(grassrc6loc);
        }

        File dir = new File(grassrc6loc);
        if (!dir.exists())
        {
          errors.add(new String("SYSTEM SETTINGS: The GRASS RC6 directory does not exist."));
          grassrc6loc = null;
        }
      }

      if (this.jTextFieldGRASS.getText().length() == 0) {
        errors.add(new String("SYSTEM SETTINGS: The GRASS directory must be specified."));
      }
      else
      {
        gisbase = this.jTextFieldGRASS.getText();

        if (!gisbase.endsWith(File.separator))
        {
          gisbase = gisbase.concat(File.separator);
          this.jTextFieldGRASS.setText(gisbase);
        }
        File dir = new File(gisbase);
        if (!dir.exists())
        {
          errors.add(new String("SYSTEM SETTINGS: The Grass 6.4 Folder does not exist."));
          gisbase = null;
        }
      }

      if (this.jTextGISDBASE.getText().length() == 0) {
        errors.add(new String("ENVIRONMENT MODEL: The GISDBASE directory must be specified."));
      }
      else
      {
        gisdbase = this.jTextGISDBASE.getText();

        if (!gisdbase.endsWith(File.separator))
        {
          gisdbase = gisdbase.concat(File.separator);
          this.jTextGISDBASE.setText(gisdbase);
        }
        grasspath = gisdbase;
        File dir = new File(grasspath);

        if (!dir.exists())
        {
          errors.add(new String("ENVIRONMENT MODEL: The GISDBASE directory does not exist."));
          grasspath = null;
        }
      }

      String permanent = "permanent";
      if ((this.jTextMapset.getText().length() == 0) || (permanent.equals(this.jTextMapset.getText().toLowerCase()))) {
        errors.add(new String("ENVIRONMENT MODEL: The MAPSET must be specified and must not be the Permanent directory."));
        errors.add(new String("ENVIRONMENT MODEL: If you are having trouble naming a mapset, try the GENERATE button."));
      }

      int index = this.jComboBoxLocation.getSelectedIndex();
      if ((index == 0) || (index == -1)) {
        errors.add(new String("ENVIRONMENT MODEL: The Location directory must be specified."));
      }
      else {
        location = String.valueOf(this.jComboBoxLocation.getSelectedItem());
      }
      index = this.jComboBoxDEM.getSelectedIndex();
      if ((index == 0) || (index == -1)) {
        errors.add(new String("ENVIRONMENT MODEL: The DEM must be specified."));
      }
      else {
        dem = String.valueOf(this.jComboBoxDEM.getSelectedItem());
      }
      index = this.jComboBoxLandCover.getSelectedIndex();
      if ((index == 0) || (index == -1)) {
        errors.add(new String("ENVIRONMENT MODEL: The LandCover map must be specified."));
      }
      if (this.jRadioButtonFrictionValue.isSelected())
      {
        if (this.jTextFrictionValue.getText().length() == 0) {
          errors.add(new String("ENVIRONMENT MODEL: Friction value selected with no value specified."));
        }
        else
        {
          float fvalue = GetFloat(this.jTextFrictionValue.getText(), -1.0F);
          if (fvalue < 0.0D) {
            errors.add(new String("ENVIRONMENT MODEL: Friction value must be positive value."));
          }
        }
      }
      else
      {
        index = this.jComboBoxFrictionMap.getSelectedIndex();
        if ((index == 0) || (index == -1)) {
          errors.add(new String("ENVIRONMENT MODEL: Friction map selected with no map specified."));
        }
      }
      if (this.jRadioButtonSoilKValue.isSelected())
      {
        if (this.jTextSoilKValue.getText().length() == 0) {
          errors.add(new String("ENVIRONMENT MODEL: Soil K-value selected with no value specified."));
        }
        else
        {
          float fvalue = GetFloat(this.jTextSoilKValue.getText(), -1.0F);
          if (fvalue < 0.0D) {
            errors.add(new String("ENVIRONMENT MODEL: Soil K-value must be positive value."));
          }
        }
      }
      else
      {
        index = this.jComboBoxSoilKMap.getSelectedIndex();
        if ((index == 0) || (index == -1)) {
          errors.add(new String("ENVIRONMENT MODEL: Soil K-value map selected with no map specified."));
        }
      }

      index = this.jComboBoxDateChangeDir.getSelectedIndex();
      if ((index < 0) || (index > 1)) {
        errors.add(new String("INTERACTION MODEL: Invalid date change direction specified."));
      }
      if (this.jTextDateChangeValue.getText().length() == 0) {
        errors.add(new String("INTERACTION MODEL: Specify the value to increment/decrement the date value."));
      }
      else
      {
        try
        {
          Integer.valueOf(this.jTextDateChangeValue.getText());
        }
        catch (NumberFormatException nfe)
        {
          errors.add(new String("INTERACTION MODEL: The value specified to increment/decrement the date is not valid; must be an integer."));
        }
      }

      if ((this.jTextDateEnd.getText().length() == 0) || (Integer.valueOf(this.jTextDateEnd.getText()).intValue() < 0)) {
        errors.add(new String("INTERACTION MODEL: End date invalid."));
      }
      else
      {
        try
        {
          Integer.valueOf(this.jTextDateEnd.getText());
        }
        catch (NumberFormatException nfe)
        {
          errors.add(new String("INTERACTION MODEL: End date is not valid; must be an integer."));
        }

      }
      
      if ((this.jTextDateStart.getText().length() == 0) ||  (Integer.valueOf(this.jTextDateStart.getText()).intValue() < 0)) {
          errors.add(new String("INTERACTION MODEL: Start date invalid."));
        }
        else
        {
          try
          {
            Integer.valueOf(this.jTextDateStart.getText());
          }
          catch (NumberFormatException nfe)
          {
            errors.add(new String("INTERACTION MODEL: Start date is not valid; must be an integer."));
          }

        }
      

      if (this.jTableVillageData.getRowCount() < 1) {
        errors.add(new String("AGENT MODEL: At least one village is required"));
      }
      else if (!this.jCheckBoxEnvironmentCoordinates.isSelected())
      {
        boolean coord_error = false;
        try
        {
          float agent_n_coord = Float.valueOf(this.jTextHumanNorthCoords.getText()).floatValue();
          float agent_s_coord = Float.valueOf(this.jTextHumanSouthCoords.getText()).floatValue();
          float agent_e_coord = Float.valueOf(this.jTextHumanEastCoords.getText()).floatValue();
          float agent_w_coord = Float.valueOf(this.jTextHumanWestCoords.getText()).floatValue();

          if (agent_n_coord <= agent_s_coord)
          {
            errors.add(new String("AGENT MODEL: Invalid agent coordinates; north coordinate must be larger than south coordinate."));
            coord_error = true;
          }

          if (agent_e_coord <= agent_w_coord)
          {
            errors.add(new String("AGENT MODEL: Invalid agent coordinates; east coordinate must be larger than west coordinate."));
            coord_error = true;
          }

          int num_rows = this.jTableVillageData.getRowCount();
          if ( !coord_error && num_rows > 0 )
          {
              javax.swing.table.DefaultTableModel model = (javax.swing.table.DefaultTableModel) jTableVillageData.getModel();

              // Check if each village coordinate lies within range
              for ( int r = 0; r < num_rows; r++ )
              {
                  float ns_coord = ((Float) model.getValueAt(r, 1)).floatValue();
                  float ew_coord = ((Float) model.getValueAt(r, 2)).floatValue();

                  if ( (ns_coord > agent_n_coord) || (ns_coord < agent_s_coord) ||
                       (ew_coord > agent_e_coord) || (ew_coord < agent_w_coord) )
                  {
                      errors.add(new String("AGENT MODEL: Invalid coordinates for village " + r + "; coordinates must be within specified region."));
                  }
              }
          }
        }
        catch (NumberFormatException nfe)
        {
          errors.add(new String("AGENT MODEL: Invalid agent coordinates; must be a float value."));
        }

      }
      else if ((dem != null) && (grassloc != null) && (pythonloc != null) && (userpath != null) && (gisbase != null) && (grassrc6loc != null) && (grasspath != null))
      {
        GrassFacade grass = null;

        String mapset = "PERMANENT";
        try
        {
          grass = GrassFacade.getInstance(this, gisbase, grassrc6loc, grasspath, grassloc, mapset);
          grass.GROUP = System.getProperty("user.name");
        }
        catch (Exception exc)
        {
          errors.add(exc.getMessage());
        }

        grass.setExecutablePath(new File(gisbase + "bin" + File.separator));
        grass.setScriptsPath(new File(userpath + "Scripts" + File.separator));
        grass.setPythonPath(new File(pythonloc));

        String map_name = dem;
        boolean run_quiet = false;
        char[] flags = { 'g' };
        try
        {
          String output = grass.rInfo(flags, map_name);
          String coord = output;
          grass.destroy(this);
          String delims = "north=";
          String[] coords = coord.split(delims);
          coord = coords[1];
          delims = "south=";
          coords = coord.split(delims);
          double nbound = Double.parseDouble(coords[0]);
          coord = coords[1];
          delims = "east=";
          coords = coord.split(delims);
          double sbound = Double.parseDouble(coords[0]);
          coord = coords[1];
          delims = "west=";
          coords = coord.split(delims);
          double ebound = Double.parseDouble(coords[0]);
          double wbound = Double.parseDouble(coords[1]);

          float northbound = (float)nbound;
          float southbound = (float)sbound;
          float eastbound = (float)ebound;
          float westbound = (float)wbound;

          int num_rows = this.jTableVillageData.getRowCount();
          for (int r = 0; r < num_rows; r++)
          {
            String vname = (String)this.jTableVillageData.getValueAt(r, 0);
            float n_s = ((Float)this.jTableVillageData.getValueAt(r, 1)).floatValue();
            float e_w = ((Float)this.jTableVillageData.getValueAt(r, 2)).floatValue();
            int srnd = ((Integer)this.jTableVillageData.getValueAt(r, 3)).intValue();
            int hh = ((Integer)this.jTableVillageData.getValueAt(r, 4)).intValue();
            int pop = ((Integer)this.jTableVillageData.getValueAt(r, 5)).intValue();

            if ((n_s < southbound) || (n_s > northbound)) {
              errors.add(new String("AGENT MODEL: Invalid village region for village, " + vname + ". DEM N is " + northbound + " and S is " + southbound));
            }
            if ((e_w < westbound) || (e_w > eastbound)) {
              errors.add(new String("E_W:" + e_w + ">" + westbound + "  E_W:" + e_w + "<" + eastbound));
              errors.add(new String("AGENT MODEL: Invalid village region for village, " + vname + ". DEM E is " + eastbound + " and W is " + westbound));
            }
            if (srnd < 0) {
              errors.add(new String("AGENT MODEL: village (" + vname + ") surround value may not be negative"));
            }
            if (hh < 0) {
              errors.add(new String("AGENT MODEL: number of village (" + vname + ") households may not be negative"));
            }
            if (pop < 0) {
              errors.add(new String("AGENT MODEL: village (" + vname + ") household population may not be negative"));
            }
          }
        }
        catch (NumberFormatException nfe)
        {
          errors.add(new String("AGENT MODEL: Invalid DEM regions read from map."));
        }
        catch (IOException localIOException)
        {
        }

      }
      else
      {
        errors.add(new String("AGENT MODEL: Cannot validate village data without a valid DEM and valid System Settings."));
      }

      label4064: if (!this.jCheckBoxEnvironmentResolutions.isSelected());
      try
      {
        int num_hh = Integer.valueOf(this.jTextVillageNumHouseholds.getText()).intValue();

        if (num_hh < 0) {
          errors.add(new String("AGENT MODEL: Number of households must be an integer of zero or greater"));
        }
      }
      catch (NumberFormatException nfe)
      {
        errors.add(new String("AGENT MODEL: Invalid number of households; must be an integer of zero or greater"));
      }

      try
      {
        int start_pop = Integer.valueOf(this.jTextVillageHhStartPopulation.getText()).intValue();

        if (start_pop < 1) {
          errors.add(new String("AGENT MODEL: Starting population must be an integer of one or greater"));
        }
      }
      catch (NumberFormatException nfe)
      {
        errors.add(new String("AGENT MODEL: Invalid starting population; must be an integer of one or greater"));
      }

      try
      {
        int mandays_pp = Integer.valueOf(this.jTextManDaysPerPerson.getText()).intValue();

        if (mandays_pp < 1) {
          errors.add(new String("AGENT MODEL: Man-days per person must be an integer of one or greater"));
        }
      }
      catch (NumberFormatException nfe)
      {
        errors.add(new String("AGENT MODEL: Invalid fields per person; must be an integer of one or greater"));
      }

      try
      {
        int max_dist = Integer.valueOf(this.jTextMaxFarmDistance.getText()).intValue();

        if (max_dist < 1) {
          errors.add(new String("AGENT MODEL: Maximum distance to travel must be an integer of one or greater"));
        }
      }
      catch (NumberFormatException nfe)
      {
        errors.add(new String("AGENT MODEL: Invalid maximum distance to travel; must be an integer of one or greater"));
      }

      return errors;
    }


    private void VillageAddRowData ( Object[] row_data )
    {
        javax.swing.table.DefaultTableModel model = (javax.swing.table.DefaultTableModel) jTableVillageData.getModel();
        model.addRow(row_data);
        
    }
    
    private void VillageClearAllData ()
    {
        int num_rows = jTableVillageData.getRowCount();
        javax.swing.table.DefaultTableModel model = (javax.swing.table.DefaultTableModel) jTableVillageData.getModel();
        
        // Remove all of the village data from the table
        for ( int r = 0; r < num_rows; r++ )
            model.removeRow(0);
        
    }
    
    /** REMOVE FOR ECLIPSE **/
    /**
    * @param args the command line arguments
    */
/*
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new ModelInitialization().setVisible(true);
            }
        });
    }
*/    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.ButtonGroup btnGroupFrictionMap;
    javax.swing.ButtonGroup btnGroupSoilKMap;
    javax.swing.JButton jButtonAPSIM;
    javax.swing.JButton jButtonBrowseRootDir;
    javax.swing.JButton jButtonBrowseRootDir1;
    javax.swing.JButton jButtonBrowseRootDir2;
    javax.swing.JButton jButtonClimFile;
    javax.swing.JButton jButtonConfigCancel;
    javax.swing.JButton jButtonConfigInitialize;
    javax.swing.JButton jButtonConfigLoad;
    javax.swing.JButton jButtonConfigSave;
    javax.swing.JButton jButtonConfigValidate;
    javax.swing.JButton jButtonGRASS;
    javax.swing.JButton jButtonGenerateMapset;
    javax.swing.JButton jButtonGenerateMapset1;
    javax.swing.JButton jButtonGenerateMapset2;
    javax.swing.JButton jButtonPython;
    javax.swing.JButton jButtonRC6;
    javax.swing.JButton jButtonVillageAdd;
    javax.swing.JButton jButtonVillageClearData;
    javax.swing.JButton jButtonVillageRemove;
    javax.swing.JCheckBox jCheckBoxEnvironmentCoordinates;
    javax.swing.JCheckBox jCheckBoxEnvironmentResolutions;
    javax.swing.JComboBox jComboBoxBedrock;
    javax.swing.JComboBox jComboBoxClimateSource;
    javax.swing.JComboBox jComboBoxDEM;
    javax.swing.JComboBox jComboBoxDEM1;
    javax.swing.JComboBox jComboBoxDEM2;
    javax.swing.JComboBox jComboBoxDateChangeDir;
    javax.swing.JComboBox jComboBoxFallFieldGrz;
    javax.swing.JComboBox jComboBoxFertility;
    javax.swing.JComboBox jComboBoxFrictionMap;
    javax.swing.JComboBox jComboBoxFrictionMap1;
    javax.swing.JComboBox jComboBoxFrictionMap2;
    javax.swing.JComboBox jComboBoxHHEconLog;
    javax.swing.JComboBox jComboBoxHHPopEconLog;
    javax.swing.JComboBox jComboBoxHHSizeLimit;
    javax.swing.JComboBox jComboBoxHouseholdLog;
    javax.swing.JComboBox jComboBoxIMLog;
    javax.swing.JComboBox jComboBoxIMLogSave;
    javax.swing.JComboBox jComboBoxLandCover;
    javax.swing.JComboBox jComboBoxLandCover1;
    javax.swing.JComboBox jComboBoxLandCover2;
    javax.swing.JComboBox jComboBoxLandTenure;
    javax.swing.JComboBox jComboBoxLocation;
    javax.swing.JComboBox jComboBoxLocation1;
    javax.swing.JComboBox jComboBoxLocation2;
    javax.swing.JComboBox jComboBoxMaxLandCover;
    javax.swing.JComboBox jComboBoxRatio;
    javax.swing.JComboBox jComboBoxScale;
    javax.swing.JComboBox jComboBoxSmoothing;
    javax.swing.JComboBox jComboBoxSoilKMap;
    javax.swing.JComboBox jComboBoxSoilKMap1;
    javax.swing.JComboBox jComboBoxSoilKMap2;
    javax.swing.JComboBox jComboBoxTempMapFiles;
    javax.swing.JComboBox jComboBoxWorldWind;
    javax.swing.JLabel jLabel1;
    javax.swing.JLabel jLabel10;
    javax.swing.JLabel jLabel100;
    javax.swing.JLabel jLabel101;
    javax.swing.JLabel jLabel102;
    javax.swing.JLabel jLabel103;
    javax.swing.JLabel jLabel104;
    javax.swing.JLabel jLabel105;
    javax.swing.JLabel jLabel106;
    javax.swing.JLabel jLabel107;
    javax.swing.JLabel jLabel108;
    javax.swing.JLabel jLabel109;
    javax.swing.JLabel jLabel11;
    javax.swing.JLabel jLabel110;
    javax.swing.JLabel jLabel111;
    javax.swing.JLabel jLabel112;
    javax.swing.JLabel jLabel113;
    javax.swing.JLabel jLabel114;
    javax.swing.JLabel jLabel115;
    javax.swing.JLabel jLabel116;
    javax.swing.JLabel jLabel117;
    javax.swing.JLabel jLabel118;
    javax.swing.JLabel jLabel119;
    javax.swing.JLabel jLabel12;
    javax.swing.JLabel jLabel120;
    javax.swing.JLabel jLabel121;
    javax.swing.JLabel jLabel122;
    javax.swing.JLabel jLabel123;
    javax.swing.JLabel jLabel124;
    javax.swing.JLabel jLabel125;
    javax.swing.JLabel jLabel126;
    javax.swing.JLabel jLabel127;
    javax.swing.JLabel jLabel128;
    javax.swing.JLabel jLabel129;
    javax.swing.JLabel jLabel13;
    javax.swing.JLabel jLabel130;
    javax.swing.JLabel jLabel131;
    javax.swing.JLabel jLabel132;
    javax.swing.JLabel jLabel133;
    javax.swing.JLabel jLabel134;
    javax.swing.JLabel jLabel135;
    javax.swing.JLabel jLabel136;
    javax.swing.JLabel jLabel137;
    javax.swing.JLabel jLabel138;
    javax.swing.JLabel jLabel139;
    javax.swing.JLabel jLabel14;
    javax.swing.JLabel jLabel140;
    javax.swing.JLabel jLabel141;
    javax.swing.JLabel jLabel142;
    javax.swing.JLabel jLabel143;
    javax.swing.JLabel jLabel144;
    javax.swing.JLabel jLabel145;
    javax.swing.JLabel jLabel146;
    javax.swing.JLabel jLabel147;
    javax.swing.JLabel jLabel148;
    javax.swing.JLabel jLabel149;
    javax.swing.JLabel jLabel15;
    javax.swing.JLabel jLabel150;
    javax.swing.JLabel jLabel151;
    javax.swing.JLabel jLabel152;
    javax.swing.JLabel jLabel153;
    javax.swing.JLabel jLabel154;
    javax.swing.JLabel jLabel155;
    javax.swing.JLabel jLabel156;
    javax.swing.JLabel jLabel157;
    javax.swing.JLabel jLabel158;
    javax.swing.JLabel jLabel159;
    javax.swing.JLabel jLabel16;
    javax.swing.JLabel jLabel160;
    javax.swing.JLabel jLabel161;
    javax.swing.JLabel jLabel162;
    javax.swing.JLabel jLabel163;
    javax.swing.JLabel jLabel164;
    javax.swing.JLabel jLabel165;
    javax.swing.JLabel jLabel166;
    javax.swing.JLabel jLabel167;
    javax.swing.JLabel jLabel168;
    javax.swing.JLabel jLabel169;
    javax.swing.JLabel jLabel17;
    javax.swing.JLabel jLabel170;
    javax.swing.JLabel jLabel171;
    javax.swing.JLabel jLabel172;
    javax.swing.JLabel jLabel173;
    javax.swing.JLabel jLabel174;
    javax.swing.JLabel jLabel175;
    javax.swing.JLabel jLabel176;
    javax.swing.JLabel jLabel177;
    javax.swing.JLabel jLabel178;
    javax.swing.JLabel jLabel179;
    javax.swing.JLabel jLabel18;
    javax.swing.JLabel jLabel180;
    javax.swing.JLabel jLabel181;
    javax.swing.JLabel jLabel182;
    javax.swing.JLabel jLabel183;
    javax.swing.JLabel jLabel184;
    javax.swing.JLabel jLabel185;
    javax.swing.JLabel jLabel186;
    javax.swing.JLabel jLabel187;
    javax.swing.JLabel jLabel188;
    javax.swing.JLabel jLabel189;
    javax.swing.JLabel jLabel19;
    javax.swing.JLabel jLabel190;
    javax.swing.JLabel jLabel191;
    javax.swing.JLabel jLabel192;
    javax.swing.JLabel jLabel193;
    javax.swing.JLabel jLabel194;
    javax.swing.JLabel jLabel195;
    javax.swing.JLabel jLabel196;
    javax.swing.JLabel jLabel197;
    javax.swing.JLabel jLabel198;
    javax.swing.JLabel jLabel2;
    javax.swing.JLabel jLabel20;
    javax.swing.JLabel jLabel21;
    javax.swing.JLabel jLabel22;
    javax.swing.JLabel jLabel23;
    javax.swing.JLabel jLabel24;
    javax.swing.JLabel jLabel25;
    javax.swing.JLabel jLabel26;
    javax.swing.JLabel jLabel27;
    javax.swing.JLabel jLabel28;
    javax.swing.JLabel jLabel29;
    javax.swing.JLabel jLabel3;
    javax.swing.JLabel jLabel30;
    javax.swing.JLabel jLabel31;
    javax.swing.JLabel jLabel32;
    javax.swing.JLabel jLabel33;
    javax.swing.JLabel jLabel34;
    javax.swing.JLabel jLabel35;
    javax.swing.JLabel jLabel36;
    javax.swing.JLabel jLabel37;
    javax.swing.JLabel jLabel38;
    javax.swing.JLabel jLabel39;
    javax.swing.JLabel jLabel4;
    javax.swing.JLabel jLabel40;
    javax.swing.JLabel jLabel41;
    javax.swing.JLabel jLabel42;
    javax.swing.JLabel jLabel43;
    javax.swing.JLabel jLabel44;
    javax.swing.JLabel jLabel45;
    javax.swing.JLabel jLabel46;
    javax.swing.JLabel jLabel47;
    javax.swing.JLabel jLabel48;
    javax.swing.JLabel jLabel49;
    javax.swing.JLabel jLabel5;
    javax.swing.JLabel jLabel50;
    javax.swing.JLabel jLabel51;
    javax.swing.JLabel jLabel52;
    javax.swing.JLabel jLabel53;
    javax.swing.JLabel jLabel54;
    javax.swing.JLabel jLabel55;
    javax.swing.JLabel jLabel56;
    javax.swing.JLabel jLabel57;
    javax.swing.JLabel jLabel58;
    javax.swing.JLabel jLabel59;
    javax.swing.JLabel jLabel6;
    javax.swing.JLabel jLabel60;
    javax.swing.JLabel jLabel61;
    javax.swing.JLabel jLabel62;
    javax.swing.JLabel jLabel63;
    javax.swing.JLabel jLabel64;
    javax.swing.JLabel jLabel65;
    javax.swing.JLabel jLabel66;
    javax.swing.JLabel jLabel67;
    javax.swing.JLabel jLabel68;
    javax.swing.JLabel jLabel69;
    javax.swing.JLabel jLabel7;
    javax.swing.JLabel jLabel70;
    javax.swing.JLabel jLabel71;
    javax.swing.JLabel jLabel72;
    javax.swing.JLabel jLabel73;
    javax.swing.JLabel jLabel74;
    javax.swing.JLabel jLabel75;
    javax.swing.JLabel jLabel76;
    javax.swing.JLabel jLabel77;
    javax.swing.JLabel jLabel78;
    javax.swing.JLabel jLabel79;
    javax.swing.JLabel jLabel8;
    javax.swing.JLabel jLabel80;
    javax.swing.JLabel jLabel81;
    javax.swing.JLabel jLabel82;
    javax.swing.JLabel jLabel83;
    javax.swing.JLabel jLabel84;
    javax.swing.JLabel jLabel85;
    javax.swing.JLabel jLabel86;
    javax.swing.JLabel jLabel87;
    javax.swing.JLabel jLabel88;
    javax.swing.JLabel jLabel89;
    javax.swing.JLabel jLabel9;
    javax.swing.JLabel jLabel90;
    javax.swing.JLabel jLabel91;
    javax.swing.JLabel jLabel92;
    javax.swing.JLabel jLabel93;
    javax.swing.JLabel jLabel94;
    javax.swing.JLabel jLabel95;
    javax.swing.JLabel jLabel96;
    javax.swing.JLabel jLabel97;
    javax.swing.JLabel jLabel98;
    javax.swing.JLabel jLabel99;
    javax.swing.JLabel jLabelFullMapName;
    javax.swing.JLabel jLabelHumanECoord;
    javax.swing.JLabel jLabelHumanNCoord;
    javax.swing.JLabel jLabelHumanSCoord;
    javax.swing.JLabel jLabelHumanWCoord;
    javax.swing.JLabel jLabelVersion;
    javax.swing.JPanel jPanel1;
    javax.swing.JPanel jPanelData;
    javax.swing.JPanel jPanelEnvironment;
    javax.swing.JPanel jPanelHouseholds;
    javax.swing.JPanel jPanelIM;
    javax.swing.JPanel jPanelLandEvol;
    javax.swing.JPanel jPanelMaps;
    javax.swing.JPanel jPanelMaps1;
    javax.swing.JPanel jPanelResources;
    javax.swing.JPanel jPanelResources1;
    javax.swing.JPanel jPanelVillages;
    javax.swing.JPanel jPanelWood;
    javax.swing.JRadioButton jRadioButtonFrictionMap;
    javax.swing.JRadioButton jRadioButtonFrictionMap1;
    javax.swing.JRadioButton jRadioButtonFrictionMap2;
    javax.swing.JRadioButton jRadioButtonFrictionValue;
    javax.swing.JRadioButton jRadioButtonFrictionValue1;
    javax.swing.JRadioButton jRadioButtonFrictionValue2;
    javax.swing.JRadioButton jRadioButtonSoilKMap;
    javax.swing.JRadioButton jRadioButtonSoilKMap1;
    javax.swing.JRadioButton jRadioButtonSoilKMap2;
    javax.swing.JRadioButton jRadioButtonSoilKValue;
    javax.swing.JRadioButton jRadioButtonSoilKValue1;
    javax.swing.JRadioButton jRadioButtonSoilKValue2;
    javax.swing.JScrollPane jScrollPane1;
    javax.swing.JScrollPane jScrollPane4;
    javax.swing.JSeparator jSeparator1;
    javax.swing.JSlider jSliderVillagePctInitClearing;
    javax.swing.JSlider jSliderYieldExpScalar;
    javax.swing.JTextField jSpinnerBirthInitProb;
    javax.swing.JTextField jSpinnerBirthProbDelta;
    javax.swing.JSpinner jSpinnerBirthProbMax;
    javax.swing.JSpinner jSpinnerBirthProbMin;
    javax.swing.JSpinner jSpinnerBirthProbPerPeople;
    javax.swing.JTextField jSpinnerDeathInitProb;
    javax.swing.JTextField jSpinnerDeathProbDelta;
    javax.swing.JSpinner jSpinnerDeathProbMax;
    javax.swing.JSpinner jSpinnerDeathProbMin;
    javax.swing.JSpinner jSpinnerDeathProbPerPeople;
    javax.swing.JSpinner jSpinnerPopulationPctLabor;
    javax.swing.JTabbedPane jTabbedPaneAgents;
    javax.swing.JTabbedPane jTabbedPaneEnv;
    javax.swing.JTabbedPane jTabbedPaneMain;
    javax.swing.JTable jTableVillageData;
    javax.swing.JTextField jTextAgpastKcal;
    javax.swing.JTextField jTextAgriProportion;
    javax.swing.JTextArea jTextArea1;
    javax.swing.JTextField jTextBarleyInitYield;
    javax.swing.JTextField jTextBarleyLaborRqd;
    javax.swing.JTextField jTextDateChangeValue;
    javax.swing.JTextField jTextDateEnd;
    javax.swing.JTextField jTextDatePrefix;
    javax.swing.JTextField jTextDateStart;
    javax.swing.JTextField jTextDateSuffix;
    javax.swing.JTextField jTextField5;
    javax.swing.JTextField jTextField6;
    javax.swing.JTextField jTextFieldAPSIM;
    javax.swing.JTextField jTextFieldBpointDA1;
    javax.swing.JTextField jTextFieldBpointDA2;
    javax.swing.JTextField jTextFieldBpointLC1;
    javax.swing.JTextField jTextFieldBpointLC2;
    javax.swing.JTextField jTextFieldClimFile;
    javax.swing.JTextField jTextFieldDepthWght;
    javax.swing.JTextField jTextFieldFarmDistWght;
    javax.swing.JTextField jTextFieldFertilityImpact;
    javax.swing.JTextField jTextFieldGRASS;
    javax.swing.JTextField jTextFieldGoatFodder;
    javax.swing.JTextField jTextFieldGoatYield;
    javax.swing.JTextField jTextFieldGrazingDistWight;
    javax.swing.JTextField jTextFieldInitGrazingReturn;
    javax.swing.JTextField jTextFieldLcovDistWight;
    javax.swing.JTextField jTextFieldMaxBarleyYield;
    javax.swing.JTextField jTextFieldMaxLcov;
    javax.swing.JTextField jTextFieldMaxWheatYield;
    javax.swing.JTextField jTextFieldPctBarleyFodder;
    javax.swing.JTextField jTextFieldPython;
    javax.swing.JTextField jTextFieldRC6;
    javax.swing.JTextField jTextFieldSeedPct;
    javax.swing.JTextField jTextFieldSheepFodder;
    javax.swing.JTextField jTextFieldSheepYield;
    javax.swing.JTextField jTextFieldSoilFtyWight;
    javax.swing.JTextField jTextFieldSoilMax;
    javax.swing.JTextField jTextFieldSoilMin;
    javax.swing.JTextField jTextFieldSoilRecovery;
    javax.swing.JTextField jTextFieldStormEvents;
    javax.swing.JTextField jTextFieldStormLength;
    javax.swing.JTextField jTextFieldStormPrecip;
    javax.swing.JTextField jTextFieldStormSpeed;
    javax.swing.JTextField jTextFieldWoodDistanceWeight;
    javax.swing.JTextField jTextFieldWoodIntensity;
    javax.swing.JTextField jTextFieldcutoff1;
    javax.swing.JTextField jTextFieldcutoff2;
    javax.swing.JTextField jTextFieldcutoff3;
    javax.swing.JTextField jTextFieldgoatRatio;
    javax.swing.JTextField jTextFieldgrazingDensity;
    javax.swing.JTextField jTextFieldkappa;
    javax.swing.JTextField jTextFieldloadExponent;
    javax.swing.JTextField jTextFieldnumOfOvicaprids;
    javax.swing.JTextField jTextFieldrFactor;
    javax.swing.JTextField jTextFieldrainValue;
    javax.swing.JTextField jTextFieldsheepRatio;
    javax.swing.JTextField jTextFieldsoilDensity;
    javax.swing.JTextField jTextFieldstreamTrans;
    javax.swing.JTextField jTextFrictionValue;
    javax.swing.JTextField jTextFrictionValue1;
    javax.swing.JTextField jTextFrictionValue2;
    javax.swing.JTextField jTextGISDBASE;
    javax.swing.JTextField jTextGISDBASE1;
    javax.swing.JTextField jTextGISDBASE2;
    javax.swing.JTextField jTextHHSizeLimit;
    javax.swing.JTextField jTextHumanEastCoords;
    javax.swing.JTextField jTextHumanNorthCoords;
    javax.swing.JTextField jTextHumanSouthCoords;
    javax.swing.JTextField jTextHumanWestCoords;
    javax.swing.JTextField jTextManDaysPerPerson;
    javax.swing.JTextField jTextMapset;
    javax.swing.JTextField jTextMapset1;
    javax.swing.JTextField jTextMapset2;
    javax.swing.JTextField jTextMaxFarmDistance;
    javax.swing.JTextField jTextMaxGrazeDistance;
    javax.swing.JTextField jTextNumVillages;
    javax.swing.JTextField jTextPastoralProportion;
    javax.swing.JTextField jTextSoilKValue;
    javax.swing.JTextField jTextSoilKValue1;
    javax.swing.JTextField jTextSoilKValue2;
    javax.swing.JTextField jTextVillageEWCoord;
    javax.swing.JTextField jTextVillageHhStartPopulation;
    javax.swing.JTextField jTextVillageNSCoord;
    javax.swing.JTextField jTextVillageName;
    javax.swing.JTextField jTextVillageNumHouseholds;
    javax.swing.JTextField jTextVillagePctInitClearing;
    javax.swing.JTextField jTextWheatCalories;
    javax.swing.JTextField jTextWheatInitYield;
    javax.swing.JTextField jTextWheatLaborRqd;
    javax.swing.JTextField jTextWoodPc;
    javax.swing.JTextField jTextYieldExpScalar;
    javax.swing.JTextField jTextYieldExpScalar1;
    javax.swing.JTextField jTextYieldExpScalar2;
    // End of variables declaration//GEN-END:variables
    
}
